#!/bin/bash
# ============================================================================
# 
# Installation and configuration of Blosxom
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHBLOSXOM" = "0" ]; then
	Info "You did not wish to install Blosxom !"
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting Blosxom4Nagios sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/blosxom4nagios-$BLOSXOMVERS.tar.gz >> "$OUTPUTFILE" 2>&1
ln -s blosxom4nagios-$BLOSXOMVERS "$TMPDIR/blosxom4nagios"

# -------------------------
# Setup the application:
# -------------------------
perl -MCPAN -e 'install Filter::Simple' 
perl -MCPAN -e 'install Logger::Syslog'

pushd tmp/blosxom4nagios >/dev/null
./bin/setup
cd ..
mv blosxom4nagios-$BLOSXOMVERS $DSTDIR/blosxom
chown -R $NAGIOSUSR:$NAAGIOSGRP $DSTDIR/blosxom/
popd >/dev/null

# Change /var/log/blosxom into $DSTDIR:
NEWVAL=`echo $DSTDIR | sed -e 's/\//\\\\\//g'`
for F in $DSTDIR/blosxom/config/*; do
	sed -i -e "s/\/var\/log\/nagios/$NEWVAL/g" $F
done

# Set the good rights:
chown $APACHEUSR $DSTDIR/blosxom/state

# Add a configuration file in Nagios:
CFGFILE=$DSTDIR/blosxom/config/commands_blosxom.cfg
sed -i -e "s/\/var\/log\/nagios/$NEWVAL/g" $CFGFILE
mv $CFGFILE $DSTDIR/etc/objects/
CFGFILE=$DSTDIR/etc/objects/commands_blosxom.cfg
chown $APACHEUSR:$NAGIOSGRP $CFGFILE
chmod 640 $CFGFILE

# Add a contact definition:
cat <<END >> $CFGFILE

define contact {
	contact_name                    blosxom
	use                             generic-contact
	alias                           blosxom
	service_notification_commands           notify-via-blosxom
	host_notification_commands              host-notify-via-blosxom
}
END

# Add this contact to the contactgroup:
sed -i -e 's/members.*nagiosadmin/&,blosxom/' $DSTDIR/etc/objects/contacts.cfg

# ---------------------------
# Configure the Web Interface:
# ---------------------------

# Add blosxom conf in nagios.conf:
cat <<END >nagios.conf.$$
# Subsection blosxom using CGI
AddHandler cgi-script .cgi
Alias /nagios/blosxom/themes $DSTDIR/blosxom/themes
Alias /nagios/blosxom/images $DSTDIR/blosxom/images
Alias /nagios/blosxom $DSTDIR/blosxom/cgi
<Directory $DSTDIR/blosxom/cgi>
	Options +Indexes
	DirectoryIndex blosxom.cgi
	RewriteEngine on
	RewriteCond %{REQUEST_FILENAME} !-f
	RewriteCond %{REQUEST_FILENAME} !-d
	RewriteCond %{REQUEST_URI} ^/nagios/blosxom
	RewriteRule ^(.*)$ /nagios/blosxom/blosxom.cgi/$1 [L,QSA]
	RewriteCond %{REQUEST_FILENAME} !-f
	RewriteCond %{REQUEST_FILENAME} !-d
	RewriteCond %{REQUEST_URI} ^/blosxom
	RewriteRule ^(.*)$ /blosxom/blosxom.cgi/$1 [L,QSA]
	<FilesMatch "\.cgi$">
		Options +ExecCGI
	</FilesMatch>
</Directory>
<Location /nagios/blosxom>
	SetEnv BLOSXOM_CONFIG_DIR $DSTDIR/blosxom/config
</Location>

END
cat $NAGIOSCONF >>nagios.conf.$$
cp nagios.conf.$$ $NAGIOSCONF

# Add an Administration option in nagios menu:
sed -i -e "163a\\
<tr><td width=13><img src=\"images/greendot.gif\" width=\"13\" height=\"14\" name=\"docs-dot\"></td><td nowrap><a href=\"/nagios/blosxom/\" target=\"main\" onMouseOver=\"switchdot('docs-dot',1)\" onMouseOut=\"switchdot('docs-dot',0)\" class=\"NavBarItem\">Blosxom4Nagios</a></td></tr>" \
$DSTDIR/share/side.html

exit 0

# EOF
