#!/bin/bash
# ============================================================================
# 
# Installation and configuration of EZNag
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting EZNag sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/eznag-$EZNAGVERS.tar.gz >>"$OUTPUTFILE" 2>&1
ln -s eznag-$EZNAGVERS "$TMPDIR/eznag"

# ------------------
# EZNag Installation:
# ------------------
echo "EZNag Installation..." | tee -a "$OUTPUTFILE"
EZNAGDIR="$DSTDIR/eznag"
if [ ! -d "$EZNAGDIR" ]; then
	mkdir -p "$EZNAGDIR"
	if [ $? -ne 0 ]; then
		Error "Could not create the '$EZNAGDIR' directory"
	fi
fi

if [ "$UPDATE" = "y" ]; then
	# Save current EZNag configuration:
	mv "$EZNAGDIR/php/mmi/config.inc.php" "$TMPDIR" 2>/dev/null
	mv "$EZNAGDIR/php/mmi/nmap.inc.php" "$TMPDIR" 2>/dev/null

	# Update EZNag:
	cp -rf $TMPDIR/eznag/ "$DSTDIR"
	cp "$EZNAGDIR/php/mmi/config.inc.php" "$EZNAGDIR/php/mmi/config.inc.php.new"
	cp "$EZNAGDIR/php/mmi/nmap.inc.php" "$EZNAGDIR/php/mmi/nmap.inc.php.new"

	# Restore previous configuration:
	mv "$TMPDIR/config.inc.php" "$EZNAGDIR/php/mmi/" 2>/dev/null
	mv "$TMPDIR/nmap.inc.php" "$EZNAGDIR/php/mmi/" 2>/dev/null

	# Update the version:
	sed -i -e "s/gVersion.*=.*/gVersion = \"$EZNAGVERS\";/" "$EZNAGDIR/php/mmi/config.inc.php"

	chown -R $APACHEUSR:$APACHEGRP "$EZNAGDIR"
	exit 0
fi

cp -r $TMPDIR/eznag/* "$EZNAGDIR"
chown -R $APACHEUSR:$APACHEGRP "$EZNAGDIR"

if [ ! -d "$EZNAGDIR/php/rrd" ]; then
	mkdir "$EZNAGDIR/php/rrd"
fi
chown $NAGIOSUSR:$APACHEGRP "$EZNAGDIR/php/rrd"

if [ -f "$EZNAGDIR/php/temp/config_db.bin" ]; then
	rm -f "$EZNAGDIR/php/temp/config_db.bin"
fi

# -------------------
# EZNag Configuration:
# -------------------
echo "EZNag configuration and integration within Nagios" | tee -a "$OUTPUTFILE"

EZNAGCONF="$EZNAGDIR/php/mmi/config.inc.php"

NEWVAL=`echo $DSTDIR/share | sed -e 's/\//\\\\\//g'`
sed -i -e "/gNagiosDocumentRoot/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

NEWVAL=`echo $DSTDIR/etc/nagios.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "/gMainConfigFileName/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

NEWVAL=`echo $DSTDIR/etc/cgi.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "/gCGIConfigFileName/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

sed -i -e "/gDebugMode/s/true/false/" $EZNAGCONF

NEWVAL=`echo sudo $DSTDIR/bin/nagios -v $DSTDIR/etc/nagios.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "/gTestCommand/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

# Added from v0.10
NEWVAL=`echo $DSTDIR/bin/nagiostats -c $DSTDIR/etc/nagios.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "/gStatCommand/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF
chgrp $APACHEGRP "$DSTDIR/bin/nagiostats"

NEWVAL=`echo $RRDPATH | sed -e 's/\//\\\\\//g'`
sed -i -e "/gRRDToolCommand/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

NEWVAL=`echo $EZNAGDIR/php/rrd | sed -e 's/\//\\\\\//g'`
sed -i -e "/gRRDGraphDir/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

# Added from v1.0
NEWVAL=`echo $HTPASSWD | sed -e 's/\//\\\\\//g'`
sed -i -e "/gApacheAuthFile/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

NMAPCONF="$EZNAGDIR/php/mmi/nmap.inc.php"

NEWVAL=`echo $NMAPPATH | sed -e 's/\//\\\\\//g'`
sed -i -e "/gNmapCommand/s/\".*\"/\"$NEWVAL\"/" $NMAPCONF

# Added from v1.0.1
if [ "$AUTHTYPE" = "digest" ]; then
	PWDCOMMAND="$EZNAGDIR/support/htdigest"
else
	PWDCOMMAND=`type htpasswd 2>/dev/null | cut -d " " -f3`
fi

NEWVAL=`echo $PWDCOMMAND | sed -e 's/\//\\\\\//g'`
sed -i -e "/^\$gApacheAuthCommand/s/\".*\"/\"$NEWVAL\"/" $EZNAGCONF

sed -i -e "/^\$gShowInternals/s/true/false/" $EZNAGCONF

if [ "$WITHNAGVIS" != "1" ]; then
	sed -i -e "/^\$gWithNagvis/s/true/false/" $EZNAGCONF
fi

PERFCONF="$EZNAGDIR/php/scripts/process-service-perfdata.php"

NEWVAL=`echo $EZNAGDIR/php/scripts | sed -e 's/\//\\\\\//g'`
sed -i -e "/^chdir/s/.*/chdir(\"$NEWVAL\");/" $PERFCONF

# Added from v1.0.3
PIDOF=`type pidof 2>/dev/null | cut -d" " -f3`
if [ "$PIDOF" != "" ]; then
	NEWVAL=`echo $PIDOF | sed -e 's/\//\\\\\//g'`
	sed -i -e "/gPidofCommand/s/\/sbin\/pidof/$NEWVAL/" $EZNAGCONF
fi

# Fix a bug in nagios 3.0.4 configuration:
sed -i -e "s/^time_change_threshold=/#time_change_threshold=/" $DSTDIR/etc/nagios.cfg

# Nagios configuration files (under ./etc) must be accessible
# in r/w mode for eznag:
chown -R $APACHEUSR:$NAGIOSGRP "$DSTDIR/etc"
chmod g+s "$DSTDIR/etc"
chmod -R 0640 $DSTDIR/etc/*
chmod 0750 "$DSTDIR/etc/objects" 2>/dev/null
chmod g+s "$DSTDIR/etc/objects" 2>/dev/null

# Add an Administration option in nagios menu:
sed -i -e "77a\\
<tr><td width=13><img src=\"images/greendot.gif\" width=\"13\" height=\"14\" name=\"docs-dot\"></td><td nowrap><a href=\"/eznag/\" target=\"main\" onMouseOver=\"switchdot('docs-dot',1)\" onMouseOut=\"switchdot('docs-dot',0)\" class=\"NavBarItem\">Administration</a></td></tr>" \
$DSTDIR/share/side.html

# Added from v0.10:
if [ "$WANTSFRENCHDOC" = "y" ]; then
	# TODO: install the doc !!!!
	echo "Installing the French Documentation" | tee -a "$OUTPUTFILE"
	mkdir "$DSTDIR/share/docs/FRENCH" 2>/dev/null
	tar -xvz -f distrib/docs/doc_nagios_fr_v2.tgz -C "$DSTDIR/share/docs/FRENCH" >/dev/null
	sed -i -e "77a\\
	<tr><td width=13><img src=\"images/greendot.gif\" width=\"13\" height=\"14\" name=\"docs-dot\"></td><td nowrap><a href=\"docs/FRENCH/fr_2.5/index.html\" target=\"main\" onMouseOver=\"switchdot('docs-dot',1)\" onMouseOut=\"switchdot('docs-dot',0)\" class=\"NavBarItem\">Documentation (fr)</a></td></tr>" \
	$DSTDIR/share/side.html
fi

# --------------------
# Build the logos page:
# --------------------
echo "Build the logos page" | tee -a "$OUTPUTFILE"
pushd $EZNAGDIR/scripts >>"$OUTPUTFILE" 2>&1
$PHPPATH -q makethumb.php $DSTDIR/share/images/logos
popd >>"$OUTPUTFILE" 2>&1

# -------------------------------------------------------
# Add commands in SUDOFILE to make eznag able to run nmap:
# -------------------------------------------------------
# Get the name of nagios script name (/etc/init.d.nagios ?)
# We must take care of configuration where many nagios filename
# may exist under /etc. In that case, we select the ones that are
# executable. In case of doubt, the user is asked:

echo "Trying to add commands in sudo file" | tee -a "$OUTPUTFILE"

# Before v0.5:
#NAGIOSINIT=`find /etc -name nagios`

# From v0.5:
OLDFS=$IFS
IFS=$'\n'
# From v0.6: turn "-perm +u+x" into "-perm -u+x" to handle perm such as -rwxrwx--
NAGIOSINITLIST=(`find /etc/ -name nagios -perm -u+x`)
IFS=$OLDFS

if [ ${#NAGIOSINITLIST[@]} -eq 0 ]; then
	Info "Could not find location of nagios startup script under /etc"
	Info "You have to modify $SUDOFILE by hand"
elif [ ${#NAGIOSINITLIST[@]} -gt 1 ]; then
	Info "We found more than one 'nagios' script under /etc"
	Info "You have to modify $SUDOFILE by hand"
else
	NAGIOSINIT=${NAGIOSINITLIST[0]}
	SUDOCMD1="$APACHEUSR ALL = NOPASSWD: $NMAPPATH"
	SUDOCMD2="$APACHEUSR ALL = NOPASSWD: $NAGIOSINIT start"
	SUDOCMD3="$APACHEUSR ALL = NOPASSWD: $NAGIOSINIT stop"
	SUDOCMD4="$APACHEUSR ALL = NOPASSWD: $NAGIOSINIT restart"
	SUDOCMD5="$APACHEUSR ALL = NOPASSWD: $DSTDIR/bin/nagios -v $DSTDIR/etc/nagios.cfg"
	SUDOCMD6="$APACHEUSR ALL = NOPASSWD: $PWDCOMMAND *"

	if [ -w "$SUDOFILE" ]; then
		# Added from v0.5:
		# Suppress old commands if they already exist:
		sed -i -e "/# EZNag Begin/,/# EZNag End/d" $SUDOFILE

		echo "# EZNag Begin" >> $SUDOFILE
		echo "$SUDOCMD1" >> $SUDOFILE
		echo "$SUDOCMD2" >> $SUDOFILE
		echo "$SUDOCMD3" >> $SUDOFILE
		echo "$SUDOCMD4" >> $SUDOFILE
		echo "$SUDOCMD5" >> $SUDOFILE
		echo "$SUDOCMD6" >> $SUDOFILE
		echo "# EZNag End" >> $SUDOFILE

		# From v1.0.3: try to add the !requiretty directive for Apache
		if grep -E -i "^Defaults[[:space:]]+requiretty" $SUDOFILE >/dev/null 2>&1; then
			# Do not add the same directive twice:
			if ! grep -E -i "^Defaults:$APACHEUSR[[:space:]]\!requiretty" $SUDOFILE >/dev/null 2>&1;then
				sed -i -e "/^Defaults[\t ]*requiretty/a\\
Defaults:$APACHEUSR \!requiretty" $SUDOFILE
			fi
		fi
	else
		echo "Could not add sudo commands in file '$SUDOFILE'"
	fi
fi

# ------------------------------------------------------------------------------
# Give the possibility to install Nagios Checker Plugin for Firefox (from v0.10):
# ------------------------------------------------------------------------------
Info "You can install the 'Nagios Checker' Plugin for Firefox"
Info "it is a very useful tool ! Just launch Firefox and open the following file :"
Info $(pwd)/distrib/browser/nagioschecker-0.13.9.1.xpi
Info "If you're using a Windows Vista, you can install nagstatus.gadget on your PC"
Info $(pwd)/distrib/browser/nagstatus.gadget

exit 0

# EOF
