#!/bin/bash
# ============================================================================
# 
# Nagios Installation
#
# ----------------------------------------------------------------------------
#
# TODO: ask for the list of client @IP that are allowed to access to Nagios
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

# -----------------------------------------------
# Extract the archives in the temporary directory:
# -----------------------------------------------
if [ ! -d "$TMPDIR" ]; then
	mkdir "$TMPDIR"
	if [ $? -ne 0 ]; then
		echo "Could not create '$TMPDIR' temporary directory !"
		TMPDIR=
	fi

	# TODO: do the test automatically !
	echo "Please check the directory '$DSTDIR' can be accessed in r/x mode for $APACHEUSR user"
fi

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# -------------------------------------------------
echo "Extracting nagios sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nagios-$NAGIOSVERS.tar.gz >>"$OUTPUTFILE" 2>&1
echo "Extracting nagios plugins sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nagios-plugins-$PLUGINSVERS.tar.gz >>"$OUTPUTFILE" 2>&1
ln -s nagios-$NAGIOSVERS "$TMPDIR/nagios"
ln -s nagios-plugins-$PLUGINSVERS "$TMPDIR/nagios-plugins"

# --------------------------------------
# Create the nagios account if necessary:
# --------------------------------------
if [ "$CREATENAGIOSUSER" = "1" ]; then
	echo "Creation of $NAGIOSUSR account" | tee -a "$OUTPUTFILE"

	# Note: 'groupadd' does not always support the '-r' option (create a system account)
	groupadd -r $NAGIOSGRP >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		groupadd $NAGIOSGRP >/dev/null 2>&1
	fi
	if [ $? -ne 0 ]; then
		Error "Error while creating the '$NAGIOSGRP' group"
	fi

	# home dir added with v0.7.2
	useradd -d "$DSTDIR" -g $NAGIOSGRP -s /bin/bash -G $APACHEGRP $NAGIOSUSR
	if [ $? -ne 0 ]; then
		Error "Error while creating the '$NAGIOSUSR' account"
	fi

        # fixed in v1.0b:
        # if the home dir is created, make it readable by apache
        chmod go+rx "$DSTDIR"
fi

# ---------------------------------------------
# Creation of the target directory if necessary:
# ---------------------------------------------
if [ ! -d "$DSTDIR" ]; then
	echo "Creation of $DSTDIR directory" | tee -a "$OUTPUTFILE"
	mkdir "$DSTDIR"
	if [ $? -ne 0 ]; then
		Error "Could not create the '$DSTDIR' directory !"
	fi
fi

# Added since v0.9
# Create the SSH key pair for check_by_ssh:
if [ ! -d "$DSTDIR/.ssh" ]; then
	mkdir "$DSTDIR/.ssh"
fi
ssh-keygen -t dsa -N "" -f "$DSTDIR/.ssh/id_dsa"
sed -i -e 's/root@/nagios@/' "$DSTDIR/.ssh/id_dsa.pub"
chown -R $NAGIOSUSR:$NAGIOSGRP "$DSTDIR/.ssh"

# Added since v1.0.1
# make sure the directory is readable by Apache
chmod go+rx "$DSTDIR"

# -----------------
# libgd Compilation: (needed for statusmap.cgi)
# -----------------
WITH_GD_INC=
WITH_GD_LIB=

# This is unuseful: we ask the user to install this stuff by hand because
# gd has too many dependencies...
if [ "$WITHLIBGD" = "1" ]; then
	# libpng:
	echo "libpng Compilation..." | tee -a "$OUTPUTFILE"
	tar -xvz -C "$TMPDIR" -f distrib/libpng-$LIBPNGVERS-config.tar.gz >>"$OUTPUTFILE" 2>&1
	ln -s libpng-$LIBPNGVERS-config "$TMPDIR/png"

	pushd $TMPDIR/png >> "$OUTPUTFILE" 2>&1

	# added from v0.7.2: it seems ./configure failed on SuSE
	# since it cannot create a tmp subdir at this step !?
	mkdir tmp

	env CFLAGS="-O3 -fPIC" \
		./configure --disable-shared >>"$OUTPUTFILE" 2>&1
	make >>"$OUTPUTFILE" 2>&1
	make install >>"$OUTPUTFILE" 2>&1

	popd >>"$OUTPUTFILE" 2>&1


	echo "libgd installation" | tee -a "$OUTPUTFILE"

	tar -xvz -C "$TMPDIR" -f distrib/gd-$LIBGDVERS.tar.gz >>"$OUTPUTFILE" 2>&1
	ln -s gd-$LIBGDVERS "$TMPDIR/gd"

	pushd $TMPDIR/gd >>"$OUTPUTFILE" 2>&1

	# added from v0.7.2: it seems ./configure failed on SuSE
	# since it cannot create a tmp subdir at this step !?
	mkdir tmp

	./configure >>"$OUTPUTFILE" 2>&1
	make >>"$OUTPUTFILE" 2>&1
	make install >>"$OUTPUTFILE" 2>&1
	popd >>"$OUTPUTFILE" 2>&1

	WITH_GD_INC=--with-gd-inc=/usr/local/include
	WITH_GD_LIB=--with-gd-lib=/usr/local/lib

	ldconfig
fi

# ------------------
# nagios compilation:
# ------------------
pushd "$TMPDIR/nagios" >/dev/null

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

echo "nagios Configuration..." | tee -a "$OUTPUTFILE"
./configure 	--prefix=$DSTDIR \
		--with-nagios-user=$NAGIOSUSR \
		--with-nagios-grp=$NAGIOSGRP \
		--with-cgiurl=$CGIURL \
		--with-htmlurl=$HTMLURL \
		$WITH_GD_INC $WITH_GD_LIB \
		$NOSTATUSMAP \
	>>"$OUTPUTFILE" 2>&1

# Check the compilation result:
if [ $? -ne 0 ]; then
	Error "It appears the configuration failed..."
fi

echo "nagios Compilation..." | tee -a "$OUTPUTFILE"
make all >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "The Compilation failed"
fi

# Added with v0.9: check if statusmap.cgi has been successfully compiled:
if [ ! -f cgi/statusmap.cgi ]; then
	Info "Remember statusmap.cgi has not been compiled"
fi

echo "nagios Installation..." | tee -a "$OUTPUTFILE"
make install >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Installation failed (make install)"
fi

# Added with v0.6
echo "make install-init" | tee -a "$OUTPUTFILE"
make install-init >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Installation failed (make install-init)"
fi

# Added with v0.6
echo "make install-commandmode" | tee -a "$OUTPUTFILE"
make install-commandmode >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Installation failed (make install-commandmode)"
fi

echo "make install-config" | tee -a "$OUTPUTFILE"
make install-config >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "The Configuration File Installation fichiers failed (make install-config)"
fi

# Added with 1.0.5
echo "install contrib eventhandlers" | tee -a "$OUTPUTFILE"
cp -r contrib/eventhandlers/ $DSTDIR/libexec

# Do some fixes:
NEWVAL=`echo $DSTDIR | sed -e 's/\//\\\\\//g'`
find $DSTDIR/libexec/eventhandlers -type f -exec sed -i -e "s/\/usr\/local\/nagios/$NEWVAL/g" {} \;

sed -i -e "s/^NscaBin=.*/$NEWVAL\/nsca\/send_nsca/" \
       -e "s/^NscaCfg=.*/$NEWVAL\/nsca\/nsca.cfg/" \
       $DSTDIR/libexec/eventhandlers/distributed-monitoring/submit_check_result_via_nsca

# Change the group for HTML files:
chgrp -R $APACHEGRP "$DSTDIR/share"

# Rename or recreate .cfg-sample files into .cfg:
CURDIR=`pwd`
pushd "$DSTDIR/etc" >/dev/null 2>&1
for F in *sample; do
	NEWF=`basename $F .cfg-sample`
	if [ "$USESAMPLEFILES" = "1" -o ! -f "$CURDIR/distrib/conf/${NEWF}.cfg" ]; then
		mv $F ${NEWF}.cfg
	else 
		# Create empty files:
		touch ${NEWF}.cfg
		rm -f $F
	fi
done 2>>"$OUTPUTFILE"
popd >/dev/null

# ------------------
# MRTG Integration:
# ------------------
if [ "$MRTGINTEGRATION" = "1" -a "$MRTGCONF" != "" ]; then
	echo "MRTG integration" | tee -a "$OUTPUTFILE"

	# Try to not update the mrtg.cfg file twice:
	grep "nagios-a.*nagiostats" $MRTGCONF >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		Info "$MRTGCONF not updated - it seems it already contains nagiostats definition"
	else
		cat sample-config/mrtg.cfg >> $MRTGCONF
	fi

	mkdir $DSTDIR/share/mrtg 2>&1
	cp $MRTGCONF $MRTGCONF.save-eznag
	NEWVAL=`echo $DSTDIR/share/mrtg | sed -e 's/\//\\\\\//g'`
	sed -i -e "s/^HtmlDir:.*/HtmlDir: $NEWVAL/" \
	       -e "s/^ImageDir:.*/ImageDir: $NEWVAL/" \
	       $MRTGCONF
	indexmaker --output $DSTDIR/share/mrtg/index.html $MRTGCONF

	# Add an option in the left menu:
	sed -i -e "/Performance Info/a\\
		<tr><td width=13><img src=\"images/greendot.gif\" width=\"13\" height=\"14\" name=\"docs-dot\"></td><td nowrap><a href=\"/nagios/mrtg/index.html\" target=\"main\" onMouseOver=\"switchdot('docs-dot',1)\" onMouseOut=\"switchdot('docs-dot',0)\" class=\"NavBarItem\">nagiostats in MRTG</a></td></tr>" \
	$DSTDIR/share/side.html
fi

popd >/dev/null

# Copy best configuration files:
# not since v0.11
#echo "Copy configuration files..." | tee -a "$OUTPUTFILE"
#cp -f distrib/conf/*.cfg "$DSTDIR/etc" >/dev/null 2>&1

# Changed from v1.0.2:
#chgrp -R $APACHEGRP "$DSTDIR/etc"
# also made in eznag.sh
chown -R $APACHEUSR:$NAGIOSGRP "$DSTDIR/etc"
chmod g+s "$DSTDIR/etc" "$DSTDIR/etc/objects" 2>/dev/null
chmod -R 0640 $DSTDIR/etc/*
chmod 0750 "$DSTDIR/etc/objects" 2>/dev/null

# ...and update nagios.cfg according to the new files:
CONFIGFILE="$DSTDIR/etc/nagios.cfg"
NEWVAL=`echo $DSTDIR/etc/nagios.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "s/^\(cfg_file=.*minimal.cfg\)/#&/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*contactgroups.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*contacts.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*dependencies.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*escalations.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*hostgroups.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*hosts.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*services.cfg\)/\1/" $CONFIGFILE
#sed -i -e "s/^#\(cfg_file=.*timeperiods.cfg\)/\1/" $CONFIGFILE
sed -i -e "s/^\(cfg_file=.*\)/#\1/" $CONFIGFILE
sed -i -e "/^#cfg_dir.*routers/a\
cfg_dir=$DSTDIR/etc/objects" $CONFIGFILE

# From v1.0.3: change /bin/mail into /usr/bin/mail if needed in commands.cfg
if [ ! -x /bin/mail ]; then
	BINMAIL=`type mail 2>/dev/null | cut -d" " -f3`
	if [ "$BINMAIL" != "" ]; then
		NEWVAL=`echo $BINMAIL | sed -e 's/\//\\\\\//g'`
		sed -i -e "s/\/bin\/mail /$NEWVAL /g" "$DSTDIR/etc/objects/commands.cfg"
	fi
fi

# From v1.0.3: create a "check_telnet" command for nmap use
# From v1.0.4: fix bug when creating check_telnet
cat <<END >>"$DSTDIR/etc/objects/commands.cfg"

# Created by EZNag installer:
define command {
        command_name    check_telnet
        command_line    \$USER1\$/check_tcp -H \$HOSTADDRESS\$ -p 23
}
END

if [ "$WITHRRDTOOL" = "1" ]; then
	# Change nagios configuration to make it process service perfdata:
	sed -i -e "s/^process_performance_data=0/process_performance_data=1/" $CONFIGFILE
	# From v1.0.4: set eznag-process-service-perfdata command:
	sed -i -e "s/^#\(service_perfdata_command=.*\)/service_perfdata_command=eznag-process-service-perfdata/" $CONFIGFILE
	sed -i -e "s/^#\(service_perfdata_file=.*\)/\1/" $CONFIGFILE
	#sed -i -e "s/^#\(service_perfdata_file_template=.*\)/\1/" $CONFIGFILE
	sed -i -e "s/^#\(service_perfdata_file_mode=.*\)/\1/" $CONFIGFILE

	# Update the associated check_command:
	cp distrib/conf/perfdata.cfg ./tmp/
	COMMANDFILE=./tmp/perfdata.cfg
	NEWVAL=`echo $DSTDIR/eznag | sed -e 's/\//\\\\\//g'`
	sed -i -e "s/\/home\/eznag/$NEWVAL/" $COMMANDFILE
	NEWVAL=`echo $PHPPATH | sed -e 's/\//\\\\\//g'`
	sed -i -e "s/\/usr\/bin\/php/$NEWVAL/" $COMMANDFILE
	cat $COMMANDFILE >> "$DSTDIR/etc/objects/commands.cfg"
fi

# -------------------
# Plugins Compilation:
# -------------------
# Since nagios-plugins-1.4.5, --with-nagios-user and 
# --with-nagios-grp option are no longer supported
		#--with-nagios-user=$NAGIOSUSR \
		#--with-nagios-grp=$NAGIOSGRP \

pushd "$TMPDIR/nagios-plugins" >/dev/null

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

echo "nagios Plugins Configuration..." | tee -a "$OUTPUTFILE"
./configure 	--prefix=$DSTDIR \
		--with-cgiurl=$CGIURL \
	>>"$OUTPUTFILE" 2>&1

if [ $? -ne 0 ]; then
	Error "Plugins Configuration failed"
fi

# Fix bug in v1.4.6:
sed -i -e 's/^MKINSTALLDIRS.*/MKINSTALLDIRS = \.\.\/mkinstalldirs/' po/Makefile

echo "nagios Plugins Compilation..." | tee -a "$OUTPUTFILE"
make >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Plugins Compilation failed"
fi

echo "nagios Plugins Installation..." | tee -a "$OUTPUTFILE"
make install >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Plugins Installation failed (make install)"
fi

# Added with v0.6:
echo "make install-root" | tee -a "$OUTPUTFILE"
make install-root >>"$OUTPUTFILE" 2>&1
if [ $? -ne 0 ]; then
	Error "nagios Plugins Installation failed (make install-root)"
fi

# Added with v1.0.1: add contrib directory
echo "copy contrib/ libexec" | tee -a "$OUTPUTFILE"
cp -p contrib/*.pl contrib/*.pm "$DSTDIR/libexec"
if [ $? -ne 0 ]; then
	Info "nagios Contrib Plugins Installation failed"
fi
chown root:root $DSTDIR/libexec/*.pl $DSTDIR/libexec/*.pm
chmod +x $DSTDIR/libexec/*.pl $DSTDIR/libexec/*.pm

# ...and make sure perl path is correct:
PERLPATH=`type perl 2>/dev/null | cut -d" " -f3`
if [ "$PERLPATH" = "" ]; then
        Info "Could not determine Perl location !"
        Info "You will have to fix $DSTDIR/libexec/*.pl scripts yourself"
else 
        NEWVAL=`echo $PERLPATH | sed -e 's/\//\\\\\//g'`
        for F in $DSTDIR/libexec/*.pl $DSTDIR/libexec/check_oracle_tbs; do
		if [ -f "$F" ]; then
			sed -i -e "s/^#!.*bin\/perl\(.*\)/#!$NEWVAL\1/" $F
		fi
        done
fi      

# also make sure utils.pm path is correct:
NEWVAL=`echo $DSTDIR/libexec/utils.pm | sed -e 's/\//\\\\\//g'`
sed -i -e "s/^require '\/usr\/libexec\/nagios\/plugins\/utils.pm';/require '$NEWVAL';/" $DSTDIR/libexec/check_rrd_data.pl

NEWVAL=`echo $DSTDIR/libexec | sed -e 's/\//\\\\\//g'`
for F in $DSTDIR/libexec/*.pl; do
        sed -i -e "s/^use lib qw(.*).*;/use lib qw($NEWVAL);/" $F
done

popd >/dev/null

# --------------------------------
# Extract and copy the logo images:
# --------------------------------
echo "extracting and copying logos images" | tee -a "$OUTPUTFILE"
mkdir "$TMPDIR/images"
cp distrib/images/*.gz "$TMPDIR/images"

pushd "$TMPDIR/images" >/dev/null 2>&1
for F in *.gz; do
	tar xvfz $F >>"$OUTPUTFILE" 2>&1
done
popd >/dev/null

find "$TMPDIR/images" -name \*.png -exec mv {} "$DSTDIR/share/images/logos" \; >>"$OUTPUTFILE" 2>&1
#find $TMPDIR/images -name \*.gd2 -exec ./mvif.sh {} \;
chown -R $NAGIOSUSR:$APACHEGRP "$DSTDIR/share/images/logos"

# --------------------
# Apache Configuration:
# --------------------
echo "Update the Apache Configuration..." | tee -a "$OUTPUTFILE"
AUTHNAME=Nagios

# Create the Apache Configuration:
cat <<END > $NAGIOSCONF
# Nagios Configuration:
ScriptAlias $CGIURL $DSTDIR/sbin
Alias /eznag/ $DSTDIR/eznag/php/mmi/
Alias $HTMLURL $DSTDIR/share

BrowserMatch "MSIE" AuthDigestEnableQueryStringHack=On

<Directory $DSTDIR/sbin>
	AllowOverride AuthConfig
	Options ExecCGI
	Order deny,allow
	Allow from all
	AuthType $AUTHTYPE
	AuthName "$AUTHNAME"
	AuthUserFile "$HTPASSWD"
	<Limit GET POST>
		Require valid-user
	</Limit>
</Directory>

<Directory $DSTDIR/share>
	AllowOverride AuthConfig
	Options Indexes FollowSymLinks
	Order deny,allow
	Allow from all
	AuthType $AUTHTYPE
	AuthName "$AUTHNAME"
	AuthUserFile "$HTPASSWD"
	<Limit GET POST>
		Require valid-user
	</Limit>
</Directory>

<Directory $DSTDIR/eznag>
	AllowOverride AuthConfig
	Options Indexes FollowSymLinks
	Order deny,allow
	Allow from all
	AuthType $AUTHTYPE
	AuthName "$AUTHNAME"
	AuthUserFile "$HTPASSWD"
	<Limit GET POST>
		Require valid-user
	</Limit>
</Directory>
END

# Password File creation:
echo "Create the '$HTPASSWD' file:" | tee -a "$OUTPUTFILE"

# Modified with the v0.7.2: SuSE uses htpasswd2 !
if [ "$AUTHTYPE" = "basic" ]; then
	for H in htpasswd htpasswd2; do
		type $H >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			$H -b -c $HTPASSWD $WEBUSER $WEBPASSWD
		fi
	done
else
	# Modified with v0.8: offers digest mode too !
	for H in htdigest htdigest2; do
		type $H >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			echo "Please, enter '$WEBPASSWD' twice as the Password (do not type the single quotes !)"
			$H -c $HTPASSWD $AUTHNAME $WEBUSER
		fi
	done
fi

chown $NAGIOSUSR:$APACHEGRP $HTPASSWD
Info "The account name is '$WEBUSER' and the password is '$WEBPASSWD'"

# ----------------------------------------------------------------------------

Info "Nagios has been compiled and installed."
Info "Run the 'configure.sh' script to configure it."

exit 0

# EOF
