#!/bin/bash
# ============================================================================
# 
# Installation and configuration of N2RRD
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHN2RRD" = "0" ]; then
	Info "You did not wish to install n2rrd !"
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting n2rrd sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/n2rrd-$N2RRDVERS.tar.gz >> "$OUTPUTFILE" 2>&1
ln -s n2rrd-$N2RRDVERS "$TMPDIR/n2rrd"

# ---------------------------
# Configure 
# ---------------------------
echo "Installation..."
pushd tmp/n2rrd >/dev/null 

mkdir -p $DSTDIR/n2rrd/{bin,etc,rra} >/dev/null 2>&1
chmod 755 $DSTDIR/n2rrd/rra

NEWVAL1=`echo $DSTDIR/n2rrd/bin | sed -e 's/\//\\\\\//g'`
NEWVAL2=`echo $DSTDIR/n2rrd/etc | sed -e 's/\//\\\\\//g'`
sed -i -e "s/^BIN_DIR=.*/BIN_DIR=$NEWVAL1/" \
       -e "s/^CONF_DIR=.*/CONF_DIR=$NEWVAL2/" \
	install.sh
./install.sh >> "$OUTPUTFILE" 2>&1

mv $DSTDIR/n2rrd/etc/dist-n2rrd.conf $DSTDIR/n2rrd/etc/n2rrd.conf
for F in $DSTDIR/n2rrd/etc/templates/*/dist-*; do
	N=`echo $F | sed 's/dist-//'`
	mv $F $N
	chmod 644 $N
done
chown -R $NAGIOSUSR:$NAGIOSGRP $DSTDIR/n2rrd

# Change Nagios configuration:
CONFIGFILE="$DSTDIR/etc/nagios.cfg"
sed -i -e 's/^host_perfdata_command=.*/host_perfdata_command=n2rrd-host-perfdata/' \
       -e 's/^#host_perfdata_command=.*/host_perfdata_command=n2rrd-host-perfdata/' \
       -e 's/^service_perfdata_command=.*/service_perfdata_command=n2rrd-service-perfdata/' \
       $CONFIGFILE

# Create the new commands:
cat <<END > $DSTDIR/etc/objects/n2rrd.cfg
# notes_url for host that have perfdata collected can be:
# http://server/nagios/cgi-bin/rrd2graph.cgi?hostname=\$HOSTNAME\$&service=check_icmp
define command {
	command_name    n2rrd-host-perfdata
	command_line    $DSTDIR/n2rrd/bin/n2rrd.pl -d -c $DSTDIR/n2rrd/etc/n2rrd.conf -T \$LASTHOSTCHECK\$ -H \$HOSTNAME\$ -s "check_icmp" -o "\$HOSTOUTPUT\$"         
}

# notes_url for service that have perfdata collected can be:
# http://server/nagios/cgi-bin/rrd2graph.cgi?hostname=\$HOSTNAME\$&service=\$SERVICEDESC\$
define command {
	command_name    n2rrd-service-perfdata
	command_line    $DSTDIR/n2rrd/bin/n2rrd.pl -d -c $DSTDIR/n2rrd/etc/n2rrd.conf -T \$LASTSERVICECHECK\$ -H \$HOSTNAME\$ -s "\$SERVICEDESC\$" -o "\$SERVICEPERFDATA\$"
}
END
chown $APACHEUSR:$NAGIOSGRP $DSTDIR/etc/objects/n2rrd.cfg
chmod 640 $DSTDIR/etc/objects/n2rrd.cfg

# ---------------------------------
# Configure the cgi interface:
# ---------------------------------
perl -MCPAN -e 'install RRD'
cp rrd2graph.cgi $DSTDIR/sbin
chmod 755 $DSTDIR/sbin/rrd2graph.cgi
chown $NAGIOSUSR:$NAGIOSGRP $DSTDIR/sbin/rrd2graph.cgi

NEWVAL=`echo $DSTDIR/n2rrd/etc/n2rrd.conf | sed -e 's/\//\\\\\//g'`
sed -i -e "s/^\$conf_file.*=.*/\$conf_file = \"$NEWVAL\";/" \
       -e 's/^$debug = 1/$debug = 0/' \
       $DSTDIR/sbin/rrd2graph.cgi

# Patch rrd2graph.cgi:
sed -i -e 's/img src=\\"\/\$config{CACHE_DIR}/img src=\\"\/nagios\/\$config{CACHE_DIR}/' \
       $DSTDIR/sbin/rrd2graph.cgi


mkdir $DSTDIR/share/n2rrd
chown $APACHEUSR:$APACHGRP $DSTDIR/share/n2rrd

# -----------------------------
# Change the main conf file:
# -----------------------------
CONF_DIR=`echo $DSTDIR/n2rrd/etc | sed -e 's/\//\\\\\//g'`
RRA_DIR=`echo $DSTDIR/n2rrd/rra | sed -e 's/\//\\\\\//g'`
LOGFILE=`echo /var/log/n2rrd.log | sed -e 's/\//\\\\\//g'`
DOCUMENT_ROOT=`echo $DSTDIR/share | sed -e 's/\//\\\\\//g'`
RRDTOOL=`echo $RRDPATH | sed -e 's/\//\\\\\//g'`
sed -i -e "s/^CONF_DIR.*=.*/CONF_DIR = $CONF_DIR/" \
       -e "s/^RRA_DIR.*=.*/RRA_DIR = $RRA_DIR/" \
       -e "s/^LOGFILE.*=.*/LOGFILE = $LOGFILE/" \
       -e "s/^DOCUMENT_ROOT.*=.*/DOCUMENT_ROOT = $DOCUMENT_ROOT/" \
       -e "s/^CACHE_DIR.*=.*/CACHE_DIR = n2rrd/" \
       -e "s/^RRDTOOL.*=.*/RRDTOOL = $RRDTOOL/" \
       -e "s/^CGIBIN.*=.*/CGIBIN = nagios\/cgi-bin/" \
       $DSTDIR/n2rrd/etc/n2rrd.conf

touch /var/log/n2rrd.log
chown $NAGIOSUSR:$APACHEGRP /var/log/n2rrd.log
chmod 664 /var/log/n2rrd.log

popd >/dev/null

exit 0

# EOF
