#!/bin/bash
# ============================================================================
# 
# Installation and configuration of NagViz
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHNAGVIS" = "0" ]; then
	Info "You did not wish to install NagViz !"
	exit 0
fi

# -------------------
# Start with graphviz:
# -------------------
if [ "$WITHGRAPHVIZ" != "0" ]; then
	echo "Extracting graphviz sources" | tee -a "$OUTPUTFILE"
	tar xvz -C "$TMPDIR" -f distrib/nagvis/graphviz-$GRAPHVIZVERS.tar.gz >> "$OUTPUTFILE" 2>&1
	ln -s graphviz-$GRAPHVIZVERS "$TMPDIR/graphviz"

	echo "Configuration..."
	pushd $TMPDIR/graphviz >>"$OUTPUTFILE" 2>&1
	./configure >>"$OUTPUTFILE" 2>&1
	echo "Compilation..."
	make >>"$OUTPUTFILE" 2>&1
	echo "Installation..."
	make install >>"$OUTPUTFILE" 2>&1
	popd >>"$OUTPUTFILE" 2>&1
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting NagViz sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nagvis/nagvis-$NAGVISVERS.tar.gz >> "$OUTPUTFILE" 2>&1
#ln -s nagvis-$NAGVISVERS "$TMPDIR/nagvis"

# ------------
# Installation:
# ------------
echo "Installation..."
mv "$TMPDIR/nagvis-$NAGVISVERS/" "$DSTDIR/share/nagvis/"

sed -i -e "163a\\
<tr><td width=13><img src=\"images/greendot.gif\" width=\"13\" height=\"14\" name=\"docs-dot\"></td><td nowrap><a href=\"/nagios/nagvis/\" target=\"main\" onMouseOver=\"switchdot('docs-dot',1)\" onMouseOut=\"switchdot('docs-dot',0)\" class=\"NavBarItem\">NagVis</a></td></tr>" \
$DSTDIR/share/side.html

# -------------------
# Configure Nagvis:
# ----------------
echo "Configuration..."
NAGVISCONF="$DSTDIR/share/nagvis/etc/nagvis.ini.php"
mv "$DSTDIR/share/nagvis/etc/nagvis.ini.php-sample" "$NAGVISCONF"

OLDVAL=`echo /usr/local/nagios | sed -e 's/\//\\\\\//g'`
NEWVAL=`echo $DSTDIR | sed -e 's/\//\\\\\//g'`
sed -i -e "s/$OLDVAL/$NEWVAL/g" "$NAGVISCONF"

sed -i -e "s/^;dbhost=.*/dbhost=\"$DBSRV\"/" "$NAGVISCONF"
sed -i -e "s/^;dbname=.*/dbname=\"$DBNAME\"/" "$NAGVISCONF"
sed -i -e "s/^;dbuser=.*/dbuser=\"$DBUSER\"/" "$NAGVISCONF"
sed -i -e "s/^;dbpass=.*/dbpass=\"$DBPWD\"/" "$NAGVISCONF"

sed -i -e "s/exec('which '/exec('type '/" "$DSTDIR/share/nagvis/nagvis/includes/classes/NagVisAutoMap.php"

# Add pack of icons:
echo "Add Icon Packs" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nagvis/NagVis-Iconset-Lemon-02.tar.gz >> "$OUTPUTFILE" 2>&1
cp $TMPDIR/lemon/nagvis/nagvis/images/iconsets/* $DSTDIR/share/nagvis/nagvis/images/iconsets/

tar xvz -C "$TMPDIR" -f distrib/nagvis/NagVis-Shapes-Server-Dropline.tar.gz >> "$OUTPUTFILE" 2>&1
cp $TMPDIR/shapes_dropline/* $DSTDIR/share/nagvis/nagvis/images/shapes/

tar xvz -C "$TMPDIR" -f distrib/nagvis/NagVis-Template-Darkglass-0.4.tar.gz >> "$OUTPUTFILE" 2>&1
cp -r $TMPDIR/darkglass/nagvis $DSTDIR/share/nagvis/

# Finalise the permissions:
chown -R $NAGIOSUSR:$APACHEGRP "$DSTDIR/share/nagvis"
chmod 664 $DSTDIR/share/nagvis/etc/nagvis.ini.php
chmod 775 $DSTDIR/share/nagvis/nagvis/images/maps
chmod 664 $DSTDIR/share/nagvis/nagvis/images/maps/*
chmod 775 $DSTDIR/share/nagvis/etc/maps
chmod 664 $DSTDIR/share/nagvis/etc/maps/*
chmod 775 $DSTDIR/share/nagvis/var
#chmod 664 $DSTDIR/share/nagvis/var/*

exit 0

# EOF
