#!/bin/bash
# ============================================================================
# 
# Installation and configuration of NDOUtils
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHNDO" = "0" ]; then
	Info "You did not wish to install NDO !"
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting NDOUtils sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/ndoutils-$NDOUTILSVERS.tar.gz >> "$OUTPUTFILE" 2>&1
ln -s  ndoutils-$NDOUTILSVERS "$TMPDIR/ndoutils"

# -----------
# Compilation:
# -----------
echo "Compilation..." | tee -a "$OUTPUTFILE"
pushd $TMPDIR/ndoutils >>"$OUTPUTFILE" 2>&1
./configure --with-mysql-lib="$MYSQLLIB" --with-mysq-inc="$MYSQLINC" >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1

# ------------
# Installation:
# ------------
echo "Installation..."
cd src
cp ndomod-3x.o ndo2db-3x file2sock log2ndo "$DSTDIR/bin"
cd ..

# -------------------------------------
# 70-Populate the database :
# -------------------------------------
pushd db >/dev/null
./installdb -u "$DBADMIN" -p "$DBADMINPWD" -h "$DBSRV" -d "$DBNAME"
popd >/dev/null

# ----------------
# Configure Nagios:
# ----------------

# Configure NDOUtils:
cp config/ndomod.cfg config/ndo2db.cfg "$DSTDIR/etc"
chown $APACHEUSR:$NAGIOSGRP $DSTDIR/etc/{ndo2db.cfg,ndomod.cfg}
chmod 640 $DSTDIR/etc/{ndo2db.cfg,ndomod.cfg}

cp config/misccommands.cfg "$DSTDIR/etc/objects/ndocommands.cfg"
chown $APACHEUSR:$NAGIOSGRP $DSTDIR/etc/objects/ndocommands.cfg
chmod 640 $DSTDIR/etc/objects/ndocommands.cfg

NEWVAL=`echo $DSTDIR | sed -e 's/\//\\\\\//g'`
sed -i -e "s/\/usr\/local\/nagios/$NEWVAL/g" "$DSTDIR/etc/objects/ndocommands.cfg"

sed -i -e "s/ndo2db_user=.*/ndo2db_user=$NAGIOSUSR/" "$DSTDIR/etc/ndo2db.cfg"
sed -i -e "s/ndo2db_group=.*/ndo2db_group=$NAGIOSGRP/" "$DSTDIR/etc/ndo2db.cfg"
sed -i -e "s/db_name=.*/db_name=$DBNAME/" "$DSTDIR/etc/ndo2db.cfg"
sed -i -e "s/db_user=.*/db_user=$DBUSER/" "$DSTDIR/etc/ndo2db.cfg"
sed -i -e "s/db_pass=.*/db_pass=$DBPWD/" "$DSTDIR/etc/ndo2db.cfg"
NEWVAL=`echo $DSTDIR/var/ndo.sock | sed -e 's/\//\\\\\//g'`
sed -i -e "s/socket_name=.*/socket_name=$NEWVAL/" "$DSTDIR/etc/ndo2db.cfg"

NEWVAL=`echo $DSTDIR/var/ndo.sock | sed -e 's/\//\\\\\//g'`
sed -i -e "s/output=.*/output=$NEWVAL/" "$DSTDIR/etc/ndomod.cfg"
NEWVAL=`echo $DSTDIR/var/ndomod.tmp | sed -e 's/\//\\\\\//g'`
sed -i -e "s/buffer_file=.*/buffer_file=$NEWVAL/" "$DSTDIR/etc/ndomod.cfg"

# Set event_broker_options = 1 in nagios.cfg
sed -i -e 's/^event_broker_options=0/event_broker_options=-1/' "$DSTDIR/etc/nagios.cfg" 2>>"$OUTPUTFILE"
NEWVAL=`echo broker_module=$DSTDIR/bin/ndomod-3x.o config_file=$DSTDIR/etc/ndomod.cfg | sed -e 's/\//\\\\\//g'`
sed -i -e "/broker_module.*debug=0/a\\
$NEWVAL" "$DSTDIR/etc/nagios.cfg" 2>>"$OUTPUTFILE"

popd >>"$OUTPUTFILE" 2>&1

exit 0

# EOF
