#!/bin/bash
# ============================================================================
# 
# Installation and configuration script for NRPE
#
# ----------------------------------------------------------------------------
#
# TODO: write a script for a remote install
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHNRPE" = "0" ]; then
	Info "You did not wish to install nrpe !"
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting nrpe sources..." | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nrpe/nrpe-$NRPEVERS.tar.gz >>"$OUTPUTFILE" 2>&1
ln -s nrpe-$NRPEVERS "$TMPDIR/nrpe"

# -----------------
# nrpe Installation:
# -----------------
echo "nrpe Installation..." | tee -a "$OUTPUTFILE"
if [ ! -d "$DSTDIR/nrpe" ]; then
	mkdir -p "$DSTDIR/nrpe"
	if [ $? -ne 0 ]; then
		Error "Could not create the '$DSTDIR/nrpe' directory"
	fi
fi

NRPEDIR=$DSTDIR/nrpe

pushd $TMPDIR/nrpe >>"$OUTPUTFILE" 2>&1

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

./configure >>"$OUTPUTFILE" 2>&1
make all >>"$OUTPUTFILE" 2>&1
cp src/nrpe "$NRPEDIR"
cp src/check_nrpe "$DSTDIR/libexec/"
cp sample-config/nrpe.cfg "$NRPEDIR"
cp sample-config/nrpe.xinetd "$NRPEDIR"

# ------------------
# nrpe Configuration:
# ------------------
echo "nrpe configuration..." | tee -a "$OUTPUTFILE"
NEWVAL=`echo "$NRPEDIR" | sed -e 's/\//\\\\\//g'`
NAGDIR=`echo "$DSTDIR" | sed -e 's/\//\\\\\//g'`


sed -i -e "s/nrpe_user=.*/nrpe_user=$NAGIOSUSR/" \
	-e "s/nrpe_group=.*/nrpe_group=$NAGIOSGRP/" \
	"$NRPEDIR/nrpe.cfg"

sed -i -e "s/.*server[^_].*=.*/\tserver = $NEWVAL\/nrpe/" \
	-e "s/.*server_args.*=.*/\tserver_args = -c $NEWVAL\/nrpe.cfg --inetd/" \
	-e "s/.*user.*=.*/\tuser = $NAGIOSUSR/" \
	-e "s/.*group.*=.*/\tgroup = $NAGIOSGRP/" \
	"$NRPEDIR/nrpe.xinetd"

cat >> "$DSTDIR/etc/objects/commands.cfg" <<END

define command {
        command_name    check_nrpe
        command_line    $DSTDIR/libexec/check_nrpe -H \$HOSTADDRESS\$ -c \$ARG1\$
}
END

popd >>"$OUTPUTFILE" 2>&1

exit 0

# EOF
