#!/bin/bash
# ============================================================================
# 
# Installation and configuration file for NSCA
#
# ----------------------------------------------------------------------------
#
# TODO: write a script to do a remote install
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHNSCA" = "0" ]; then
	Info "You did not wish to install nsca !"
	exit 0
fi

# -----------------------------
# Install libmcrypt if required:
# -----------------------------
#tar xvz -C $TMPDIR -f distrib/libmcrypt-$MCRYPTVERS.tar.gz >>"$OUTPUTFILE" 2>&1
#ln -s libmcrypt-$MCRYPTVERS $TMPDIR/libmcrypt
#pushd $TMPDIR/libmcrypt >>"$OUTPUTFILE" 2>&1
#./configure
#make
#make install
#popd >>"$OUTPUTFILE" 2>&1

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting nsca sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/nsca/nsca-$NSCAVERS.tar.gz >>"$OUTPUTFILE" 2>&1
ln -s nsca-$NSCAVERS "$TMPDIR/nsca"

# -----------------
# nsca Installation:
# -----------------
echo "nsca Installation..." | tee -a "$OUTPUTFILE"
if [ ! -d "$DSTDIR/nsca" ]; then
	mkdir -p "$DSTDIR/nsca"
	if [ $? -ne 0 ]; then
		Error "Could not create the '$DSTDIR/nsca' directory"
	fi
fi

pushd $TMPDIR/nsca >>"$OUTPUTFILE" 2>&1

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

./configure >>"$OUTPUTFILE" 2>&1
make all >>"$OUTPUTFILE" 2>&1
cp src/nsca "$DSTDIR/nsca/"
cp src/send_nsca "$DSTDIR/nsca/"
cp sample-config/nsca.cfg.in "$DSTDIR/nsca/nsca.cfg"
cp sample-config/send_nsca.cfg "$DSTDIR/nsca/send_nsca.cfg"
cp sample-config/nsca.xinetd.in "$DSTDIR/nsca/nsca.xinetd"

# ------------------
# nsca Configuration:
# ------------------
echo "nsca configuration..." | tee -a "$OUTPUTFILE"
NSCADIR="$DSTDIR/nsca"
NEWVAL=`echo "$NSCADIR" | sed -e 's/\//\\\\\//g'`
NAGDIR=`echo "$DSTDIR" | sed -e 's/\//\\\\\//g'`

sed -i -e "s/nsca_user=.*/nsca_user=$NAGIOSUSR/" \
	-e "s/nsca_group=.*/nsca_group=$NAGIOSGRP/" \
	-e "s/server_port=.*/server_port=5667/" \
	-e "s/command_file=.*/command_file=$NAGDIR\/var\/rw\/nagios.cmd/" \
	-e "s/alternate_dump_file=.*/alternate_dump_file=$NAGDIR\/var\/rw\/nsca.dump/" \
	-e "s/#password=.*/password=$NSCAPASSWD/" \
	-e "s/decryption_method=.*/decryption_method=$NSCAENCODING/" \
	"$NSCADIR/nsca.cfg"

sed -i -e "s/#password=.*/password=$NSCAPASSWD/" \
	-e "s/encryption_method=.*/encryption_method=$NSCAENCODING/" \
	"$NSCADIR/send_nsca.cfg"

# ------------------
# xinetd Integration:
# ------------------

sed -i -e "s/.*server[^_].*=.*/\tserver = $NEWVAL\/nsca/" \
	-e "s/.*server_args.*=.*/\tserver_args = -c $NEWVAL\/nsca.cfg --inetd/" \
	-e "s/@nsca_user@/$NAGIOSUSR/" \
	-e "s/@nsca_grp@/$NAGIOSGRP/" \
	"$NSCADIR/nsca.xinetd"

if [ -d /etc/xinetd.d ]; then
	echo "Integration avec xinetd"
	TARGETLN=/etc/xinetd.d/nsca
	if [ -e $TARGETLN ]; then
		AskYesNo "File $TARGETLN already exists. Do you want to recreate it ?"
		if [ "$REP" = "y" ]; then
			rm $TARGETLN
			ln -s "$NSCADIR/nsca.xinetd" $TARGETLN
		fi
	else
		ln -s "$NSCADIR/nsca.xinetd" $TARGETLN
	fi
fi

grep nsca /etc/services >/dev/null 2>&1
if [ $? -ne 0 ]; then
	echo "nsca	5667/tcp	# for Nagios" >> /etc/services
fi

popd >>"$OUTPUTFILE" 2>&1

exit 0

# EOF
