<?php
//
// Interface with nagios configuration
//
include_once "objectfactory.class.php";
include_once "log.class.php";
include_once "mainparms.class.php";
include_once "cgiparms.class.php";
include_once "daemon.class.php";

class Config
{

public function __construct($db)
{
	$this->SetDB($db);
}

public function __destruct() 
{
}

public function SetDB($db)
{
	$this->m_db = $db;
}

public function GetLastError()
{
	return $this->m_lastError;
}

public function SetNagiosMainFileName($fname)
{
	if (!@file_exists($fname)) {
		$this->Log("Could not find main configuration file: $fname");
		return false;
	}
	if (!@is_readable($fname)) {
		$this->Log("Could not read main Configuration file: $fname");
		return false;
	}
	
	Config::$m_mainFile = $fname;
	
	return true;
}

public function GetNagiosMainFileName()
{
	// If no name given, try with the following command:
	// ps -edf | grep nagios | grep "nagios -d"
	if (!empty(Config::$m_mainFile))
		return Config::$m_mainFile;
		
	if (!Daemon::IsRunning())
		return false;
		
	$cmd = "ps -edf | grep nagios | grep ' -d'";
	$lastline = @exec($cmd,$output,$ret);
	
	if ($ret) return false;
	
	$arg = strstr($output[0],"-d");
	if (!$arg) return false;	// abnormal !
	
	$parts = explode(" ",$arg);
	
	$this->SetNagiosMainFileName($parts[1]);
	
	return $parts[1];
}

// Added from v1.0.1:
public function SetNagiosCGIFileName($fname)
{
	if (!@file_exists($fname)) {
		$this->Log("Could not find CGI configuration file: $fname");
		return false;
	}
	if (!@is_readable($fname)) {
		$this->Log("Could not read CGI Configuration file: $fname");
		return false;
	}
	
	Config::$m_cgiFile = $fname;
	
	return true;
}

// Added from v1.0.1:
public function GetNagiosCGIFileName()
{
	// If no name given, try with the following command:
	// ps -edf | grep nagios | grep "nagios -d"
	if (!empty(Config::$m_cgiFile))
		return Config::$m_cgiFile;
		
	return false;
}

// The parsed file are supposed to be correct.
// We assume they have been parsed and validated by nagios itself !
public function ReadMainFile()
{
	// Read the file line per line and call the object
	// handler according to the 'define ...' statement
	
	$fname = Config::$m_mainFile;
	$this->Debug("ReadMainFile($fname)");
	if (!@is_readable($fname)) {
		$this->Log("File '$fname' does not exist or is not readable !");
		return false;
	}
	if (!($lines = @file($fname))) {
		$this->Log("Could not open file '$fname'");
		return false;
	}
	
	$mainparms = new MainParms();
	
	$linenu = 0;
	$comments = array();
	
	foreach ($lines as $realine) {
		$linenu++;
		$line = trim($realine);
		
		// line is a comment ?
		if (substr($line,0,1) == "#") {
			$comments[] = $realine;
			continue;
		}
		
		// blank line ?
		if ($line == "") {
			$comments[] = $realine;
			continue;
		}
	
		// Configuration line:
		$parts = split("=",$line);
		$parm  = $parts[0];
		$value = $parts[1];
		
		// TODO: handle the possible ';' after the parameter value
		
		if (!$mainparms->IsValidParm($parm)) {
			$this->Log("Invalid Parameter '$parm' at line '$linenu'");
			continue;
		}
		//echo "$parm<br>";
		
		// From v0.10: also parse resource_file to read the macros
		// Build the list of resource files:
		if ($parm == "cfg_file" || $parm == "resource_file") {
			//echo $value."<br>";
			$this->m_db->AddResourceFile($value,$parm == "cfg_file");
			$this->ReadObjectFile($value);
		}
		else if ($parm == "cfg_dir") {
			# Added from v0.11
			$this->m_db->AddResourceDir($value);

			$this->RecursiveScan($value);
		}
		else {
			$this->m_db->AddMainParm($parm,$value);
		}
		
		// TODO: two parms affect regexp
	}		
	
	// TODO: store parms

	// from v1.0.1: also reads the CGI file:
	$this->ReadCGIFile();

	// The configuration is synchronized:
	$this->m_db->SetInternalParm('ConfModified','0');
	
	return true;
}

private function RecursiveScan($curdir)
{
	$this->Debug("RecursiveScan($curdir)");

	if (!@file_exists($curdir)) return;
	if (!is_dir($curdir)) return;
	
	$fd = @opendir($curdir);
	while ($fname = @readdir($fd)) {
		if ($fname == "." || $fname == "..") continue;
		if (is_dir("$curdir/$fname")) {
			$this->RecursiveScan("$curdir/$fname");
			continue;
		}
		if (ereg(".cfg$",$fname)) {
			$this->m_db->AddResourceFile("$curdir/$fname",true);
			$this->ReadObjectFile("$curdir/$fname");
		}
	}
	closedir($fd);
}

// The parsed file are supposed to be correct.
// We assume they have been parsed and validated by nagios itself !
public function ReadObjectFile($filename)
{
	$this->Debug("ReadObjectFile($filename)");

	// Read the file line per line and call the object
	// handler according to the 'define ...' statement
	
	if (!@is_readable($filename)) {
		$this->Log("File '$filename' does not exist or is not readable !");
		return false;
	}
	if (!($lines = @file($filename))) {
		$this->Log("Could not open file '$filename'");
		return false;
	}

	$outside_object = true;
	$curobj = NULL;
	$linenu = 0;
	$comments = array();

	foreach ($lines as $realine) {
		//echo $realine."<br>";
		$linenu++;
		$line = trim($realine);
		
		// line is a comment ?
		if (substr($line,0,1) == "#") {
			$comments[] = $realine;
			continue;
		}
		
		// blank line ?
		if ($line == "") {
			$comments[] = $realine;
			continue;
		}
		
		// end of definition ?
		if ($line == "}") {
			$outside_object = true;
			if ($curobj) {
				$this->m_db->Store($curobj,$linenu,$comments);
			}
			$curobj = NULL;
			$comments = array();
			continue;
		}
		
		// line is a new object definition ?
		$tok = strtok($line," \t{");
		if ($tok == "define") {
			$tok = strtok(" \t{");
			$this->Debug("a new $tok has been found !");
			$outside_object = false;
			
			// create a new objet according to the type:
			$curobj = ObjectFactory::Factory($tok);
			$curobj->SetContext($filename);
			continue;
		}
		
		// attribute definition ?
		if (!$curobj) {
			// Changed from v0.10: the line can be a variable definition
			$parts = split("=",$tok);
			$parm  = trim($parts[0]);
			$value = isset($parts[1]) ? trim($parts[1]) : "";
			$this->Log("Line '$linenu' is outside an object definition ($parm=$value)");
			$this->m_db->AddMainParm($parm,$value);
			continue;
		}
		
		$attr = $tok;
		$value = trim(strtok("\n"));

		if (!$curobj->IsValidAttr($attr) && $attr != ";") {
			$this->Log("Invalid Attribute '$attr' at line '$linenu'");
			continue;
		}
		
		// Handle the possible comment after the value:
		$valparts = explode(";",$value);

		if ($attr == ";") {
			# no attribute on the line, just a comment
			# attach it to the last attr:
			$comments[] = $realine."\n";
			continue;
		}

		if (isset($valparts[1])) {
			$comments[] = "; ".$valparts[1]."\n";
		}

		$curobj->AddAttr($attr,trim($valparts[0]));
	}	
	
	return true;
}

// This function is called to recreate all the config files
// from the DB content:
public function WriteConf($dbfile,$backupdir = ".")
{
	// TODO: should also save the main file and cgi file
	
	// Loop on all the configuration file:
	$filelist = $this->m_db->GetConfigFileList();
	foreach ($filelist as $file) {
		//$this->Debug("Handle file ${file['filename']}");
		$filename = $file['filename'];
		$this->Debug("Write file $filename");
		
		// Backup the former file:
		$backupname = $backupdir."/".basename($filename);
		
		// It may happens that many config file have the same basename
		// (because they are located under different directories)
		if (@file_exists($backupname)) {
			$idx = 1;
			do {
				$bkpname = "$backupname-$idx";
				$idx++;
			} while (@file_exists($bkpname));
			$backupname = $bkpname;
		}
		if (!@rename($filename,$backupname)) {
			throw new Exception ("Could not backup file $filename to $backupname");
		}
	
		// We call the function that can throw an exception here, to make sure
		// that when the file is opened in write mode, it will be fill with 
		// the object definition or it will be left unchanged.
		// Before v0.4, the following call was made in WriteObjectFile() and could
		// leave the configuration file empty (thanks to Ryan Schwartz who raised the bug !)
		try {
			$objlist = $this->m_db->GetObjForFile($filename);

			if (!($fout = @fopen($filename,"w"))) {
				throw new Exception ("File '$filename' is not writeable !");
			}
				
			$this->WriteObjectFile($objlist,$filename,$fout);
		
			fclose($fout);
			
			// Added from v1.0.2:
			@chmod($filename,0640);	// nagios group readable
		}
		catch (Exception $e) {
			throw new Exception("Could not update configuration file '$filename'"); 
		}
	}

	// Added from v1.0.1: update the cgi.cfg file to take care of user rights:
	$this->UpdateCGIFile();
	
	// Touch the bin file to avoid a configuration reload...
	if (!@touch($dbfile))
		throw new Exception ("Could not change the modification time of file '$dbfile'");

	// Also mark the conf as synchronized:
	$this->m_db->SetInternalParm('ConfModified','0');
}

public function WriteObjectFile(&$objlist,$file,$fout)
{
	//print_r($objlist);
	foreach ($objlist as $obj) {
		$objattr = $obj->GetAttr();
		$objdef  = $obj->GetDef();

		if (isset($objattr['__objprecomment']))
			fputs($fout,$objattr['__objprecomment']);
				
		fputs($fout,"define ".strtolower($obj->GetType())." {\n");
		
		while (list($attr,$attrdef) = each($objdef)) {
			if (isset($attrdef['flag']) && 
			    ($attrdef['flag'] & OA::NOT_DUMPABLE))
				continue;
			
			if (!isset($objattr[$attr]) || $objattr[$attr] == NULL)
				continue;
				
			$val = $objattr[$attr];
			fputs($fout,"\t$attr\t$val\n");
		}
		
		// From v1.0: also saves the xattrs:
		foreach ($objattr as $attr => $value) {
		  if (NagiosObject::IsCustomAttr($attr)) {
			  fputs($fout,"\t$attr\t$value\n");
		  }
		}
		
		fputs($fout,"}\n");
	}
}

// Added from v0.11
public function AddConfFile($filename,$is_new=false)
{
	$this->m_db->AddResourceFile($filename,true);

	// Update the main configuration file:
        $content = file(Config::$m_mainFile); 
	$fname = str_replace("/","\\/",$filename);
	$match = preg_grep("/^cfg_file.*=.*$fname/",$content);
	if (!count($match)) {
		$content[] = "cfg_file=$filename\n";
		$this->UpdateMainConfFile($content);
		if (!$is_new) $this->m_db->SetInternalParm('ConfModified','1');
	}
}

// Added from v0.11
public function DeleteConfFile($filename)
{
	$this->m_db->DeleteResourceFile($filename);

	// Update the main configuration file:
        $content = file(Config::$m_mainFile); 
	$fname = str_replace("/","\\/",$filename);
	foreach ($content as $line) {
		if (!preg_match("/^cfg_file.*=.*$fname/",$line)) {
			$newcontent[] = $line;
		}
	}

	$this->UpdateMainConfFile($newcontent);
	$this->m_db->SetInternalParm('ShouldRestart','1');
}

// Added from v0.11
public function AddConfDir($dirname,$is_new=false)
{
	$this->m_db->AddResourceDir($dirname);

	// Update the main configuration file:
        $content = file(Config::$m_mainFile); 
	$fname = str_replace("/","\\/",$dirname);
	$match = preg_grep("/^cfg_dir.*=.*$fname/",$content);
	if (!count($match)) {
		$content[] = "cfg_dir=$dirname\n";
		$this->UpdateMainConfFile($content);
		if (!$is_new) $this->m_db->SetInternalParm('ConfModified','1');
	}
}

// Added from v0.11
public function DeleteConfDir($dirname)
{
	$this->m_db->DeleteResourceDir($dirname);

	// Update the main configuration file:
        $content = file(Config::$m_mainFile); 
	$fname = str_replace("/","\\/",$dirname);
	foreach ($content as $line) {
		if (!preg_match("/^cfg_dir.*=.*$fname/",$line)) {
			$newcontent[] = $line;
		}
	}

	// Also remove all the included config files:
	$filelist = $this->m_db->GetConfigFileList();
        foreach ($filelist as $file) {
		$filename = $file['filename'];
		if ($this->_IsUnderConfDir($filename,$dirname)) {
			$this->m_db->DeleteResourceFile($filename);
		}
	}

	$this->UpdateMainConfFile($newcontent);
	$this->m_db->SetInternalParm('ShouldRestart','1');
}

// Added from v0.11
public function IsUnderConfDir($filename)
{
	$dirlist = $this->m_db->GetMainDirList();

        $fdir = dirname($filename)."/";
        foreach ($dirlist as $dir) {
		if ($this->_IsUnderConfDir($filename,$dir['dirname']))
			return true;
        }

	return false;
}

// Added from v0.11
private function _IsUnderConfDir($filename,$dirname)
{
        $fdir = dirname($filename)."/";
	if ($dirname[strlen($dirname)-1] != '/') $dirname .= "/";
	if (strpos($fdir,$dirname) !== false) {
		return true;
        }

	return false;
}

// Added from v0.11
public function CountOfObjectInFile($filename)
{
	return $this->m_db->GetObjCountForFile($filename);
}

////////////////////////////////////////////////////////////////////////////

static private $m_mainFile;
static private $m_cgiFile;
private $m_db;

// Added from v0.11
private function UpdateMainConfFile($content)
{
	if (($fd = fopen(Config::$m_mainFile,"w")) === FALSE) {
		throw new Exception("Could not open file ".Config::$m_mainFile." in write mode");
	}

	foreach ($content as $line) 
		fwrite($fd,$line);

	fclose($fd);

	// To avoid a rebuild of the DB file, touch it ! (ugly globals access)
	touch($GLOBALS['gDBFile']);
}

// Added from v1.0.1
private function ReadCGIFile()
{
	// Read the file line per line and memorize 
	// user authorizations
	
	$fname = Config::$m_cgiFile;
	$this->Debug("ReadCGIFile($fname)");
	if (!@is_readable($fname)) {
		$this->Log("File '$fname' does not exist or is not readable !");
		return false;
	}
	if (!($lines = @file($fname))) {
		$this->Log("Could not open file '$fname'");
		return false;
	}
	
	$cgiparms = new CGIParms();
	
	$linenu = 0;
	$comments = array();
	
	foreach ($lines as $realine) {
		$linenu++;
		$line = trim($realine);
		
		// line is a comment ?
		if (substr($line,0,1) == "#") {
			$comments[] = $realine;
			continue;
		}
		
		// blank line ?
		if ($line == "") {
			$comments[] = $realine;
			continue;
		}
	
		// Configuration line:
		$parts = split("=",$line);
		$parm  = $parts[0];
		$value = $parts[1];
		if ($cgiparms->IsAuthParm($parm)) {
			// Get the user list and update the database:
			$userlist = explode(',',$value);
			foreach ($userlist as $user) {
				$this->m_db->SetUserAuth($user,$parm);
			}
		}
	}		
	
	return true;
}

private function UpdateCGIFile()
{
	$fname = Config::$m_cgiFile;
	$this->Debug("UpdateCGIFile($fname)");
	if (!@is_writable($fname)) {
		$this->Log("File '$fname' does not exist or is not writeable !");
		return false;
	}

	// Build the new authorizations:
	$cgiparms = new CGIParms();
	$cgioptions = $cgiparms->GetCGIOptions();
	$newvalues = array();
	foreach ($cgioptions as $option) $newvalues[$option] = "";

	try {
		$userlist = $this->m_db->GetUsersAuth();
	}
	catch (Exception $e) {
		throw new Exception("Could not update configuration file '$fname': ".$e->getMessage()); 
	}

	foreach ($userlist as $user) {
		$username = $user['user'];
		foreach ($cgioptions as $option) {
			if ($user[$option]) {
				if ($newvalues[$option]) $newvalues[$option] .= ",";
				$newvalues[$option] .= $username;
			}
		}
	}

	// Read the entire file:
	if (!($lines = @file($fname))) {
		$this->Log("Could not open file '$fname'");
		return false;
	}

	// Recreate the file and substitute the auth lines:
	if (!($fout = @fopen($fname,"w"))) {
		throw new Exception ("File '$fname' is not writeable !");
	}
	foreach ($lines as $line) {
		$line = trim($line);

		$parts = split("=",$line);
		$parm  = $parts[0];
		$value = isset($parts[1]) ? $parts[1] : "";
		if (!$parm || !$value) {
			fputs($fout,"$line\n");
			continue;
		}
		
		$authfound = false;
		foreach ($cgioptions as $option) {
			if ($parm == $option) {
				fputs($fout,"$parm=".$newvalues[$option]."\n");
				$authfound = true;
				break;
			}
		}

		if (!$authfound)
			fputs($fout,"$line\n");
	}
	
	fclose($fout);
	
	return true;
}

private function Debug($msg)
{
	global $gLogger;
	$gLogger->Debug($msg);
}

private function Log($msg)
{
	global $gLogger;
	$gLogger->Log($msg);
	$this->m_lastError = $msg;
}

};
?>
