<?php
#
# Starting with v0.6, we support both sqlite and PDO_sqlite
#
include_once "objectfactory.class.php";
include_once "cgiparms.class.php";

// Note: we could add some PRIMARY KEY / UNIQUE INDEX in the created tables
//       to make them more robust.

class Database 
{
public function __construct($dbname)
{
	// $dbname must be writeable:
	$dir = dirname($dbname);
	if (!@is_writeable($dir)) {
		throw new Exception("Directory '$dir' is not writeable");
	}
	if (@file_exists($dbname) && !@is_writeable($dbname)) {
		throw new Exception("'$dbname' is not writeable");
	}
	
	$this->m_dbname = $dbname;

	if ($GLOBALS['g_use_sqlite'])
		$this->m_db = new SQLiteDatabase($dbname,0666,$sqliteerror);
	if ($GLOBALS['g_use_pdo_sqlite'])
		$this->m_db = new PDO("sqlite:$dbname");

	if ($this->m_db === false)
		throw new Exception($sqliteerror);
}

public function __destruct()
{
}

// Create the tables for each object type to store:
public function Create()
{
	$objtypes = ObjectFactory::GetObjectTypeList();
	
	foreach ($objtypes as $objtype) {
		$obj = ObjectFactory::Factory($objtype);
		$req = $obj->GetCreateTableRequest();
		$result = $this->Exec($req,$rowcount);
	}

	// from v1.0: add a table for "custom attributes" 
	// (the ones which names start with an '_' but not '__')
	$req = "CREATE TABLE xattrs (
			__objid INT,
			name	VARCHAR(255),
			value	VARCHAR(255)
			)";
	$result = $this->Exec($req,$rowcount);

	// Create the additional tables:
	// the way object are memorized is not the most efficient one
	// but it is simple:
	
	// Note: "linenu" is never used...
	$req = "CREATE TABLE cfgfile (
			filename VARCHAR(255),
			objtype CHAR(32),
			objid INT,
			linenu INT)";
	$result = $this->Exec($req,$rowcount);

	// Create tables for the main configuration file:
	$req = "CREATE TABLE mainfiles (filename VARCHAR(255), is_cfg INT)";
	$result = $this->Exec($req,$rowcount);
		
	$req = "CREATE TABLE mainparms (parm VARCHAR(255), value TEXT)";
	$result = $this->Exec($req,$rowcount);

	// Added since v0.11:
	$req = "CREATE TABLE maindirs (dirname VARCHAR(255))";
	$result = $this->Exec($req,$rowcount);

	// (New since version 0.2):
	// Create a table for internal parameters:
	$req = "CREATE TABLE internalparms (parm VARCHAR(255), value TEXT)";
	$result = $this->Exec($req,$rowcount);

	// And create the supported internal parameters:
	$this->AddInternalParm("ConfModified","0");
	$this->AddInternalParm("ShouldRestart","0"); # added from v0.11

	// (New since version 0.3):
	// Create a table for the Graphs:
	// TODO: suppress 
	/*
        $req = "CREATE TABLE graphs (".
			  "idx INT(11) NOT NULL PRIMARY KEY,".
			  "host VARCHAR(64) NOT NULL DEFAULT '',".
			  "service VARCHAR(64) NOT NULL DEFAULT '',".
			  "rrdfile VARCHAR(128) NOT NULL DEFAULT '',".
			  "label VARCHAR(64) NOT NULL DEFAULT '',".
			  "unit VARCHAR(64) NOT NULL DEFAULT '',".
			  "extraargs VARCHAR(64) DEFAULT '',".
			  "comment VARCHAR(255) DEFAULT '',".
			  "changed TIMESTAMP NOT NULL,".
			  "created TIMESTAMP NOT NULL)";
	$result = $this->m_db->query($req,SQLITE_ASSOC,$sqliteerror);
	if ($result === false)
		throw new Exception($sqliteerror);
	*/

	// From v1.0.1: create table for user auth:
	$auth = new CGIParms();
	$authlist = $auth->GetCGIOptions();
	$req = "CREATE TABLE users (user VARCHAR(64) NOT NULL DEFAULT ''";
	foreach ($authlist as $auth) {
		$req .= ",$auth INT(1)";
	}
	$req .= ")";
	$result = $this->Exec($req,$rowcount);
}

// Store configuration filenames:
public function AddResourceFile($filename,$is_cfg)
{
	$filename = $this->Quote($filename);
	$req = "INSERT INTO mainfiles VALUES($filename,'$is_cfg')";
	$result = $this->Exec($req,$rowcount);
}

// Added from v0.11
public function DeleteResourceFile($filename)
{
	$filename = $this->Quote($filename);
	$req = "DELETE FROM mainfiles WHERE filename = $filename";
	$result = $this->Exec($req,$rowcount);
}

// Added from v0.11
public function AddResourceDir($dirname)
{
	$dirname = $this->Quote($dirname);
	$req = "INSERT INTO maindirs VALUES($dirname)";
	$result = $this->Exec($req,$rowcount);
}

// Added from v0.11
public function DeleteResourceDir($dirname)
{
	$dirname = $this->Quote($dirname);
	$req = "DELETE FROM maindirs WHERE dirname = $dirname";
	$result = $this->Exec($req,$rowcount);
}

// Add a main parameter in the DB:
public function AddMainParm($parm,$value)
{
	$parm = $this->Quote($parm);
	$value = $this->Quote($value);
	
	$req = "INSERT INTO mainparms VALUES ($parm,$value)";
	$result = $this->Exec($req,$rowcount);
}

// Return the value of the specified main parm:
public function GetMainParm($parm)
{
	$req = "SELECT value FROM mainparms WHERE parm = '$parm'";
	$row = $this->GetResult($req);
	return $row[0]['value'];
}

// Compute and return the higher allocated Object ID:
public function GetMaxObjID()
{
	$req = "SELECT MAX(objid) AS objid FROM cfgfile";
	$row = $this->GetResult($req);
	return $row[0]['objid']+1;
}

// Store a new object in the Database:
public function Store($obj,$linenu,$comments,$objid = 0)
{
	//print_r($obj);
	// if no $objid specified or 0, it is called by config.class.php
	// and we use the private counter:
	if ($objid == 0)
		$objid = Database::$m_objid++;
	
	$req = $obj->Store($this,$comments,$objid);
	
	$result = $this->Exec($req,$rowcount);
		
	// We must also memorize in "cfgfile" table the order of 
	// object appearance
	$req = "INSERT INTO cfgfile (filename,objtype,objid,linenu) VALUES(".
			$this->Quote($obj->GetCfgFile()).",'".$obj->GetType()."',".$obj->GetID().",$linenu)";
	$result = $this->Exec($req,$rowcount);

	// From v1.0: stores the custom attributes:
	$this->StoreCustomAttr($obj);

	$this->SetInternalParm('ConfModified','1');
}

// From v1.0: store custom attributes
private function StoreCustomAttr($obj)
{
	$attrlist = $obj->GetCustomAttrList();
	foreach ($attrlist as $attr) {
		$value = $obj->GetAttr();
		$value = $value[$attr];
		$req = "INSERT INTO xattrs (__objid,name,value) VALUES (".$obj->GetID().",".$this->Quote($attr).",".$this->Quote($value).")";
		$result = $this->Exec($req,$rowcount);
	}
}

// Update an existing object in the Database:
public function Update($obj)
{
	// if __cfgfile has changed, update the cfgfile table:
	$objtype = $obj->GetType();
	$objid   = $obj->GetID();
	$oldobj  = $this->BuildObject($objtype,$objid);
	
	$req = $obj->Update($this);
	
	$result = $this->Exec($req,$rowcount);
		
	$oldattr = $oldobj->GetAttr();
	$newattr = $obj->GetAttr();
	if ($oldattr['__cfgfile'] != $newattr['__cfgfile']) {
		// Note: now, objid is unique among all object types, 
		//       but, in case of...
		$req = "UPDATE cfgfile SET filename = '".$newattr['__cfgfile']."' WHERE objid = '$objid' AND objtype = '$objtype'";
		$result = $this->Exec($req,$rowcount);
	}

	// From v1.0: update custom attributes: (delete and recreate)
	$this->DeleteCustomAttr($objid);
	$this->StoreCustomAttr($obj);

	$this->SetInternalParm('ConfModified','1');
}

// Delete an object:
public function Delete($objtype,$objid)
{
	// assert($type) is in ObjectFactory::GetObjectTypeList
	// throw exception otherwise
	$objid = $this->Quote($objid);
	$req = "DELETE FROM $objtype WHERE __objid = $objid";
	$result = $this->Exec($req,$rowcount);

	// Bug fixed since v0.4 (thanks to Ryan Schwartz)
	// must also delete the entry in cfgfile table:
	$req = "DELETE FROM cfgfile WHERE objid = $objid";
	$result = $this->Exec($req,$rowcount);
	
	// From v1.0: delete custom attributes
	$this->DeleteCustomAttr($objid);

	$this->SetInternalParm('ConfModified','1');
				
	return $rowcount;
}

// From v1.0: delete custom attributes:
private function DeleteCustomAttr($objid)
{
	$req = "DELETE FROM xattrs WHERE __objid = $objid";
	$result = $this->Exec($req,$rowcount);
}

// Return the list of objects for a given type:
public function GetObjectOfType($objtype)
{
	// assert($type) is in ObjectFactory::GetObjectTypeList
	// throw exception otherwise
	$curobj = ObjectFactory::Factory($objtype);
	$indexes = $curobj->GetIndex();
	
	// Changed from v0.5:
	//$req = "SELECT * FROM $objtype ORDER BY name DESC ";
	$req = "SELECT * FROM $objtype ORDER BY ";
	
	for ( $idx = 0 ; $idx < count($indexes) ; $idx++ ) {
		if ($idx) $req .= ", ";
		$req .= "${indexes[$idx]} ASC";
	}
	$this->Debug("GetObjectOfType: $req");
	
	return $this->GetResult($req);
}

// Return the list of the configuration files from the main configuration:
public function GetMainFileList()
{
	$req = "SELECT filename FROM mainfiles WHERE is_cfg = 1 GROUP BY filename";
	return $this->GetResult($req);
}

// Return the list of the configuration files:
public function GetConfigFileList()
{
	return $this->GetMainFileList();
	#$req = "SELECT filename FROM cfgfile GROUP BY filename";
	#return $this->GetResult($req);
}

// Added from v0.11
public function GetMainDirList()
{
	$req = "SELECT dirname FROM maindirs";
	return $this->GetResult($req);
}

// Return the list of the objects defined in the given filename:
public function GetObjForFile($filename)
{
	$req = "SELECT * FROM cfgfile WHERE filename = ".$this->Quote($filename)." ORDER BY objid ASC, objtype ASC";
	$this->Debug("GetObjForFile: $req");

	$rowcount = 0;
	$result = $this->GetResult($req,$rowcount);

	$objlist = array();
	if (!is_array($result) || count($result) == 0)
		return $objlist;

	foreach ($result as $row) {
		// Get the real object:
		$objtype = $row['objtype'];
		$objid = $row['objid'];
		try {
			$objlist[] = $this->BuildObject($objtype,$objid);
		}
		catch (Exception $e) {
			throw new Exception("Error in GetObjForFile($filename): $e"); 
		}
	}
	
	return $objlist;
}

// Added from v0.11
public function GetObjCountForFile($filename)
{
	$req = "SELECT COUNT(*) AS count FROM cfgfile WHERE filename = ".$this->Quote($filename);
	$this->Debug("GetObjCountForFile: $req");

	$result = $this->GetResult($req,$rowcount);
	return $result[0]['count'];
}

// Retrieve an object from the Database using the Object ID:
public function BuildObject($objtype,$objid)
{
	$obj = ObjectFactory::Factory($objtype);
	$req = "SELECT * FROM $objtype WHERE __objid = '$objid'";
	$this->Debug("BuildObject: $req");

	$row = $this->QuerySingleRow($req,$rowcount);

	if ($rowcount != 1) {
		//echo "rowcount=$rowcount\n";
		throw new Exception("Multiple objects for $objtype/$objid !");
	}
	
	foreach ($row as $col => $val) {
		$obj->AddAttr($col,$val);
	}

	// From v1.0: add custom attributes:
	$req = "SELECT * FROM xattrs WHERE __objid = '$objid'";
	$result = $this->GetResult($req,$rowcount);
	foreach ($result as $row) {
		$obj->AddAttr($row['name'],$row['value']);
	}
	
	return $obj;	
}

// Compute all the inherited and the default attributes for an object:
public function BuildObjectInherited($obj)
{
	assert(isset($obj));
	assert($obj->GetType() != "");
	//assert($obj->GetID() != "");

	$attrdef = $obj->GetDef();
	$attr = $obj->GetAttr();
	if (isset($attr['use']) && $attr['use']) {
		try {
			// "user ".$attr['use']." ";
			// maybe the parent has its own parent ?
			$parent = $this->BuildObjectFromName($obj->GetType(),$attr['use']);
			$this->BuildObjectInherited($parent);
			#print_r($parent);
			
			$parentattr = $parent->GetAttr();
			foreach ($parentattr as $key => $val) {
				// some attributes are never inherited (like 'register'):
				if (isset($attrdef[$key]['flag']) &&
				    $attrdef[$key]['flag'] & OA::DONT_INHERIT) 
					continue;
							
				#echo $key."/".$val."<br>";
				// Parent attribute is inherited if it is non NULL
				// and if the object attribute is empty:
				if (!isset($attr[$key])) $attr[$key] = "";
				if ($attr[$key] !== "0" && 
				    $attr[$key] == "" && 
				    (($val !== "0" && $val != "") || $parent->GetOrigin($key) != "")) {
					$obj->AddAttr($key,$val);
					if ($parent->GetOrigin($key)) {
						#echo "$key:setorigin ".$parent->GetOrigin($key)."<br>";
						$obj->SetOrigin($key,$parent->GetOrigin($key));
					}
					else {
						#echo "$key:setorigin2 ".$attr['use']."<br>";
						$obj->SetOrigin($key,$attr['use']);
					}
				}
				
				// From v1.0: handle implied inheritance:
				elseif ($attr[$key] !== "0" && $attr[$key] == "") {
				    $impliedlist = $obj->GetImpliedInheritance();
				    foreach ($impliedlist as $implkey => $implval) {
				        if ($key == $implkey) {
				            $inheritedValue = $origin = "";
				            list($inheritedValue,$origin) = $this->GetInheritedValue($attr,$implkey,$implval);
				            if ($inheritedValue !== "" && $origin != "") {
    				            $obj->AddAttr($key,$inheritedValue);	
	   			                $obj->SetOrigin($key,$origin);            
				            }
				            break;
				        }
				    }
				}				

				// From v0.5: add an attribute that indicates if the
				// attribute COULD BE inherited
				if ($val !== "") {
					$obj->SetInheritable($key,$attr['use']);
				}
				else if ($parent->GetOrigin($key) != "") {
					$obj->SetInheritable($key,$parent->GetOrigin($key));
				}
			}
		}
		catch (Exception $e) {
			throw new Exception("BuildObjectInherited failed: $e");
		}
	}
}

// Added from v1.0: tries to determine the value of an implied inheritance:
public function GetInheritedValue($attr,$key,$valdescr)
{
	// From v1.0.1, we handle multiple inheritance. In that case,
	// $objname may be an array of values
	//echo "attr="; print_r($attr);echo "<br>key="; print_r($key); echo "<br>valdescr=";print_r($valdescr);echo "<br>";
	if ($attr[$valdescr['refattr']] == "") return array("","");

	if (is_array($attr[$valdescr['refattr']])) {
		foreach ($attr[$valdescr['refattr']] as $refname) {
			$obj = $this->BuildObjectFromName($valdescr['objtype'],$refname,$valdescr['refattr']);
			$this->BuildObjectInherited($obj);
			$parentattr = $obj->GetAttr();
			if (isset($parentattr[$valdescr['attrname']]))
				return array($parentattr[$valdescr['attrname']],$refname);
		}
		return array("","");
	}

	// From v1.0.1: the value can be a comma-separated list of values:
	$parts = explode(",",$attr[$valdescr['refattr']]);
	foreach ($parts as $part) {
		$obj = $this->BuildObjectFromName($valdescr['objtype'],$part,$valdescr['refattr']);
		$this->BuildObjectInherited($obj);
		$parentattr = $obj->GetAttr();
		if (isset($parentattr[$valdescr['attrname']]))
			return array($parentattr[$valdescr['attrname']],$part);
	}
	return array("","");
}

// Retrieve an object from the Database using the specified Object name:
// if $nameattr is null, the used field is "name" and this function is called
// to build the parent object which must be a template (only templates have
// a "name" attribute). 
// if $nameattr is not null, it gives the name of the search field
public function BuildObjectFromName($objtype,$objname,$nameattr = "name")
{
	$obj = ObjectFactory::Factory($objtype);
	$req = "SELECT * FROM $objtype WHERE $nameattr = '$objname'";
	$this->Debug("BuildObjectFromName: $req");

	$row = $this->QuerySingleRow($req,$rowcount);

	if ($rowcount != 1) {
		throw new Exception("Multiple objects for $objtype/$objname !");
	}
	
	foreach ($row as $col => $val) {
		$obj->AddAttr($col,$val);
	}
	
	return $obj;	
}

public function GetObjectFromKey($obj,&$attrvalues)
{
	$objtype = $obj->GetType();
	$indexes = $obj->GetIndex();
	
	$req = "SELECT * FROM $objtype WHERE ";
	$firstindex = true;
	foreach ($indexes as $index) {
		if (!$firstindex) $req .= "AND ";
		$firstindex = false;
		if (is_array($attrvalues[$index])) 
			$value = implode(",",$attrvalues[$index]);
		else
			$value = $attrvalues[$index];
		$value = $this->Quote($value);
		$req .= "$index = $value";
	}
	$this->Debug("GetObjectFromKey: $req");

	return $this->GetResult($req);
}

public function GetTemplateList($objtype)
{
	$req = "SELECT * FROM $objtype WHERE name <> '' AND register = 0";
	$this->Debug("GetTemplateList: $req");

	return $this->GetResult($req);
}

// Return the list of the object which uses the specified template:
public function GetInheritedUsage($objtype,$objname)
{
	$objname = $this->Quote($objname);
	$req = "SELECT * FROM $objtype WHERE use = $objname";
	$this->Debug("GetInheritedUsage: $req");

	return $this->GetResult($req);
}

// Set an internal parameter value:
public function SetInternalParm($parm,$value)
{
	$value = $this->Quote($value);
	$req = "UPDATE internalparms SET value = $value WHERE parm = '$parm'";
	$result = $this->Exec($req,$rowcount);
	if ($result === false)
		throw new Exception($sqliteerror);	
	return true;
}

// Get an internal parameter value:
public function GetInternalParm($parm)
{
	$parm = $this->Quote($parm);
	$req = "SELECT value FROM internalparms WHERE parm = $parm";
	return $this->GetResult($req);
}

// User Auth management added from v1.0.1:
public function SetUserAuth($user,$parm,$value=1)
{
	$parm = $this->Quote($parm);
	$result = $this->GetUserAuth($user);
	$user = $this->Quote($user);
	if (count($result) == 0) {
		$req = "INSERT INTO users (user) VALUES ($user)";
		$result = $this->Exec($req,$rowcount);
	}

	$req = "UPDATE users SET $parm = $value WHERE user = $user";
	$result = $this->Exec($req,$rowcount);

	return true;
}

public function GetUserAuth($user)
{
	$user = $this->Quote($user);
	$req = "SELECT * FROM users WHERE user = $user";
	return $this->GetResult($req);
}

public function GetUsersAuth()
{
	$req = "SELECT * FROM users";
	return $this->GetResult($req);
}

public function DeleteUser($user)
{
	$user = $this->Quote($user);
	$req = "DELETE FROM users WHERE user = $user";
	return $this->GetResult($req);
}

public function UsersModified()
{
	$this->SetInternalParm('ConfModified','1');
}

// Internal functions start here:
private function Exec($req,&$rowcount)
{
	$this->Debug($req);
	#echo "$req<br>";

	if ($GLOBALS['g_use_sqlite']) {
		$result = $this->m_db->query($req,SQLITE_ASSOC,$sqliteerror);
		if ($result) $rowcount = $result->count();
	}
	if ($GLOBALS['g_use_pdo_sqlite']) {
		try {
			$rowcount = $this->m_db->exec($req);
			$result = true;
			#echo "result=$result, rowcount=$rowcount<br>";
		}
		catch (PDOException $e) {
			$result = false;
			$sqliteerror = $e->getMessage();
		}
	}
	if ($result === false)
		throw new Exception($sqliteerror);
}

# Normally called to get a SINGLE row:
private function QuerySingleRow($req,&$rowcount)
{
	$this->Debug($req);
	#echo "$req<br>";
	$rowcount = 0;

	if ($GLOBALS['g_use_sqlite']) {
		$result = @$this->m_db->query($req,SQLITE_ASSOC,$sqliteerror);
		if ($result) {
			$rowcount = $result->count();
			$row = $result->fetch();
		}
	}
	if ($GLOBALS['g_use_pdo_sqlite']) {
		try {
			$result = $this->m_db->query($req);
			$row = $result->fetch();
			#print_r($row);
			#$rowcount = $result->rowCount();
			if (is_array($row)) { $rowcount = 1; }
			#echo "rowcount=$rowcount<br>";
		}
		catch (PDOException $e) {
			$result = false;
			$sqliteerror = $e->getMessage();
		}
	}
	if ($result === false)
		throw new Exception($sqliteerror);

	return $row;
}

// Quote a string:
public function Quote($s)
{
	if ($GLOBALS['g_use_sqlite'])
		return "'".sqlite_escape_string($s)."'";
	if ($GLOBALS['g_use_pdo_sqlite']) 
		return $this->m_db->quote($s);

	return "Error !";
}

private function AddInternalParm($parm,$value)
{
	$value = $this->Quote($value);
	$req = "INSERT INTO internalparms VALUES ('$parm',$value)";
	$result = $this->Exec($req,$rowcount);
	return true;
}

private function GetResult($req)
{
	$this->Debug($req);
	#echo "$req<br>";

	if ($GLOBALS['g_use_sqlite'])
		$result = @$this->m_db->query($req,SQLITE_ASSOC,$sqliteerror);
	if ($GLOBALS['g_use_pdo_sqlite']) {
		try {
			$result = $this->m_db->query($req);
		}
		catch (PDOException $e) {
			$result = false;
			$sqliteerror = $e->getMessage();
		}
	}
	if ($result === false)
		throw new Exception($sqliteerror);

	$objlist = array();
	while ($row = $result->fetch()) {
		foreach ($row as $col => $val) {
			$row[$col] = $val;
		}
		$objlist[] = $row;
	}

	#echo "GetResult returns ".count($objlist)." items<br>";

	return $objlist;
}

private function Debug($msg)
{
	global $gLogger;
	$gLogger->Debug($msg);
} 

private $m_dbname;
private $m_db;
static private $m_objid = 0;
};
?>
