<?php
include_once "objectattribute.class.php";

abstract class NagiosObject
{
public function __construct()
{
	$this->m_attr = array ();	// attributes value
	$this->m_origin = array();	// attributes origin (self, parent...)
}

public function SetContext($file)
{
	$this->m_attr['__cfgfile'] = $file;
}

public function AddAttr($attr,$val)
{
	$this->m_attr[$attr] = $val;
}

public function SetOrigin($attr,$origin)
{
	$this->m_origin[$attr] = $origin;
}

public function GetOrigin($attr)
{
	if (isset($this->m_origin[$attr]))
		return $this->m_origin[$attr];
	return NULL;
}

public function SetInheritable($attr,$origin)
{
	$this->m_inheritable[$attr] = $origin;
}

public function IsInheritable($attr)
{
	// TODO: should add a new flag to object attribute to distinguish inheritable and non-inheritable attrs
	if ($attr == "use" || $attr == "__cfgfile" || $attr == "__objid") return false;
	if (in_array($attr,$this->GetIndex())) return false;

	return (isset($this->m_inheritable[$attr])) ? $this->m_inheritable[$attr] : "";
}

public function GetCfgFile()
{
	return $this->m_attr['__cfgfile'];
}

public function GetType()
{
	return $this->m_objtype;
}

public function GetID()
{
	// Computed once the object is stored:
	return $this->m_attr['__objid'];
}

public function GetAttr()
{
	return $this->m_attr;
}

public function HasDetails()
{
	if (!isset($this->m_hasDetails)) return false;
	return $this->m_hasDetails;
}

public function UnsetAttr($attr)
{
	unset($this->m_attr[$attr]);
}

public function GetDef()
{
	return array_merge($this->m_attrdef2,NagiosObject::$m_commonattrdef);
}

public function GetGroups()
{
	if (!isset($this->m_groups))
		return NULL;
	
	return $this->m_groups;
}

public function IsTemplate()
{
	if (!isset($this->m_attr['name'])) return false;
	if (empty($this->m_attr['name'])) return false;
	if (isset($this->m_attr['register']) &&
	    $this->m_attr['register']) return false;
	    
	return true;
}

public function IsValidAttr($attr)
{
	foreach ($this->m_attrdef2 as $key => $def) {
		if (isset($def['flag']) && ($def['flag'] & OA::NOT_DUMPABLE))
			continue;
		if ($attr == $key) return true;
	}
	foreach (NagiosObject::$m_commonattrdef as $key => $def) {
		if (isset($def['flag']) && ($def['flag'] & OA::NOT_DUMPABLE))
			continue;
		if ($attr == $key) return true;
	}

	// From v1.0: custom attributes are valid
	if (NagiosObject::IsCustomAttr($attr)) return true;
	
	return false;
}

// From v1.0: handle custom attribute:
static public function IsCustomAttr($name)
{
	return $name[0] == '_' && $name[1] != '_';
}

// From v1.0: handle implied inheritance:
public function GetImpliedInheritance()
{
    return $this->m_implied2;    
}

public function GetIndex()
{
	$index = array();
	
	foreach ($this->GetDef() as $attr => $def) {
		if (isset($def['flag']) && 
		    ($def['flag'] & OA::IS_INDEX))
			array_push($index,$attr);
	}
	
	return $index;
}

// When called by config.class.php to create the Database, $objid are
// automatically computed. In fact, the $objid must be unique across
// all files and all object types. Thus it should always be specified 
// by the caller.
// When called by newobject.php, the $objid must be explicitly given
public function Store($db,$comment,$objid = 0)
{
	if ($objid == 0)
		$this->m_attr['__objid'] = $this->m_objidref++;
	else
		$this->m_attr['__objid'] = $objid;
	$this->m_attr['__objtype']       = $this->m_objtype;
	$this->m_attr['__objprecomment'] = implode("",$comment);
	
	$req = "INSERT INTO ".$this->m_attr['__objtype']." (";
	
	// From v1.0: custom attributes are stored in a specific table
	//print_r($this);
	$firstattr = true;
	foreach ($this->m_attr as $attr => $value) {
		if (NagiosObject::IsCustomAttr($attr))
			continue;
		if (!isset($this->m_attr[$attr]))
			continue;
		if (!$firstattr) $req .= ",";
		$req .= "'".$attr."'";
		$firstattr = false;
	}
	$req .= ") VALUES (";
	$firstattr = true;
	foreach ($this->m_attr as $attr => $value) {
		if (NagiosObject::IsCustomAttr($attr))
			continue;
		if (!isset($this->m_attr[$attr]))
			continue;
		if (!$firstattr) $req .= ",";
		// Note: this is DB-dependent
		$value = $db->Quote($value);
		$req .= $value;
		$firstattr = false;
	}
	$req .= ")";

	return $req;
}

public function GetCustomAttrList()
{
	$attrlist = array();

	foreach ($this->m_attr as $attr => $value) {
		if (NagiosObject::IsCustomAttr($attr))
			$attrlist[] = $attr;
	}

	return $attrlist;
}

public function Update($db)
{
	global $gLogger;

	$req = "UPDATE ".$this->m_attr['__objtype']." SET ";
	
	//print_r($this);
	$firstattr = true;
	$attrdef = $this->GetDef();
	foreach ($attrdef as $attr => $dummy) {
		// From v1.0: do not save custom attributes
		if (NagiosObject::IsCustomAttr($attr)) continue;

		//print_r($attr);
		if (!$firstattr) $req .= ",";
		
		// Update parm set and with an empty origin (no overload):
		if ((!isset($this->m_attr[$attr]) || (empty($this->m_attr[$attr]) && $this->m_attr[$attr] !== "0" && $this->m_attr[$attr] !== false))
			 || 
		    (isset($this->m_origin[$attr]) && !empty($this->m_origin[$attr]))
			 ||
		    (is_array($this->m_attr[$attr]) && !count($this->m_attr[$attr]))) {
		    	$req .= "'$attr' = NULL";
		}
		else {
			$value = $this->m_attr[$attr];
			// Added fomr v0.10:
			if (is_array($value)) $value = NULL;	# abnormal
			$value = $db->Quote($value);
			$req .= "'$attr'=$value";
		}
		
		$firstattr = false;
	}
	
	$req .= " WHERE __objid = '".$this->m_attr['__objid']."'";

	return $req;
}

public function GetCreateTableRequest()
{
	$attrlist = array_merge(NagiosObject::$m_commonattrdef,$this->GetDef());
	
	$req = "CREATE TABLE ".$this->GetType()." (";
	
	$firstattr = true;
	foreach ($attrlist as $attr => $value) {
		if (!$firstattr) $req .= ",";
		$req .= "'".$attr."' ";
		$req .= OA::GetType($value['type']);
		$firstattr = false;
	}
	$req .= ")";
	
	return $req;
}

// $what can be "change" or "new":
public function CheckAttributes($db,&$attrvalues,&$add_msg,$what)
{
	//echo "CheckAttributes ";
	//print_r($attrvalues);
	$has_error = 0;
	$attrdef = $this->GetDef();
	if ($what == "change")
		$isTemplate = $this->IsTemplate();
	else
		$isTemplate = $attrvalues['name'] != "" && !$attrvalues['register'];

	// The key must be unique:
	try {
		$objlist = $db->GetObjectFromKey($this,$attrvalues);
	}
	catch (Exception $e) {
		$add_msg['name'] = "Internal Error: $e";
		return 1;
	}
	$indexes = $this->GetIndex();
		
	if (is_array($objlist)) {
		//print_r($objlist);
		// For a creation the row count must be 0
		if ($what == "new" && count($objlist) > 0) {
			if (count($indexes) == 1) {
				$add_msg[$indexes[0]] = "This value must be unique";
			}
			else {
				foreach ($indexes as $index) {
					$add_msg[$index] = "This value is part of the index which must be unique";
				}
			}
			$has_error++;
		}
		// For a change, the row count must be 0 if the new index changed
		if ($what == "change") {
			foreach ($objlist as $obj) {
				if ($obj['__objid'] != $attrvalues['__objid']) {
					foreach ($indexes as $index) {
						$add_msg[$index] = "This value must be unique";
					}
					$has_error++;
					break;
				}
			}
		}
	}
	
	// Check illegal characters presence:
	// (only indexes should be constrained ? TODO: ... not sure)
	$illegal_chars = $db->GetMainParm("illegal_object_name_chars");
	$ic = preg_quote($illegal_chars);
	foreach ($indexes as $index) {
		if (!isset($attrvalues[$index])) continue;
		if (is_array($attrvalues[$index])) {
			// remove the comma and to make the match
			$value = implode("",$attrvalues[$index]);
		}
		else {
			$value = $attrvalues[$index];
		}
		// Hack: host_name of service can contain a '*'
		if ($index == "host_name")
			$value = str_replace("*","",$value);
			
		if (preg_match("/[$ic]/",$value)) {
			$add_msg[$index] = "This field contains illegal characters";
			$has_error++;
		}
	}
	
	// Mandatory attributes may be inherited !
	// So get the inherited attributes !
	$parentattr = array();
	$inherits = false;
	if (isset($attrvalues['use']) && $attrvalues['use']) {
		try {
			// maybe the parent has its own parent ?
			$parent = $db->BuildObjectFromName($this->GetType(),$attrvalues['use']);
			$db->BuildObjectInherited($parent);
			//print_r($parent);

			$parentattr = $parent->GetAttr();
		}
		catch (Exception $e) {
			throw new Exception("CheckAttributes failed: $e");
		}
		$inherits = true;
	}

	// From v0.11 : we support multiple sets of semi-mandatory attrs:
	$mandgrp_list = array();
	$semi_mand_empty_attrs = array();
	$semi_mand_full_attrs  = array();
	$has_semi_mand_attrs = 0;
	
	$obj = ObjectFactory::Factory($attrvalues['__objtype']);

	foreach ($attrdef as $attr => $chars) {
		if (isset($add_msg[$attr]) && $add_msg[$attr] != "") 
			continue;
			
		// First we must "rebuild" some attributes from checkboxes:
		$this->LinearizeCheckBox($attr,$chars['type'],$attrvalues);
		
		// Modified from v0.10 to handle semi-mandatory attributes
		// at this time, there can be only on set of semi-mand attributes
		// that depends on each other
		if (!$isTemplate && isset($chars['flag'])) {
			if ($chars['flag'] & OA::IS_MANDATORY) {
				if (!isset($attrvalues[$attr]) || empty($attrvalues[$attr]) ||
				    (is_array($attrvalues[$attr]) && 
				     (!count($attrvalues[$attr]) || 
				      (count($attrvalues[$attr]) == 1 &&
				       empty($attrvalues[$attr][0]))))) {
					
				    // Mandatory attributes may be inherited !
					if (!$inherits || !isset($parentattr[$attr])) {

					    // From v1.0: handles implied inheritance:
					    $impliedInheritance = false;
					    $impliedlist = $obj->GetImpliedInheritance();
				        foreach ($impliedlist as $implkey => $implval) {
				            if ($attr == $implkey) {
				                $inheritedValue = $origin = "";
				                list($inheritedValue,$origin) = $db->GetInheritedValue($attrvalues,$implkey,$implval);
				                if ($inheritedValue !== "" && $origin != "") {
                                    $impliedInheritance = true;
				                    break;
				                }
				            }
				        }
                        
				        if (!$impliedInheritance) {
					       $add_msg[$attr] = "This is a mandatory field that cannot be empty";
					       $has_error++;
					       continue;
				        }
					}
				}
			}
			
			if ($chars['flag'] & OA::IS_SEMIMAND) {
				# Changed from v0.11: semi-mand attributes belong to a group of attr
				# we check the attributes within their group
				$mandgrp = $chars['mandgrp'];
				assert($mandgrp);
				$mandgrp_list[$mandgrp] = $mandgrp;

				// Semi-mandatory attributes may also be inherited.
				// We memorize the fact that at least one semi-mand attr is set
				$has_semi_mand_attrs = 1;

				if (!isset($attrvalues[$attr]) || empty($attrvalues[$attr]) ||
				    (is_array($attrvalues[$attr]) && 
				     (!count($attrvalues[$attr]) || 
				      (count($attrvalues[$attr]) == 1 &&
				       empty($attrvalues[$attr][0]))))) {
					// Mandatory attributes may be inherited !
					if (!$inherits || !isset($parentattr[$attr])) {
					    
					    // From v1.0: handles implied inheritance:
					    $impliedInheritance = false;
					    $impliedlist = $obj->GetImpliedInheritance();
				        foreach ($impliedlist as $implkey => $implval) {
				            if ($attr == $implkey) {
				                $inheritedValue = $origin = "";
				                list($inheritedValue,$origin) = $db->GetInheritedValue($attrvalues,$implkey,$implval);
				                if ($inheritedValue !== "" && $origin != "") {
                                    $impliedInheritance = true;
				                    break;
				                }
				            }
				        }

				        if (!$impliedInheritance) {
    						$semi_mand_empty_attrs[$mandgrp][] = $attr;
    						continue;
				        }
					}
				}
				$semi_mand_full_attrs[$mandgrp][] = $attr;
			}
		}

		if ($isTemplate) {
			if ($attr == "name" && $attrvalues['name'] == "") {
				$add_msg[$attr] = "This is a mandatory field that cannot be empty";
				$has_error++;
				continue;
			}
		}
		
		// Empty and not mandatory attributes are ignored:
		if ((!isset($chars['flag']) || empty($chars['flag'])) &&
		    (!isset($attrvalues[$attr]) || empty($attrvalues[$attr])))
			continue;
		
		switch ($chars['type']) {
		
		case OA::SYMBOL_T:
			// Test uniqueness of template name:
			if ($attr != 'name') break;
			if (!isset($attrvalues['name']) || $attrvalues['name'] == "") break;
			try {
				$templist = $db->GetTemplateList($attrvalues['__objtype']);
				if (!is_array($templist)) break;
				foreach ($templist as $templ) {
					if ($templ['name'] == $attrvalues['name'] &&
					    $templ['__objid'] != $attrvalues['__objid']) {
						$add_msg[$attr] = "The template name must be unique";
						$has_error++;
					}
				}
			}
			catch (Exception $e) {
				$add_msg['name'] = "Internal Error: ".$e->getMessage();
				return;
			}
			break;
		
		case OA::COMMAND_T:
			// TODO: to be tested
			// looks like "check_ping!parm1!parm2...."
			$parts = explode("!",$attrvalues[$attr]);
			if (is_array($parts) && !empty($parts[0])) {
				$has_error += $this->CheckRefName($db,"Command",$attr,$parts[0],$add_msg);
			}
			break;
			
		case OA::HOST_T:
			// Note: not useful - test done by the GUI
			$has_error += $this->CheckRefName($db,"Host",$attr,$attrvalues[$attr],$add_msg);
			break;
			
		case OA::SRVDESC_T:
			// not useful - test done by the GUI
			$has_error += $this->CheckRefName($db,"Service",$attr,$attrvalues[$attr],$add_msg);
			break;
			
		case OA::HOSTSRVLIST_T:
			// must be a list of comma separated host,service:
			// Note: the GUI now assumes this list is valid
			$parts = explode(",",$attrvalues[$attr]);
			if (count($parts) % 2) {
				$add_msg[$attr] = "The list must have an even count of elements: host,service,...";
				$has_error++;
				break;
			}
			
			try {
				$servlist = $db->GetObjectOfType("Service");
			}
			catch (Exception $e) {
				$add_msg[$attr] = "Internal Error: ".$e->getMessage();
				$has_error++;
				break;
			}

			for ( $cnt = 0 ; $cnt <count($parts) ; $cnt += 2) {
				$found = false;
				foreach ($servlist as $serv) {
					if ($serv['host_name'] == $parts[$cnt] &&
					    $serv['service_description'] == $parts[$cnt+1]) {
						$found = true;
						break;
					}
				}
				if (!$found) {	
					$add_msg[$attr] = "This is not a valid couple Host/Service";
					$has_error++;
				}						
			}
			break;
			
		case OA::TIMERANGELIST_T:
			$ranges = explode(",",$attrvalues[$attr]);
			if (!is_array($ranges)) break;
			foreach ($ranges as $range) {
				if (!preg_match('/(([01][0-9])|2[0-4]):[0-5][0-9]-(([01][0-9])|2[0-4]):[0-5][0-9]/',$range)) {
					$add_msg[$attr] = "This is not a valid TimePeriod value list";
					$has_error++;
				}
			}
			break;
		}
	}
		
	// Fixed from v1.0a: this test was made in the former loop and thus some attrs were not handled
	foreach ($attrdef as $attr => $chars) {
		// If the attribute is an array, we must linearize it
		// the GUI assumes the selected elements are valid
		// Added from v0.10: if the array is empty, set a blank value
		if (isset($attrvalues[$attr]) && is_array($attrvalues[$attr])) {
			if (!count($attrvalues[$attr]))
				$attrvalues[$attr] = "";
			else
				$attrvalues[$attr] = implode(",",$attrvalues[$attr]);
		}
	}
	
	// From v0.10 : check at least one of the semi-mand attr is set
	// From v0.11 : we support multiple sets of semi-mandatory attributes
	if ($has_semi_mand_attrs) {
		foreach ($mandgrp_list as $mandgrp) {
			if (count($semi_mand_full_attrs[$mandgrp])) {
				// at least one attr of the mandgrp set: no error
				continue;
			}
			foreach ($semi_mand_empty_attrs[$mandgrp] as $attr) {
				$add_msg[$attr] = "At least one of the semi-mandatory attributes (the \"orange\" ones) must be set";
			}
			$has_error++;
		}
	}
		
	//echo "has_error=$has_error";
	//print_r($add_msg);
	
	return $has_error;
}

private function LinearizeCheckBox($attr,$attrtype,&$attrvalues)
{
	$compact = "";
	
	switch ($attrtype) {
	
	case OA::NOTIFOPT_T:
		// notif_{d,u,r}
		$this->CompactCheckbox($compact,'notif_d',$attrvalues);
		$this->CompactCheckbox($compact,'notif_u',$attrvalues);
		$this->CompactCheckbox($compact,'notif_r',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
	
	case OA::HOSTNOTIFOPT_T:
		# option "s" is new from Nagios v3.0
		// hostnotif_{d,u,r,f,s,n}
		$this->CompactCheckbox($compact,'hostnotif_d',$attrvalues);
		$this->CompactCheckbox($compact,'hostnotif_u',$attrvalues);
		$this->CompactCheckbox($compact,'hostnotif_r',$attrvalues);
		$this->CompactCheckbox($compact,'hostnotif_f',$attrvalues);
		$this->CompactCheckbox($compact,'hostnotif_s',$attrvalues);
		$this->CompactCheckbox($compact,'hostnotif_n',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
		
	case OA::SRVNOTIFOPT_T:
		// srvnotif_{w,d,u,r,f,s,n}
		$this->CompactCheckbox($compact,'srvnotif_w',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_u',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_c',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_r',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_f',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_s',$attrvalues);
		$this->CompactCheckbox($compact,'srvnotif_n',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
	
	case OA::ESCALHOSTOPT_T:
		// hostescal_{d,u,r}
		$this->CompactCheckbox($compact,'hostescal_u',$attrvalues);
		$this->CompactCheckbox($compact,'hostescal_r',$attrvalues);
		$this->CompactCheckbox($compact,'hostescal_d',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
		
	case OA::ESCALSRVOPT_T:
		// srvescal_{w,c,u,r}
		$this->CompactCheckbox($compact,'srvescal_u',$attrvalues);
		$this->CompactCheckbox($compact,'srvescal_r',$attrvalues);
		$this->CompactCheckbox($compact,'srvescal_w',$attrvalues);
		$this->CompactCheckbox($compact,'srvescal_c',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
			
	case OA::EXECHOSTFAIL_T:
		// hostfail_{o,d,u,p,n}
		$this->CompactCheckbox($compact,'hostfail_o',$attrvalues);
		$this->CompactCheckbox($compact,'hostfail_d',$attrvalues);
		$this->CompactCheckbox($compact,'hostfail_u',$attrvalues);
		$this->CompactCheckbox($compact,'hostfail_p',$attrvalues);
		$this->CompactCheckbox($compact,'hostfail_n',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
			
	case OA::EXECSRVFAIL_T:
		// srvfail_{o,w,u,c,p,n}
		$compact = "";
		$this->CompactCheckbox($compact,'srvfail_o',$attrvalues);
		$this->CompactCheckbox($compact,'srvfail_w',$attrvalues);
		$this->CompactCheckbox($compact,'srvfail_u',$attrvalues);
		$this->CompactCheckbox($compact,'srvfail_c',$attrvalues);
		$this->CompactCheckbox($compact,'srvfail_p',$attrvalues);
		$this->CompactCheckbox($compact,'srvfail_n',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
			
	case OA::HOSTSTALKOPT_T:
		// stalk_{o,d,u}
		$this->CompactCheckbox($compact,'stalk_o',$attrvalues);
		$this->CompactCheckbox($compact,'stalk_d',$attrvalues);
		$this->CompactCheckbox($compact,'stalk_u',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
			
	case OA::SRVSTALKOPT_T:
		// stalk_{o,w,u,c}
		$this->CompactCheckbox($compact,'stalk_o',$attrvalues);
		$this->CompactCheckbox($compact,'stalk_w',$attrvalues);
		$this->CompactCheckbox($compact,'stalk_u',$attrvalues);
		$this->CompactCheckbox($compact,'stalk_c',$attrvalues);
		$attrvalues[$attr] = $compact;
		break;
	}
}

private function CompactCheckbox(&$compact,$attr,&$attrvalues)
{
	if (isset($attrvalues[$attr])) {
		if (!empty($compact)) $compact .= ",";
		$compact .= $attrvalues[$attr];
	}
}

// This function builds the list of the references for the specified
// object. It is used to know if an object can be modified or deleted:
public function GetReferences($db,&$attrvalues,&$ref)
{
	$ref = array();
	$indexes = $this->GetIndex();
	
	// It is also possible to overload this function in the "service" subclass
	if ($this->GetType() == "Service") {
		$this->GetServiceReferences($db,$attrvalues,$ref);
		return;
	}
	
	$index = $indexes[0];
	#print_r($attrvalues);	
	
	// v1.0.1: array to avoid setting references of same name multiple times:
	$reflist = array();

	// Loop on all the possible object types that refer to the referenced object:
	foreach ($this->m_refby2 as $reftype => $attrs) {
		$objlist = $db->GetObjectOfType($reftype);
		$refobj = ObjectFactory::Factory($reftype);
		$indexes2 = $refobj->GetIndex();
		#print_r($indexes2);
		
		foreach ($attrs as $attr) {
			#echo " reftype=$reftype,attr=$attr<br>";
		
			// TODO: some attributes are list or service members
			foreach ($objlist as $obj) {
				#print_r($obj);
				// Bug fixed with v0.5: distinguish template and object
				//if ($obj[$attr] == $attrvalues[$index]) {
				$cmpvalue = ($attrvalues['name']) ? $attrvalues['name'] : $attrvalues[$index];
				#echo "<br>cmpvalue=$cmpvalue, obj[attr]=".$obj[$attr]."<br>";

				// Bug fixed with v1.0.3: some values are a comma-separated list:
                                $members = explode(",",$obj[$attr]);

				// Bug fixed with v1.0.1: check_commands may end with a parm list "!...!..."
				$pos = strpos($obj[$attr],"!");
                                if ($obj[$attr] == $cmpvalue || ($pos && substr($obj[$attr],0,$pos) == $cmpvalue) ||
                                    in_array($cmpvalue,$members)) {
				//if ($obj[$attr] == $cmpvalue) {
					$firstindex = true;
					$output = "";
					foreach ($indexes2 as $idx2) {
						// Test added from v1.0.1: because of semimandindex, some index may be empty
						#echo "idx2=$idx2, obj[idx2]=".$obj[$idx2].",obj['name']=".$obj['name']."<br>\n";
						if (!$obj[$idx2]) {
							// but template have their "index" attr empty: use the 'name' instead:
							if ($obj['name'] && !in_array($obj['name'],$reflist)) {
								if (!$firstindex) $output .= "/";
								$output .= $obj['name'];
								$firstindex = false;
								array_push($reflist,$obj['name']);
							}
							continue;
						}

						# bug fixed from v1.0.3: parms were in the wrong order !
						if (!in_array($obj['name'],$reflist)) {
							if (!$firstindex) $output .= "/";
							$output .= $obj[$idx2];
							$firstindex = false;
							array_push($reflist,$obj['name']);
						}
					}
					if ($output)
						$ref[] = "Referenced by $reftype: $output";
				}
			}
		}
	}

	// Added from v1.0.1: special cas for Host which can have parents
	if ($this->GetType() == "Host") {
		$objlist = $db->GetObjectOfType("Host");
		foreach ($objlist as $obj) {
			$parentlist = explode(",",$obj['parents']);
			foreach ($parentlist as $parent) {
				if ($parent == $attrvalues['host_name']) {
					$ref[] = "Referenced as a Parent by Host ".$obj['host_name'];
				}
			}
		}
	}
}

// Added from v1.0.1
private function GetServiceReferences($db,&$attrvalues,&$ref)
{
	//$ref[] = "not yet implemented";

	// Examine the "members" attribute of the ServiceGroup:
	// members = h1,s1,h2,s2,...
	$objlist = $db->GetObjectOfType("ServiceGroup");
	foreach ($objlist as $obj) {
		$memberlist = explode(",",$obj['members']);
		for ( $i = 0 ; $i < count($memberlist) ; $i += 2) {
			if ($attrvalues['host_name'] == $memberlist[$i] &&
			    $attrvalues['service_description'] == $memberlist[$i+1]) {
			    	$ref[] = "Referenced by ServiceGroup: ".$obj['servicegroup_name'];
			}
		}
	}

	// Examine the ServiceDependency:
	$objlist = $db->GetObjectOfType("ServiceDependency");
	// Test if some services depend on this one:
	foreach ($objlist as $obj) {
		if ($attrvalues['host_name'] == $obj['host_name'] &&
		    $attrvalues['service_description'] == $obj['service_description']) {
			$ref[] = "Referenced by ServiceDependency: ".$obj['dependent_host_name']."/".$obj['dependent_service_description'];
		}
	}

	// Test if the current service depends on other service
	// TODO: also take dependent_hostgroup_name in account
	foreach ($objlist as $obj) {
		if ($attrvalues['host_name'] == $obj['dependent_host_name'] &&
		    $attrvalues['service_description'] == $obj['dependent_service_description']) {
			$ref[] = "Referenced by ServiceDependency: ".$obj['host_name']."/".$obj['service_description'];
		}
	}

	// Examine the ServiceEscalation:
	// TODO: hostgroug_name not take in account
	$objlist = $db->GetObjectOfType("ServiceEscalation");
	foreach ($objlist as $obj) {
		if ($attrvalues['host_name'] == $obj['host_name'] &&
		    $attrvalues['service_description'] == $obj['service_description']) {
			$ref[] = "Referenced by ServiceEscalation: ".$obj['host_name']."/".$obj['service_description'];
		}
	}

	// Examine the ServiceExtInfo:
	$objlist = $db->GetObjectOfType("ServiceExtInfo");
	foreach ($objlist as $obj) {
		if ($attrvalues['host_name'] == $obj['host_name'] &&
		    $attrvalues['service_description'] == $obj['service_description']) {
			$ref[] = "Referenced by ServiceExtInfo: ".$obj['host_name']."/".$obj['service_description'];
		}
	}
}

private function CheckRefName($db,$objtype,$attr,$value,&$add_msg)
{
	try {
		$objlist = $db->GetObjectOfType($objtype);
	}
	catch (Exception $e) {
		$add_msg[$attr] = "Internal Error: ".$e->getMessage();
		return 1;
	}
	
	if (!is_array($objlist) || count($objlist) == 0) return 0;
	$refobj = ObjectFactory::Factory($objtype);
	$indexes = $refobj->GetIndex();
	$index = $indexes[0];

	foreach ($objlist as $obj) {
		if ($obj[$index] == $value) return 0;
	}
	
	$add_msg[$attr] = "This is not a valid $objtype name";
	
	return 1;
}

// TODO: unuseful: Function not yet called:
private function CheckList($db,$objtype,$attr,$value,$ref_attr,&$add_msg)
{
	// Remove spaces
	$value = str_replace(" ","",$value);
	$items = explode(",",$value);
	
	// There must be at least ONE item
	if (!$items || count($items) == 0) {
		// Note: MANDATORY values have already been checked
		$add_msg[$attr] = "List cannot be empty";
		return 1;
	}
	
	try {
		$objlist = $db->GetObjectOfType($objtype);
	}
	catch (Exception $e) {
		$add_msg[$attr] = "Internal Error: ".$e->getMessage();
		return 1;
	}
	
	// Each item must be a valid objtype name: 
	foreach ($items as $item) {
		if ($item == "") {
			$add_msg[$attr] = "List Item is empty";
			return 1;
		}
		
		$found = false;
		foreach ($objlist as $obj) {
			if ($obj[$ref_attr] == $item) {
				$found = true;
				break;
			}
		}
		
		if (!$found) {
			$add_msg[$attr] = "'$item' is not a valid $objtype";
			return 1;
		}
	}
	
	return 0;
}

protected $m_attr;	// attributes
protected $m_attrdef2;	// attributes formal definition
protected $m_refby2;	// references definition

static protected $m_commonattrdef = array (

	"__objtype"		=> array (
		"type"	=> OA::OBJTYPE_T,
		"flag"	=> OA::NOT_DUMPDISP,
	),
	"__objid"		=> array ( 
		"type"	=> OA::OBJID_T, 
		"flag"	=> OA::NOT_DUMPDISP,
	),
	"__objprecomment"	=> array (
		"type"	=> OA::COMMENT_T,
		"flag" 	=> OA::NOT_DUMPDISP, 
	),
);

};
?>
