<?php
/*
 * Note: when changing the order or the definition of object attributes
 *       take care at the order of the indexes (for GetIndex or CheckRefName functions)
 */

class OA {

//
// Possible attribute types with their definition:
//
const CFGFILE_T 	= 1;
const OBJTYPE_T 	= 2;
const OBJID_T   	= 3;
const SYMBOL_T		= 4;
const IPADDR_T  	= 5;
const BOOL_T		= 6;
const NUMERIC_T 	= 7;
const TIMEPERIOD_T 	= 8;
const HOST_T		= 9;
const NOTIFOPT_T	= 10;
const COMMAND_T		= 11;
const PERCENT_T		= 12;
const HOSTSTALKOPT_T	= 13;
const HOSTLIST_T	= 14;
const SRVDESC_T		= 15;
const CONTACTGRP_T	= 16;
const COMMENT_T		= 17;
const HOSTTEMPL_T	= 18;
const TIMERANGELIST_T	= 19;
const HOSTGRPLIST_T	= 20;
const COMMANDLIST_T	= 21;
const HOSTNOTIFOPT_T	= 22;
const SRVNOTIFOPT_T	= 23;
const CONTACTLIST_T	= 24;
const SERVICELIST_T	= 25;
const EXECSRVFAIL_T	= 26;
const CONTACTGRPLIST_T	= 27;
const ESCALSRVOPT_T	= 28;
const EXECHOSTFAIL_T	= 29;
const ESCALHOSTOPT_T	= 30;
const SERVTEMPL_T	= 31;
const TIMETEMPL_T	= 32;
const HOSTGRPTEMPL_T	= 33;
const HOSTESCTEMPL_T	= 34;
const HOSTDEPTEMPL_T	= 35;
const SRVESCTEMPL_T	= 36;
const SRVDEPTEMPL_T	= 37;
const SRVGRPTEMPL_T	= 38;
const CONTACTTEMPL_T	= 39;
const CONTGRPTEMPL_T	= 40;
const CMDTEMPL_T	= 41;
const EXTHOSTTEMPL_T	= 42;
const EXTSRVTEMPL_T	= 43;
const HOSTSRVLIST_T	= 44;
const COORD2D_T		= 45;
const COORD3D_T		= 46;
const IMAGE_T		= 47;
const SRVSTALKOPT_T	= 48;
const TIMEPERIODLIST_T	= 49;	# added from nagios v3.0

// Note: it could be handy to add some flags, for example a flag
//       to distinguish list and single values. This would offer a more
//       robust control of the values and constraints (see GetReferences())

const IS_MANDATORY	= 0x1;	// mandatory attribute, except for template
const NOT_DUMPABLE	= 0x2;
const NOT_DUMPMAND	= 0x3;	// see below (PHP syntax problem)
const IS_INDEX		= 0x4;	
const IS_MANDINDEX	= 0x5;  // usually, it is the name
				// hack: it seems not possible to initialize a const
				// with a constant expression ! so we consider
				// an INDEX is MANDATORY !
const NOT_DISPLAYABLE	= 0x8;
const NOT_DUMPDISP	= 0xA;	// same hack (0x8 + 0x2)
const DONT_INHERIT	= 0x10; // 'register' attribute is never inherited
const SUPPORT_STAR	= 0x20; // allow the "*" wildcard in the value
const IS_MANDSTAR	= 0x21;
const IS_MANDINDEXSTAR	= 0x25;

// Added from v0.10
const IS_SEMIMAND	= 0x40;	// sometimes one of two attr must be set (ex: host_name or hostgroup_name for services)
const IS_SEMIMANDINDEXSTAR = 0x64;
const IS_SEMIMANDINDEX  = 0x44;


static public $attrtype = array (
	OA::CFGFILE_T		=> array ( "VARCHAR(255)" ),
	OA::OBJTYPE_T		=> array ( "CHAR(48)" ),
	OA::OBJID_T		=> array ( "INT" ),
	OA::SYMBOL_T		=> array ( "VARCHAR(255)" ),
	OA::IPADDR_T		=> array ( "CHAR(16)" ),
	OA::BOOL_T		=> array ( "INT" ),
	OA::NUMERIC_T		=> array ( "INT" ),
	OA::TIMEPERIOD_T	=> array ( "VARCHAR(255)" ),
	OA::TIMEPERIODLIST_T	=> array ( "VARCHAR(255)" ),
	OA::HOST_T		=> array ( "VARCHAR(255)" ),
	OA::NOTIFOPT_T		=> array ( "CHAR(16)" ),
	OA::COMMAND_T		=> array ( "VARCHAR(255)" ),
	OA::PERCENT_T		=> array ( "FLOAT" ),
	OA::HOSTSTALKOPT_T	=> array ( "CHAR(16)" ),
	OA::HOSTLIST_T		=> array ( "TEXT" ),
	OA::SRVDESC_T		=> array ( "VARCHAR(255)" ),
	OA::CONTACTGRP_T	=> array ( "VARCHAR(255)" ),
	OA::COMMENT_T		=> array ( "TEXT" ),
	OA::HOSTTEMPL_T		=> array ( "TEXT" ),
	OA::TIMERANGELIST_T	=> array ( "VARCHAR(255)" ),
	OA::HOSTGRPLIST_T	=> array ( "VARCHAR(255)" ),
	OA::COMMANDLIST_T	=> array ( "TEXT" ),
	OA::HOSTNOTIFOPT_T	=> array ( "CHAR(16)" ),
	OA::SRVNOTIFOPT_T	=> array ( "CHAR(16)" ),
	OA::CONTACTLIST_T	=> array ( "TEXT" ),
	OA::SERVICELIST_T	=> array ( "TEXT" ),
	OA::EXECSRVFAIL_T	=> array ( "CHAR(16)" ),
	OA::CONTACTGRPLIST_T	=> array ( "VARCHAR(255)" ),
	OA::ESCALSRVOPT_T	=> array ( "CHAR(16)" ),
	OA::EXECHOSTFAIL_T	=> array ( "CHAR(16)" ),
	OA::ESCALHOSTOPT_T	=> array ( "CHAR(16)" ),
	OA::SERVTEMPL_T		=> array ( "VARCHAR(255)" ),
	OA::TIMETEMPL_T		=> array ( "VARCHAR(255)" ),
	OA::HOSTGRPTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::HOSTESCTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::HOSTDEPTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::SRVESCTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::SRVDEPTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::SRVGRPTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::CONTACTTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::CONTGRPTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::CMDTEMPL_T		=> array ( "VARCHAR(255)" ),
	OA::EXTHOSTTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::EXTSRVTEMPL_T	=> array ( "VARCHAR(255)" ),
	OA::HOSTSRVLIST_T	=> array ( "VARCHAR(255)" ),
	OA::COORD2D_T		=> array ( "CHAR(16)" ),
	OA::COORD3D_T		=> array ( "CHAR(16)" ),
	OA::IMAGE_T		=> array ( "VARCHAR(255)" ),
	OA::SRVSTALKOPT_T	=> array ( "CHAR(16)" ),
);

static public function GetType($oa)
{
	$type = OA::$attrtype[$oa];
	assert($type);
	return $type[0];
}

};
?>
