<?php
include_once "actionbase.inc.php";
include_once "displayobject.inc.php";
include_once "rrdgraph.inc.php";

class Action extends ActionBase
{
public $m_sigs = array (
		'__objtype' => array (
				'req' => true, 
				'type' => 'string',
				'filter' => 'addslashes'
			 	),
		'__objid'	  => array (
				'req' => true, 
				'type' => 'string',
				'filter' => 'onlydigit'
				),
		'Discard' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'addslashes'
			 	),
		'detail'	  => array (
				'req' => false, 
				'type' => 'boolean',
				'filter' => 'boolean'
				),
	);

public function Run()
{
	global $gDB;
	global $gLogger;
	global $gMSG;
	global $gErrorMSG;
	global $gShowDetail;
	
	//phpinfo();
	
	$objtype = $_REQUEST['__objtype'];
	$objid   = $_REQUEST['__objid'];
	$showDetail = isset($_REQUEST['detail']) ? $_REQUEST['detail'] : $gShowDetail;
	
	if (isset($_REQUEST['Discard']) && $_REQUEST['Discard']) {
		$gMSG = "Changes discarded";
		DisplayContinueButton($this,$objtype);
		return;
	}
	
	if (!ObjectFactory::IsValidObjectType($objtype)) {
		$gErrorMSG = "Internal Error: Object Type '$objtype' not supported !";
		return;
	}

	try {
		$ze_obj = $gDB->BuildObject($objtype,$objid);
		$gDB->BuildObjectInherited($ze_obj);
		//print_r($ze_obj);
	}
	catch (Exception $e) {
		$gErrorMSG = "Internal Error: ".$e->getMessage();
		return;
	}
	
	$attrdef = $ze_obj->GetDef();
	$attrval = $ze_obj->GetAttr();
	$has_error = 0;
	$add_msg = array();

	// Added on the 04 Apr 2008 (v0.11) to take care of magic_quotes_gpc
	// moved from v1.0.3 (it was placed after the index checking)
	$this->NormalizeValues($_POST);

	// Check the modification consistency:	
	// It is not possible to change the "index" value of referenced objects:	
	$ref = array();
	try {
		$ze_obj->GetReferences($gDB,$attrval,$ref);
	}
	catch (Exception $e) {
		$gErrorMSG = "Internal Error: ".$e->getMessage();
		return;
	}
	
	if (count($ref)) {
		$indexes = $ze_obj->GetIndex();
		foreach ($indexes as $index) {
			if (isset($_POST[$index]) && isset($attrval[$index])) {
				if (!is_array($_POST[$index]) && $_POST[$index] != $attrval[$index]) {
					$has_error++;
					$add_msg[$index] = "It is not possible to change this field, because this object is referenced by other objects";
				}
				// from v1.0.3: handle case of array index (for services)
				if (is_array($_POST[$index])) {
					$val = implode(",",$_POST[$index]);
					if ($val != $attrval[$index]) {
						$has_error++;
						$add_msg[$index] = "It is not possible to change this field, because this object is referenced by other objects";
					}
                                }
			}
		}
	}
	
	// Fixed on the 12/12/07 (since v0.8)
	//$has_error = $ze_obj->CheckAttributes($gDB,$_POST,$add_msg,"change");
	$has_error += $ze_obj->CheckAttributes($gDB,$_POST,$add_msg,"change");
	
	if ($has_error) { 
		if ($has_error > 1)
			$gErrorMSG = "Invalid values makes the Updating impossible !"; 
		else	
			$gErrorMSG = "Invalid value makes the Updating impossible !"; 
		DisplayObject("Edit",$this,$objtype,$objid,$ze_obj,$add_msg,$showDetail);
		return;
	}
	
	// Apply the modifications:
	//phpinfo();
	foreach ($attrdef as $attr => $chars) {
		// Unset all desired attributes:
		if (isset($_POST["unset_$attr"])) {
			$gLogger->Debug("Unset $attr");
			$ze_obj->UnsetAttr($attr);
		}
		else if (isset($_POST["overload_$attr"])) {
			// Overload all desired attributes:
			$gLogger->Debug("Overload $attr");
			$ze_obj->AddAttr($attr,$_POST[$attr]);
			$ze_obj->SetOrigin($attr,"");
			$gLogger->Debug(" origin reset ");
		}
		else if ($ze_obj->GetOrigin($attr) == "") {
			ConsolidateValues($_POST,$attr,$chars);
			if (isset($_POST[$attr])) {
				$newvalue = $_POST[$attr];
				$ze_obj->AddAttr($attr,$newvalue);
				$gLogger->Debug(" $attr has a new value: $newvalue ");
			}
		}
		else {
			$gLogger->Debug(" $attr is overloaded ");
		}
	}

	// From v1.0: handle custom atributes:
	foreach ($attrval as $attr => $value) {
		if (NagiosObject::IsCustomAttr($attr)) {
		    // Value changed or value deleted ?
            if (isset($_POST["del_$attr"])) {
	   		    $gLogger->Debug("Unset $attr");
    			$ze_obj->UnsetAttr($attr);
            }
            else {
    		    $newvalue = $_POST[$attr];
	   	        $ze_obj->AddAttr($attr,$newvalue);
		        $gLogger->Debug(" $attr has a new value: $newvalue ");
            }
		}
	}
	
	// Do we have to create a new custom attribute ?
	if (isset($_POST['but_custattr'])) {
	    /* Add a _ if necessary */
	    $newattr = $_POST['new_custattr'];
	    if ($newattr[0] != '_') $newattr = "_".$newattr;
	    $newvalue = $_POST['new_custvalue'];
        $ze_obj->AddAttr($newattr,$newvalue);
        $gLogger->Debug(" $attr has a new value: $newvalue ");
	}
	
	//print_r($ze_obj);
	
	try {
		$gDB->Update($ze_obj);

		# update the service extended info if RRDTool enabled
		RRDToolSynchronize($ze_obj);
	}
	catch (Exception $e) {
		$gErrorMSG = "Internal Error: ".$e->getMessage();
		return;
	}
	
	$gMSG = "$objtype updated";
	
	DisplayContinueButton($this,$objtype);
}

};
?>
