<?php
//
// This script check whether or not we must rescan the text files
// to rebuild the internal DB
//
// From v1.0.1: add test on cgi.cfg
//
include_once "../conf/log.class.php";
include_once "../conf/config.class.php";
include_once "../conf/database.class.php";
include_once "../conf/daemon.class.php";

$gLogger->SetDebug($gDebugMode);

// Nagios must be running (at this point $gDB is not set):
$conf = new Config(NULL);

/*	
if (!Daemon::IsRunning()) {
	$gErrorMSG = "Nagios is not running - please start it";
	DisplayConfError($gMustStartURL,"Retest");
}
*/

// Get the Main Configuration File:
if (!empty($gMainConfigFileName)) {
	$ret = $conf->SetNagiosMainFileName($gMainConfigFileName);
	if (!$ret) {
		$gErrorMSG = "Main Configuration File '$gMainConfigFileName' does not exist or is not readable";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}
else {	
	$mainfile = $conf->GetNagiosMainFileName();
	if ($mainfile == "") {
		$gErrorMSG = "Unable to find the Main Configuration Filename. Please, update 'config.inc.php' file";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}

// Get the CGI Configuration File:
if (!empty($gCGIConfigFileName)) {
	$ret = $conf->SetNagiosCGIFileName($gCGIConfigFileName);
	if (!$ret) {
		$gErrorMSG = "CGI Configuration File '$gCGIConfigFileName' does not exist or is not readable";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}
else {	
	$mainfile = $conf->GetNagiosCGIFileName();
	if ($mainfile == "") {
		$gErrorMSG = "Unable to find the CGI Configuration Filename. Please, update 'config.inc.php' file";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}

// Check if we must rebuild:
if ($action == "Rebuild") {
	return;
}

// Check the current real configuration to check if we need to 
// rebuild the internal Database:
// If the internal database does not exist, we must rebuild it:
if (!@file_exists($gDBFile)) {
	$gMSG = "The internal database '$gDBFile' does not exist. We must rebuild it.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}

try {
	$gDB = new Database($gDBFile);
}
catch (Exception $e) {
	$gErrorMSG = "Database error: ".$e->getMessage();
	DisplayConfError($gMustRebuildURL,"Rebuild");
}

$conf->SetDB($gDB);
	
// If the internal database is older than the configuration file: rebuild it
$time_nagios = @filemtime($conf->GetNagiosMainFileName());
$time_db     = @filemtime($gDBFile);
if ($time_nagios > $time_db) {
	$gMSG = "Nagios Main Configuration File is newer than the internal Database.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}
$time_nagios = @filemtime($conf->GetNagiosCGIFileName());
if ($time_nagios > $time_db) {
	$gMSG = "Nagios CGI Configuration File is newer than the internal Database.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}

// get the nagios main config file name and check this file is older than the DB
try {
	$filelist = $gDB->GetConfigFileList();
}
catch (Exception $e) {
	$gMSG = "Could not get the list of Configuration Files from '$gDBFile' Database.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}
foreach ($filelist as $file) {
	$mtime = @filemtime($file['filename']);
	if ($mtime > $time_db) {
		$gMSG = "Nagios Configuration File ${file['filename']} is newer than the internal Database.";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}

// Added from v0.11: does the same test with directories:
try {
	$dirlist = $gDB->GetMainDirList();
}
catch (Exception $e) {
	$gMSG = "Could not get the list of Configuration Directories from '$gDBFile' Database.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}
foreach ($dirlist as $dir) {
	$mtime = @filemtime($dir['dirname']);
	if ($mtime > $time_db) {
		$gMSG = "Nagios Configuration Directory '${dir['dirname']} is newer than the internal Database.";
		DisplayConfError($gMustRebuildURL,"Rebuild");
	}
}

// Check if the DB has been modified and not still saved:
try {
	$result = $gDB->GetInternalParm("ConfModified");
	$gShouldSaveConfig = $result[0]['value'];
	$result = $gDB->GetInternalParm("ShouldRestart");
	$gShouldRestart = $result[0]['value'];
}
catch (Exception $e) {
	$gMSG = "Could not get the value of 'ConfModified' internal parameter.";
	DisplayConfError($gMustRebuildURL,"Rebuild");
}

// 3rd test: parse the nagios config file to get the list of the other files
//	     these files must also be older thant the DB
// this test is not done, you must rebuild the DB to make sure of this.
// but normally, if a new file has been added to Nagios Conf, the main file has
// been touched and the rebuilt will happen.

function DisplayConfError($page,$action)
{
	include "header.inc.php";
	include "$page";
	include "footer.inc.php";
	die;
}
?>
