<?php
#
# Interface with RRDTool
#
include_once "config.inc.php";

function GetRRDFileName($host,$service)
{
	return $GLOBALS['gRRDGraphDir']."/$host/$service.rrd";
}

function GetDSName($name)
{
        return ereg_replace("[^[:alnum:]_]","",$name);
}

function IsaRRDCompatibleService($srv)
{
	return ($srv == "check_tcp" || $srv == "check_http" || $srv == "check_ping" ||
		$srv == "check_dns" || $srv == "check_load" ||
		$srv == "check_procs" || $srv == "check_users");
}

function RRDToolSynchronize($obj)
{
	global $gDB;
	global $gLogger;

	$gLogger->Debug("RRDToolSynchronize: ".$obj->GetType());

	if (!$GLOBALS['gMakeRRDGraph']) return;

	// RRDTool graphs are made for Service only
	if ($obj->GetType() != "Service") return;

	// ...and for Services that use specific check_commands:
	// Note: if you want to add more check commands, you must also
	//       change scripts/process-service-perfdata.php script
	$attr = $obj->GetAttr();
	$command = $attr['check_command'];	// syntax ->GetAttr()['..'] not accepted

	// Get the extended info object that may be associated with this service:
	$objtype = "ServiceExtInfo";
	$extinfo = ObjectFactory::Factory($objtype);

	try {
		$objlist = $gDB->GetObjectFromKey($extinfo,$attr);
		$extobj = $objlist[0];
	}
	catch (Exception $e) {
		//$gLogger->Log("GetObjectFromKey() failed in RRDToolSynchronize(): ".$e->getMessage());
		$extobj = NULL;
		return;
	}

	$gLogger->Debug("GetObjectFromKey succeeded");

	// fixed from v1.0.4 (split instead of explode)
	$parts = explode("!",$command);
	$gLogger->Debug("Service is compatible ? ".$parts[0]);
	if (!IsaRRDCompatibleService($parts[0])) {
		// Suppress the extended info notes if it was present:
		//if ($objlist && is_array($objlist) && count($objlist) == 1) {
		if ($extobj) {
			$gLogger->Debug("Clean the existing Service Extended Info");

			try {
				$ze_obj = $gDB->BuildObject($objtype,$objlist[0]['__objid']);
			}
			catch (Exception $e) {
				$gLogger->Log("BuildObject() failed in RRDToolSynchronize(): ".$e->getMessage());
				return;
			}

			$ze_obj->AddAttr('notes_url',"");

			try {
				$gDB->Update($ze_obj);
			}
			catch (Exception $e) {
				$gLogger->Log("Update() failed in RRDToolSynchronize(): ".$e->getMessage());
				return;
			}
		}

		$gLogger->Debug("Clean the Service");
		$obj->AddAttr('notes_url','');
		$obj->AddAttr('process_perf_data','0');
		try {
			$gDB->Update($obj);
		}
		catch (Exception $e) {
			$gLogger->Log("Update() failed in RRDToolSynchronize(): ".$e->getMessage());
			return;
		}
		return;
	}

	$gLogger->Debug("Service is compatible: ".$parts[0]);

	// Update the "notes_url" field of the Service Extended Info:

	// Changed from v1.0.1: "notes_url" attribute is now part of Host & Service description
	//if (!$objlist || !is_array($objlist) || count($objlist) == 0) {
	if (0) {
		$gLogger->Debug("Create a new Service Extended Info");

		$extinfo->AddAttr('notes_url',"/eznag/index.php?action=displaygraph&host=${attr['host_name']}&service=${attr['service_description']}");

		// Create a new service extended info:
		$extinfo->AddAttr('host_name',$attr['host_name']);
		$extinfo->AddAttr('service_description',$attr['service_description']);
		$extinfo->AddAttr('notes_url',"/eznag/index.php?action=displaygraph&host=${attr['host_name']}&service=${attr['service_description']}");
		$extinfo->AddAttr('__cfgfile',$attr['__cfgfile']);
		# TODO: could use a template ?

		// Get a new ID:
		try {
			$objid = $gDB->GetMaxObjID();
			$extinfo->AddAttr("__objid",$objid);
		}
		catch (Exception $e) {
			$gLogger->Log("GetMaxObjID() failed in RRDToolSynchronize(): ".$e->getMessage());
			return;
		}
		
		try {
			$comments = array();
			$comments[] = "# Created by eznag!\n";
			$gDB->Store($extinfo,$objid,$comments,$objid); 
		}
		catch (Exception $e) {
			$gLogger->Log("Store() failed in RRDToolSynchronize(): ".$e->getMessage());
			return;
		}
	}
	else {
		// From v1.0.4: do nothing if host_name is a list of hosts
		if (strchr($attr['host_name'],",")) {
			$gLogger->Debug("No notes_url for a service associated with multiple hosts");
		}
		else {
			// Update the existing object with the good 'notes_url':
			$obj->AddAttr('notes_url',"/eznag/index.php?action=displaygraph&host=${attr['host_name']}&service=${attr['service_description']}");
			$obj->AddAttr('process_perf_data','1');

			try {
				$gDB->Update($obj);
			}
			catch (Exception $e) {
				$gLogger->Log("Update() failed in RRDToolSynchronize(): ".$e->getMessage());
				return;
			}
		}
	}
}
?>
