<?php
include_once "actionbase.inc.php";
include_once "displayobject.inc.php";
include_once "../conf/cgiparms.class.php";

class Action extends ActionBase
{
public $m_sigs = array (
		'_newuser' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'addslashes'
			 	),
		'_deluser' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'addslashes'
			 	),
		'username' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'addslashes'
			 	),
		// added from v1.0.2
		'passwd1' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'nagiosstring'
			 	),
		'passwd2' => array (
				'req' => false, 
				'type' => 'string',
				'filter' => 'nagiosstring'
			 	),
	);

public function Run()
{
	global $gMSG;
	global $gErrorMSG;
	global $gDB;

	//phpinfo();

	// Check the action:
	if (isset($_POST['Discard'])) {
		$gMSG = "Changes discarded";
		return;
	}

	if (isset($_POST['newuser'])) {
		// A new user must be created:
		// --------------------------
		// the new name must be specified and not already in use:
		$newuser = isset($_POST['_newuser']) ? $_POST['_newuser'] : "";
		if (!$newuser) {
			$gErrorMSG = "The name of the new user is empty";
			$this->DisplayUserList();
			return;
		}

		// Get the user list:
		try {
			$userlist = $gDB->GetUsersAuth();
			//print_r($userlist);
		}
		catch (Exception $e) {
			$gMSG = "Database error: ".$e->getMessage();
			$this->DisplayUserList();
			return;
		}

		// the name must be unique:
		foreach ($userlist as $user) {
			if ($newuser == $user['user']) {
				$gErrorMSG = "User '$newuser' already exists";
				$this->DisplayUserList();
				return;
			}
		}

		try {
			$cgiparms = new CGIParms();
			$cgiopts  = $cgiparms->GetCGIOptions();
			foreach ($cgiopts as $opt) {
				$optname = substr($opt,strlen("authorized_for_"));
				$parm = "${optname}__newuser";
				if (isset($_POST[$parm]) && $_POST[$parm])
					$gDB->SetUserAuth($newuser,$opt,1);
				else
					$gDB->SetUserAuth($newuser,$opt,0);
			}
			$gDB->UsersModified();
		}
		catch (Exception $e) {
			$gMSG = "Database error: ".$e->getMessage();
			$this->DisplayUserList();
			return;
		}
	}
	elseif (isset($_REQUEST['_deluser']) && $_REQUEST['_deluser']) {
		// A user must be deleted
		// ----------------------
		try {
			$gDB->DeleteUser($_REQUEST['_deluser']);
			$gDB->UsersModified();
			$gMSG = "User '".$_REQUEST['_deluser']."' deleted";
		}
		catch (Exception $e) {
			$gMSG = "Database error: ".$e->getMessage();
			$this->DisplayUserList();
			return;
		}
	}
	elseif (isset($_POST['setpassword'])) {
		// A password must be changed or set:
		// ---------------------------------
		global $gApacheAuthFile,$gApacheAuthCommand;
		if (!isset($_POST['username']) || !$_POST['username']) {
			$gErrorMSG = "The username must not be empty";
			$this->DisplayUserList();
			return;
		}
		if (!isset($_POST['passwd1']) || !$_POST['passwd1']) {
			$gErrorMSG = "The password must not be empty";
			$this->DisplayUserList();
			return;
		}
		if (!isset($_POST['passwd2']) || !$_POST['passwd2']) {
			$gErrorMSG = "You must confirm the password";
			$this->DisplayUserList();
			return;
		}
		if ($_POST['passwd2'] != $_POST['passwd1']) {
			$gErrorMSG = "The given passwords differ !";
			$this->DisplayUserList();
			return;
		}
		// Build the command line to change the password:
		$passwd = $_POST['passwd1'];
		$username = $_POST['username'];
		$createflag = @file_exists($gApacheAuthFile) ? "" : "-c ";
		$cmd = "sudo \"$gApacheAuthCommand\" ";

		if (basename($gApacheAuthCommand) == "htdigest") {
			$cmd .= "$createflag \"$gApacheAuthFile\" Nagios \"$username\" \"$passwd\"";
		}
		else {
			$cmd .= "-b $createflag \"$gApacheAuthFile\" \"$username\" \"$passwd\"";
		}
		$lastline = @exec($cmd,$output,$status);
		if ($status) {
			$gErrorMSG = addslashes("Error while executing command (status=$status):<br>'$cmd'");
		}
	}
	elseif (isset($_POST['Apply'])) {
		// Apply modifications:
		// -------------------
		// Get the user list:
		try {
			$userlist = $gDB->GetUsersAuth();
			//print_r($userlist);
		}
		catch (Exception $e) {
			$gMSG = "Database error: ".$e->getMessage();
			$this->DisplayUserList();
			return;
		}

		// Check if their rights have changed:
		$cgiparms = new CGIParms();
		$cgiopts  = $cgiparms->GetCGIOptions();
		$adminuserlist = "";
		foreach ($userlist as $user) {
			$username = $user['user'];
			foreach ($cgiopts as $opt) {
				$optname = substr($opt,strlen("authorized_for_"));
				$parm = "${optname}_${username}";
				try {
					if (isset($_POST[$parm]) && $_POST[$parm])
						$gDB->SetUserAuth($username,$opt,1);
					else
						$gDB->SetUserAuth($username,$opt,0);
				}
				catch (Exception $e) {
					$gMSG = "Database error: ".$e->getMessage();
					$this->DisplayUserList();
					return;
				}
			}

			$parm = "isadmin_${username}";
			if (isset($_POST[$parm]) && $_POST[$parm] ) {
				if ($adminuserlist) $adminuserlist .= ",";
				$adminuserlist .= $username;
			}
		}
		try {
			$gDB->UsersModified();
		}
		catch (exception $e) {
			$gMSG = "Database error: ".$e->getMessage();
			$this->DisplayUserList();
			return;
		}

		// Update the admin users list:
		# Conf files must be writeable:
                $confname = "config.inc.php";
		if (!is_writeable($confname)) {
			$gErrorMSG = "File 'config.inc.php' is not writeable via this interface.";
			$this->DisplayUserList();
			return;
		}
                $lines = @file($confname);
		$lines = preg_replace('/^(\$gAdminUsers.*=.*")(.*)(".*)/',"\${1}".$adminuserlist.'\3',$lines);
		$this->UpdateConfFile($confname,$lines);
		$GLOBALS['gAdminUsers'] = $adminuserlist;
	}

	$this->DisplayUserList();
}

private function DisplayUserList()
{
	global $gMSG;
	global $gDB;

	// Get the user list:
	try {
		$userlist = $gDB->GetUsersAuth();
		//print_r($userlist);
	}
	catch (Exception $e) {
		$gMSG = "Database error: ".$e->getMessage();
		return;
	}

	// Get the Contact list:
	try {
		$objlist = $gDB->GetObjectOfType("Contact");
	}
	catch (Exception $e) {
		$gErrorMSG = "Internal Error: ".$e->getMessage();
		return;
	}

	// Get Apache User list:
	$apachelist = $this->GetApacheAuthUsers();

	// Get the list of Admin users:
	$adminusers = explode(',',$GLOBALS['gAdminUsers']);

	// Display the header:
	$this->Output("<div class=title>Users Authorization Management</div><hr>");
	
	// Display the User list with their rights:
	$this->Output("<table width=450><tr>");
	$this->Output("<td valign=center><span class=listhdr>Current User List:</span></td>");
	$this->Output("</tr></table>");

	$cgiparms = new CGIParms();
	$cgiopts  = $cgiparms->GetCGIOptions();

	// Bug fixed from v1.0.2: the GetPassword() function was defined for each user:
	// only the last user password could be changed !
	$htmlcontent  = "<html><body><br><div align=center>";
	$htmlcontent .= "<form action=index.php?action=userauth method=post>";
	$htmlcontent .= "<input type=hidden name=username value=\"' + user + '\">";
	$htmlcontent .= "<table><tr><td colspan=2>Set Password for User <b>' + user + '</b> :</td></tr>";
	$htmlcontent .= "<tr><td font-size=10>New Password :</td><td><input type=password name=passwd1 size=20></td></tr>";
	$htmlcontent .= "<tr><td>Confirm Password :</td><td><input type=password name=passwd2 size=20></td></tr>";
	$htmlcontent .= "<tr><td></td><td><input class=button type=submit name=setpassword value=\"Set Password\"></td></tr>";
	$htmlcontent .= "</table>";
	$htmlcontent .= "</form>";
	$htmlcontent .= "</div></body></html>";

	$this->Output("\n<script>");
	$this->Output("function GetPassword(user) {\nvar win = new Window({width:360, height:150, className:\"mac_os_x_dialog\", buttonClass:\"myButtonClass\",id:\"myDialogId\",closable:true,draggable:true,wiredDrag:true, destroyOnClose:true}); win.setHTMLContent('$htmlcontent'); win.show();");
	$this->Output("return false;\n}\n");
	$this->Output("</script>");

	$this->Output("<form action=index.php?action=userauth method=post>\n");
	$this->Output("<table>\n");
	$this->DisplayButtons();
	$this->Output("<tr class=rowhdr><th rowspan=2 align=center>Name</th><th rowspan=2 align=center>Has an Apache<br>Account ?</th><th rowspan=2 align=center>Is a<br>Contact ?</th>");
	$this->Output("<th colspan=8 align=center>Authorization for:</th><th rowspan=2 align=center colspan=2>Action</th></tr>");
	$this->Output("<tr class=rowhdr><th align=center>System<br>Information</th><th align=center>System<br>Commands</th><th align=center>Configuration<br>Information</th><th align=center>All Hosts</th><th align=center>All Hosts<br>Commands</th><th align=center>All<br>Services</th><th align=center>All Service<br>Commands</th><th>Can use<br>EZNag</th></tr>");

	$objnu = 0;

	// Display the users listed in cgi.cfg:
	foreach ($userlist as $user) {
		if (($objnu % 2) == 0)		
			$cellclass = "evenCell";
		else 
			$cellclass = "oddCell";
		$objnu++;
			
		$this->Output("<tr class=$cellclass>");
				
		$this->Output("<td align=center>");
		$this->Output($user['user']);
		$this->Output("</td>");

		// Has an Apache account ?
		$hasaccount = "No";
		foreach ($apachelist as $auser) {
			if ($auser == $user['user']) {
				$hasaccount = "Yes";
				break;
			}
		}
		$this->Output("<td align=center>$hasaccount</td>");

		// Is a contact ?
		$isacontact = "No";
		foreach ($objlist as $contact) {
			if ($user['user'] == $contact['contact_name']) {
				$isacontact = "Yes";
				break;
			}
		}
		$this->Output("<td align=center>$isacontact</td>");

		foreach ($cgiopts as $opt) {
			$optname = substr($opt,strlen("authorized_for_"));
			$this->DisplayCheckBox($user['user'],$optname,$user[$opt]);
		}

		// Is Admin ?
		$isadmin = false;
		foreach ($adminusers as $admin) {
			if ($admin == $user['user']) {
				$isadmin = true;
				break;
			}
		}
		$this->DisplayCheckBox($user['user'],'isadmin',$isadmin);

		$htmlcontent  = "<html><body><br><div align=center>";
		$htmlcontent .= "<form action=index.php?action=userauth method=post>";
		$htmlcontent .= "<input type=hidden name=username value=\"".$user['user']."\">";
		$htmlcontent .= "<table><tr><td colspan=2>Set Password for User <b>".$user['user']."</b> :</td></tr>";
		$htmlcontent .= "<tr><td font-size=10>New Password :</td><td><input type=password name=passwd1 size=20></td></tr>";
		$htmlcontent .= "<tr><td>Confirm Password :</td><td><input type=password name=passwd2 size=20></td></tr>";
		$htmlcontent .= "<tr><td></td><td><input class=button type=submit name=setpassword value=\"Set Password\"></td></tr>";
		$htmlcontent .= "</table>";
		$htmlcontent .= "</form>";
		$htmlcontent .= "</div></body></html>";

		$this->Output("<td align=center>");
		if ($hasaccount == "Yes") {
			$this->Output("<a href=\"\" onclick=\"javascript: return GetPassword('".$user['user']."');\">Change Password</a>");
		}
		else {
			$this->Output("<a href=\"\" onclick=\"javascript: return GetPassword('".$user['user']."');\">Set Password</a>");
		}
		$this->Output("</td>");

		$this->Output("<td align=center>");
		$urlenc = urlencode($user['user']);
		$this->Output("<a href=\"\" onclick=\"javascript: return ConfirmDelete('index.php?action=userauth&_deluser=$urlenc');\">Delete</a>");
		$this->Output("</td>");
				
		$this->Output("</tr>\n");

		$objnu++;
	}

	// Display the Apache users not already listed:

	// Display the "new user" form:
	if (($objnu % 2) == 0)		
		$cellclass = "evenCell";
	else 
		$cellclass = "oddCell";
	$this->Output("<tr class=$cellclass>");
	$this->Output("<td>&nbsp;<input type=text name=_newuser size=10>&nbsp;</td><td></td><td></td>");
	foreach ($cgiopts as $opt) {
		$optname = substr($opt,strlen("authorized_for_"));
		$this->DisplayCheckBox("_newuser",$optname,false,false);
	}
	$this->DisplayCheckBox("_newuser","isadmin",false,false);
	$this->Output("<td align=center colspan=2>");
	$this->Output("<input class=button type=submit name=newuser value=Create>");
	$this->Output("</td>");
	$this->Output("</tr>");

	$this->DisplayButtons();
		
	$this->Output("</table>\n");
	$this->Output("</form>\n");
}

private function DisplayCheckBox($username,$authname,$authvalue,$mustcolor=true)
{
	$chkboxname = "${authname}_${username}";
	$chkboxvalue = ($authvalue) ? "checked" : "";
	$color = "";
	if ($mustcolor) $color = ($chkboxvalue) ? "bgcolor=#7FFF00" : "bgcolor=red" ;
	$this->Output("<td align=center $color><input type=checkbox name=\"$chkboxname\" $chkboxvalue></td>");
}

private function DisplayButtons()
{
	$this->Output("<tr><td></td><td></td><td colspan=6>");
	$this->Output("<input class=button type=submit name=Apply value='Apply Changes'> ");
	$this->Output("<input class=button type=submit name=Discard value='Discard Changes'> ");
	$this->Output("</td><td align=leftcolspan=3><input type=submit class=button name=Help value=Help onClick=\"var win = new Window({className: 'mac_os_x', title: 'Documentation', top:70, left:100, width:650, height:500, url: '/nagios/docs/cgiauth.html', showEffectOptions: {duration:1.5}}); win.show(); return false;\">");
	$this->Output("</td></tr>\n");
}	

private function GetApacheAuthUSers()
{
	global $gMSG;
	global $gErrorMSG;
	global $gApacheAuthFile;

	if (!is_readable($gApacheAuthFile)) {
		$gMSG = "Could not open file '$gApacheAuthFile'";
		return false;
	}

	$userlist = array();
	$lines = @file($gApacheAuthFile);
	foreach ($lines as $line) {
		$parts = explode(':',$line);
		$userlist[] = $parts[0];
	}

	return $userlist;
}

};
?>
