<?php
# 
# This script is called by Nagios for each performance data to process
#
chdir("/software/eznag-v3-1.0.4/php/scripts");
include_once "../mmi/config.inc.php";
include_once "../mmi/rrdgraph.inc.php";
include_once "../conf/log.class.php";
include_once "../conf/database.class.php";

# Nothing to do if RRDTool is disabled:
if (!$gMakeRRDGraph) exit(0);

function CheckOption(&$opts,$opt,&$var)
{
	if (!isset($opts[$opt])) {
		$GLOBALS['gLogger']->Log("Option '$opt' not set");
		exit(1);
	}
	
	$var = $opts[$opt];
}

# Check the parameters:
$options = getopt("H:S:T:D:K:");
CheckOption($options,"H",$host);
CheckOption($options,"S",$serv);
CheckOption($options,"T",$status);
CheckOption($options,"D",$data);
CheckOption($options,"K",$check_cmd);

$gLogger->Debug("host=$host, service=$serv, status=$status, data=$data, check_cmd=$check_cmd");

# status must be OK:
if ($status != "OK") exit(1);

# data must not be empty:
if (empty($data)) exit(1);

# Skip check command not handled:
$parts = split("!",$check_cmd);
$command = $parts[0];
if (!isset($parts[0]) || empty($command)) exit(0);
if (!IsaRRDCompatibleService($command)) exit(0);

# Eliminate non alnum chars in $serv
$serv = ereg_replace("[^[:alnum:]]","",$serv);

# If RRD File does not exist, create it:
$rrdfile = GetRRDFileName($host,$serv);
$dsname = GetDSName($serv);
if (!@file_exists($rrdfile)) {
	$gLogger->Log("Create RRD Archive '$rrdfile'");

	# dirname must exist and must be writeable
	$dir = dirname($rrdfile);
	@mkdir($dir,0755,true);
	if (!is_writeable($dir)) {
		$gLogger->Log("Directory '$dir' is not writeable by nagios Process");
		exit(1);
	}

	# create the rrd file:
	$step = 300;

	switch ($command) {
	case "check_tcp":
	case "check_http":
		$dsheartbeat = 2 * $step;
		$dsstring = "DS:$dsname:GAUGE:$dsheartbeat:0:U";
		$rras = "RRA:AVERAGE:0.5:1:43800";
		break;
	}

	$cmd = "${GLOBALS['gRRDToolCommand']} create $rrdfile -s $step $dsstring $rras";
	$errstr = system($cmd,$retval);
	if ($retval) {
		$gLogger->Log("Command '$cmd' failed (errstr=$errstr)");
		exit(1);
	}
}

# Update the graph:
$DATA = "";
switch ($command) {
case "check_tcp":
	case "check_http":
	# data format:
	# time=value;value;value;value;value
	$values = split(';',$data);
	$parts  = split("=",$values[0]);
	$DATA   = Convert2Milliseconds($parts[1]);
	break;
}

$cmd = "${GLOBALS['gRRDToolCommand']} update $rrdfile -t $dsname N:$DATA >>/tmp/rrdtool.log 2>&1";
$gLogger->Debug("Update cmd='$cmd'");

# sanity check:
if (empty($DATA)) exit(1);

$errstr = system($cmd,$retval);
if ($retval) {
	$gLogger->Log("Command '$cmd' failed (errstr=$errstr)");
	exit(1);
}

exit(0);

function Convert2MilliSeconds($val)
{
	global $gLogger;

	# strip non digit chars and substitute ',' by a '.':
	$val = ereg_replace(",",".",$val);
	$val = ereg_replace("[^[:digit:]\.]","",$val);
	$gLogger->Debug("val=$val");
	return $val * 1000.0;
}

# EOF
?>
