#!/bin/bash
# ============================================================================
# 
# Get the installation parameters
#
# ----------------------------------------------------------------------------
#
# TODO: write a script to install NRPE on remote hosts
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
OUTPUTFILE=`pwd`/outfile.$$
TMPFILE=`pwd`/tmpfile.$$
TMPDIR=
DSTDIR=
NAGIOSVERS="3.0.6"
PLUGINSVERS="1.4.13"
EZNAGVERS="v3-1.1.0"
RRDTOOLVERS="1.2.30"
ZLIBVERS="1.2.2"
LIBGDVERS="2.0.33"
NAGTRAPVERS="0.1.3"
BLOSXOMVERS="0.2.4"
SNMPTTVERS="1.2"
DNXVERS="0.19a"
MRTGINTEGRATION=0
MRTGCONF=""
NAGIOSUSR=
NAGIOSGRP=
CREATENAGIOSUSER=0
CGIURL=
HTMLURL=
APACHE_PNAME=
APACHEUSR=
APACHEGRP=
APACHECONFDIR=
APACHECONFFILE=
NAGIOSCONF=
USESAMPLEFILES=0
HTPASSWD=
NEEDSRRDTOOL=1
WITHRRDTOOL=1
RRDPATH=
WITHNMAP=0
NMAPVERS="4.11"
NMAPPATH=
LIBPNGVERS="1.2.8"
FREETYPEVERS="2.1.9"
LIBARTVERS="2.3.17"
WITHLIBGD=0
WEBUSER=nagios
WEBPASSWD=nagios
NSCAVERS="2.7.2"
MCRYPTVERS="2.5.7"
NRPEVERS="2.12"
NDOUTILSVERS="1.4b7"
NAGVISVERS="1.3.2"
GRAPHVIZVERS="2.20.2"
N2RRDVERS="1.3.2"
WITHNSCA=0
NSCAPASSWD=""
WITHNRPE=0
SUDOPATH=
SUDOFILE=
PHPPATH=
AUTHTYPE=
NOSTATUSMAP=
WANTSFRENCHDOC=0
WITHNAGVIZ=0
WITHNDO=0
WITHNAGTRAP=0
WITHGRAPHVIZ=0
WITHBLOSXOM=0
WITHDNX=0
WITHN2RRD=0
NEEDSMYSQL=0
DBNAME=""
DBUSER=""
DBPWD=""
DBSRV=""
DBADMIN=""
DBADMINPWD=""
CREATEDB=0
UPDATE=
INSTALL=

CheckRoot

# ---------------------------------------------
# Added from v1.0.3u: ask for install or update:
# ---------------------------------------------
while [ "$UPDATE" != "y" -a "$UPDATE" != "n" ]; do
	echo "You can install both Nagios and EZNag, or you can update EZNag only"
	read -p "Do you choose install or update ? (i/u) " RESP
	if [ "$RESP" = "i" ]; then
		UPDATE=n
		INSTALL=y
	elif [ "$RESP" = "u" ]; then
		UPDATE=y
		INSTALL=n
	else
		echo "Please, answer 'i' or 'u'"
	fi
done

# ----------------------------------------------------------------------
# Beware: nagios plugins will not compile if current dir contains spaces:
# ----------------------------------------------------------------------
CURDIR=`pwd`
PARTS=`echo $CURDIR | cut -d' ' -f2`
if [ "$PARTS" != "$CURDIR" ]; then
	Error "The current directory must not contains spaces. Nagios Plugins will not compile."
fi

# -------------------------
# Check for essential tools:
# -------------------------
if [ "$INSTALL" = "y" ]; then
	type gcc cc g++ >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		Error "No compiler found (gcc/g++/cc) - Installation aborted"
	fi
fi

# ---------------------------------------
# Get the name of the temporary directory:
# ---------------------------------------
while [ "$TMPDIR" = "" ]; do
	read -p "Enter the name of the temporary directory to use ? [./tmp]: " TMPDIR

	if [ "$TMPDIR" = "" ]; then
		TMPDIR="./tmp"
	fi
done

# Added with v0.6:
if  [ -d "$TMPDIR" ]; then
	echo "Directory $TMPDIR already exists"
	read -p "I must clean it. Press <Enter> to continue or <Ctrl-C> to abort "
	echo "Cleaning..."
	rm -rf "$TMPDIR"/*
fi

# ------------------------
# Get the Apache user name:
# ------------------------
# 1) Guess by itself:
#
# added on v0.7.1: Apache process name can be httpd (RH), apache or apache2 (Debian)
# added on v0.7.2: Apache process name can be httpd2-prefork/worker or httpd2 (SuSE)
APACHE_PNAME=
for P in httpd apache apache2 httpd2-prefork httpd2-worker httpd2; do
	pidof $P >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		APACHE_PNAME=$P
	fi
done

# added with v0.7.2:
# if apache is not running try to determine the file name:
if [ "$APACHE_PNAME" = "" ]; then
	for P in httpd apache apache2 httpd2-prefork httpd2-worker httpd2; do
		type $P >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			APACHE_PNAME=$P
		fi
	done
fi

# if apache is not running try do determine its process name from the service script name:
if [ "$APACHE_PNAME" = "" ]; then
	for P in httpd apache apache2; do
		if [ -f /etc/init.d/$P -o -f /etc/rc.d/init.d/$P ]; then
			APACHE_PNAME=$P
		fi
	done
fi

pidof $APACHE_PNAME >/dev/null 2>&1
if [ $? -eq 0 ]; then
	IND=0
	# added on v0.6: compute the real httpd process name:
	REALNAME=`type $APACHE_PNAME`
	if  [ $? -eq 0 ]; then
		HTTPDNAME=`echo $REALNAME | cut -d' ' -f3`
	else
		HTTPDNAME="$APACHE_PNAME"
	fi
 
	ps aux | grep "/$HTTPDNAME" > "$TMPFILE"
	while read LINE; do
		# before v0.6, the test was made on the 1st line.
		# but sometimes the 1st line shows a daemon belonging
		# to root...
		if [ $IND -eq 2 ]; then
			APADFLT=`echo $LINE | cut -d" " -f1`
		fi
		IND=$((IND+1))
	done < "$TMPFILE"
fi

# 2) Try with the local database:
if [ "$APADFLT" = "" ]; then
	# added from v0.7.1 and v0.7.2: www-date (Debian), wwwuser (SuSE):
	for USER in apache httpd www-data wwwrun; do
		id -u $USER >/dev/null 2>&1
		if [ $? -eq 0 ]; then
			APADFLT=$USER
		fi
	done
fi

while [ "$APACHEUSR" = "" ]; do
	read -p "What is the user name for the Apache server ? [$APADFLT]: " APACHEUSR
	if [ "$APACHEUSR" = "" ]; then
		APACHEUSR=$APADFLT
	fi

	id -u $APACHEUSR >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		echo "This account does not exist"
		APACHEUSR=
	fi
done

# Added from v0.7.2: on SuSE and Debian, the group and the user are different:
APACHEGRP=`groups $APACHEUSR 2>/dev/null | cut -d" " -f3`

while [ "$APACHEGRP" = "" ]; do
	read -p "What is the group name for the Apache server ? " APACHEGRP
	if [ "$APACHEUSR" != "" ]; then
		groupmod $APACHEGRP >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "This group does not exist"
			APACHEGRP=
		fi
	fi
done

# Added from v0.8: choose between Basic and Digest authentication:
if [ "$INSTALL" = "y" ]; then
while [ "$AUTHTYPE" = "" ]; do
	read -p "What Apache Authentication Type do you want to protect the Web Interface ? ([digest]/basic) " AUTHTYPE
	if [ "$AUTHTYPE" = "" ]; then
		AUTHTYPE=digest
	fi
	if [ "$AUTHTYPE" != "digest" -a "$AUTHTYPE" != "basic" ]; then
		echo "Incorrect Value"
		AUTHTYPE=
	fi
done
fi

# -----------------------------------
# Get the user name for nagios daemon:
# -----------------------------------
while [ "$NAGIOSUSR" = "" ]; do
	read -p "What is the user account for nagios ? [nagios]: " NAGIOSUSR
	if [ "$NAGIOSUSR" = "" ]; then
		NAGIOSUSR="nagios"
	fi
	id $NAGIOSUSR >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		AskYesNo "Do you want to create this account ?" 
		if [ "$REP" = "y" ]; then
			CREATENAGIOSUSER=1
		fi
	fi
done
NAGIOSGRP=$NAGIOSUSR

# --------------------------------
# Get the configuration parameters:
# --------------------------------
while [ "$DSTDIR" = "" ]; do
	if [ "$UPDATE" = "y" ]; then
		read -p "What is the root directory of nagios ? [/usr/local/nagios] " DSTDIR
	else
		read -p "In which directory to do you want to install nagios ? [/usr/local/nagios] " DSTDIR
	fi
	if [ "$DSTDIR" = "" ]; then
		DSTDIR="/usr/local/nagios"
	fi

	if [ `echo "$DSTDIR" | cut -c1` != "/" ]; then
		echo "You must give an absolute pathname"
		continue
	fi
done

if [ "$INSTALL" = "y" ]; then

# Added from v1.0.1: backup the previous configuration
if [ -d "$DSTDIR/etc" ]; then
	AskYesNo "Directory $DSTDIR/etc already exists. Do you want to save it ?"
	if [ "$REP" = "y" ]; then
		mv "$DSTDIR/etc" "$DSTDIR/etc-"`date "+%y-%m-%d"`
	fi
fi

while [ "$HTMLURL" = "" ]; do
	read -p "What will be the base URL to access to the Web Interface ? [/nagios] " HTMLURL
	if [ "$HTMLURL" = "" ]; then
		HTMLURL="/nagios"
	fi
	# This directory must be readable by the Apache user
	# this will be done in the install.sh script
done

while [ "$CGIURL" = "" ]; do
	read -p "What will be the base URL to access to the CGI scripts ? [$HTMLURL/cgi-bin] " CGIURL
	if [ "$CGIURL" = "" ]; then
		CGIURL="$HTMLURL/cgi-bin"
	fi
	# This directory must be readable by the Apache user
	# this will be done in the install.sh script
done

#AskYesNo "Do you want to copy nagios configuration sample files (not recommended) ? "
#if [ "$REP" = "y" ]; then
	USESAMPLEFILES=1
#fi

# --------------------
# Apache configuration:
# --------------------

# Try to determine the localization of Apache conf file:
pidof $APACHE_PNAME >/dev/null 2>&1
if [ $? -eq 0 ]; then
	$APACHE_PNAME -V > "$TMPFILE"
	while read LINE; do
		VAL=`echo $LINE | grep HTTPD_ROOT`
		if [ "$VAL" != "" ]; then
			HTTPD_ROOT=`echo $VAL | cut -d" " -f2 | cut -d"=" -f2 | tr -d '"'`
		fi
		VAL=`echo $LINE | grep SERVER_CONFIG_FILE`
		if [ "$VAL" != "" ]; then
			SERVER_CONFIG_FILE=`echo $VAL | cut -d" " -f2 | cut -d"=" -f2 | tr -d '"'`
		fi
	done < "$TMPFILE"

	# in some case, HTTP_ROOT is empty:
	if [ "$HTTPD_ROOT" = "" ]; then
		APACHECONFDIR=${SERVER_CONFIG_FILE}
	else
		# changed with v0.7.2: sometimes SERVER_CONFIG_FILE is an aboslute file name !
		ABS=`echo $SERVER_CONFIG_FILE | cut -d/ -f1`
		if [ "$ABS" != "" ]; then
			APACHECONFDIR=${HTTPD_ROOT}/${SERVER_CONFIG_FILE}
		else
			APACHECONFDIR=${SERVER_CONFIG_FILE}
		fi
	fi
	APACHECONFFILE=`basename $APACHECONFDIR`
	APACHECONFDIR=`dirname $APACHECONFDIR`
else
	# changed with v0.6: better handling of Apache dir and conf:
	echo "$APACHE_PNAME is not running - I could not get the Apache configuration filename by myself"
fi

if [ "$APACHECONFDIR" = "" ]; then
	while [ "$APACHECONFDIR" = "" ]; do
		read -p "Enter the Directory of Apache Configuration File  [/etc/httpd/conf]: " APACHECONFDIR
		if [ "$APACHECONFDIR" = "" ]; then
			APACHECONFDIR="/etc/httpd/conf"
		fi
		if [ ! -d "$APACHECONFDIR" ]; then
			echo "This directory does not exist !"
			APACHECONFDIR=
		fi
	done

	# TODO: should ask for the name of this file...
	for F in httpd.conf apache.conf apache2.conf; do
		if [ -f $APACHECONFDIR/$F ]; then
			APACHECONFFILE=$F
		fi
	done

	if [ "$APACHECONFFILE" = "" ]; then
		Error "Could not determine Apache Configuration File Name"
	fi
fi

#echo $APACHECONFDIR $APACHECONFFILE

# Do we have to add an Include directive or does a "Include conf.d/*.conf" directive already exist ?
grep -i "^Include conf.d/" ${APACHECONFDIR}/${APACHECONFFILE} >/dev/null 2>&1
if [ $? -eq 0 ]; then
	# TODO: we should take the value of the directive...
	NAGIOSCONF=${APACHECONFDIR}/../conf.d/nagios.conf
elif [ -d "$APACHECONFDIR/../conf.d" ]; then
	# added with v0.7.2: SuSE uses conf.d/ but it is not defined in httpd.conf:
	NAGIOSCONF=${APACHECONFDIR}/../conf.d/nagios.conf
else
	# added with v0.7.1: for debian there is a mods-enabled directive:
	grep -i "^Include.*mods-enabled" ${APACHECONFDIR}/${APACHECONFFILE} >/dev/null 2>&1
	if [ $? -eq 0 ]; then
		NAGIOSCONF=${APACHECONFDIR}/mods-available/nagios.conf
		# changed with v0.8: test with -h instead of -f
		if [ ! -h ${APACHECONFDIR}/mods-enabled/nagios.conf ]; then
			ln -s $NAGIOSCONF ${APACHECONFDIR}/mods-enabled/nagios.conf
		fi
	else
		NAGIOSCONF=${APACHECONFDIR}/nagios.conf

		# Added with v0.7.2: does not include the same directive twice:
		ND="Include ${APACHECONFDIR}/nagios.conf"
		grep "$ND" ${APACHECONFDIR}/${APACHECONFFILE} >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			echo "Include ${APACHECONFDIR}/nagios.conf" >> ${APACHECONFDIR}/${APACHECONFFILE}
		fi
	fi
fi
#echo $NAGIOSCONF

if [ "$AUTHTYPE" = "basic" ]; then
	HTPASSWD="$DSTDIR/etc/htpasswd"
else
	HTPASSWD="$DSTDIR/etc/htdigest"
fi

read -p "Enter the name for the Web Administrator ? [nagios] " WEBUSER
if [ "$WEBUSER" = "" ]; then
	WEBUSER=nagios
fi
read -p "Enter the password for the Web Administrator ? [nagios] " WEBPASSWD
if [ "$WEBPASSWD" = "" ]; then
	WEBPASSWD=nagios
fi

# --------------------------
# Check for PHP presence:
# --------------------------

# added with v0.7.2: php name can be php5 !
DFLTPHPPATH=/usr/bin/php
for P in php php5; do
	PHPPATH=`type $P 2>/dev/null | cut -d " " -f3`
	if [ "$PHPPATH" != "" -a -x "$PHPPATH" ]; then
		echo "$P found under $PHPPATH"
		DFLTPHPPATH=$PHPPATH
	fi
done

while [ "$PHPPATH" = "" ]; do
	read -p "Enter the full pathname of php ($DFLTPHPPATH): " PHPPATH
	if [ "$PHPPATH" = "" ]; then
		PHPPATH=/usr/bin/php
	fi
	if [ ! -x "$PHPPATH" ]; then
		Info "php not found"
		PHPPATH=
	fi
done

# --------------------------
# Check for RRDTool presence:
# --------------------------
if [ $NEEDSRRDTOOL -ne 0 ]; then
	RRDPATH=`type rrdtool 2>/dev/null | cut -d" " -f3`
	if [ "$RRDPATH" = "" ]; then
		Info "RRDTool not found"
		AskYesNo "Do you want to install it ?"
		if [ "$REP" = "y" ]; then
			WITHRRDTOOL=1
			RRDPATH=$DSTDIR/rrdtool/bin/rrdtool
		else
			RRDPATH=
			while [ "$RRDPATH" = "" ]; do
				read -p "Enter the full pathname of rrdttol (/usr/local/bin/rrdtool): " RRDPATH
				if [ "$RRDPATH" = "" ]; then
					RRDPATH=/usr/local/bin/rrdtool
				fi
				if [ ! -x "$RRDPATH" ]; then
					Info "rrdtool command not found"
					RRDPATH=
				fi
			done
			WITHRRDTOOL=0
		fi
	fi
fi

# -----------------------
# Check for nmap presence:
# -----------------------
NMAPPATH=`type nmap 2>/dev/null | cut -d" " -f3`
if [ "$NMAPPATH" = "" ]; then
	Info "nmap not found"
	AskYesNo "Do you want to install it (required for Auto-populating) ?"
	if [ "$REP" = "y" ]; then
		WITHNMAP=1
		NMAPPATH=$DSTDIR/nmap/bin/nmap
	else
		NMAPPATH=
		while [ "$NMAPPATH" = "" ]; do
			read -p "Enter the full pathname of nmap (/usr/bin/nmap): " NMAPPATH
			if [ "$NMAPPATH" = "" ]; then
				NMAPPATH=/usr/bin/nmap
			fi
			if [ ! -x "$NMAPPATH" ]; then
				Info "nmap command not found"
				NMAPPATH=
			fi
		done
		WITHNMAP=0
	fi
fi

# -----------------------
# Check for sudo presence:
# -----------------------
SUDOPATH=`type sudo 2>/dev/null | cut -d" " -f3`
if [ "$SUDOPATH" = "" ]; then
	Info "I did not find 'sudo'. I need it to run 'nmap' and manage 'nagios' process..."
else
	# Where is the sudoers file ?
	SUDOFILE=/etc/sudoers
	if [ ! -w /etc/sudoers ]; then
		SUDOFILE=
		while [ "$SUDOFILE" = "" ]; do
			read -p "Enter the full filename for the 'sudoers' file: " SUDOFILE
			if [ "$SUDOFILE" != "" -a ! -w "$SUDOFILE" ]; then
				Info "'$SUDOFILE' file not found or not writeable !"
			fi
		done
	fi
fi

# -------------------------
# Check for libgd existence:
# -------------------------
# Added with v0.9: detects if gd is needed or not
if [ ! -f /usr/include/gd.h ]; then
	Info "gd.h not found"
	#AskYesNo "Do you want to install libgd ?"
	#if [ "$REP" = "y" ]; then
	#	WITHLIBGD="1"
	#fi
	Info "You must install the gd library and its header - this library also needs "
	Info "jpeg, png, freetype libraries"
	Info "You can proceed, but statusmap.cgi will not be compiled, or you can abort, "
	Info "install the missing libs and restart ./runall.sh"
	AskYesNo "Do you want to stop the process. If you answer 'n', statusmap.cgi will not compile "
	if [ "$REP" = "y" ]; then
		Error "Process aborted !"
	fi
	NOSTATUSMAP=--disable-statusmap
fi

# --------------------------
# Check for MRTG integration
# --------------------------
type mrtg >/dev/null 2>&1
if [ $? -eq 0 ]; then
	Info "MRTG found"
	AskYesNo "Do you want to interface nagiostats with MRTG "
	if [ "$REP" = "y" ]; then
		MRTGINTEGRATION=1
		if [ -e /etc/mrtg.cfg ]; then
			MRTGCONF="/etc/mrtg.cfg"
		elif [ -e /etc/mrtg/mrtg.cfg ]; then
			MRTGCONF="/etc/mrtg/mrtg.cfg"
		else
			Error "Could not find mrtg.cfg file"
			MRTGINTEGRATION=0
		fi
	fi
fi

# -------------------
# NSCA installation ?
# -------------------
AskYesNo "Do you want to install nsca ?"
if [ "$REP" = "y" ]; then
	WITHNSCA=1
	while [ "$NSCAPASSWD" = "" ]; do
		read -p "Enter the password for the nsca daemon (must not be empty): " NSCAPASSWD
	done
fi

# -------------------
# NRPE installation ?
# -------------------
AskYesNo "Do you want to install nrpe ?"
if [ "$REP" = "y" ]; then
	# added since v0.9: determine SSL headers and libs location (pb on Debian/Ubuntu - A.Fernandes)

	# libssl required:
	echo "Checking for SSL libraries..."
	LIBSSL=`find /lib /usr/lib /usr/local/lib -name libssl\* 2>/dev/null`
	if [ "$LIBSSL" = "" ]; then
		Error "no SSL library found (under /lib, /usr/lib, /usr/local/lib) - nrpe will probably not compile"
	fi
	WITHNRPE=1
fi

# ---------------------
# NagVis Installation ?
# ---------------------
AskYesNo "Do you want to install NagVis ?"
if [ "$REP" = "y" ]; then
	# Check requirements: MySQL, PHP5, NDO, Graphviz
	type dot >/dev/null 2>&1
	if [ $? -ne 0 ]; then
		Info "The 'dot' command has not been found. We must install the 'graphviz' package"
		WITHGRAPHVIZ=1
	fi
	WITHNAGVIZ=1
	WITHNDO=1
fi
if [ "$WITHNAGVIZ" = 1 ]; then
	Info "NDOUtils is required by NagVis and will be also installed"
fi

# ---------------------
# NDO Installation ?
# ---------------------
if [ $WITHNDO -eq 0 ]; then
	AskYesNo "Do you want to install NDO ?"
	if [ "$REP" = "y" ]; then
		# Check requirements: MySQL
		WITHNDO=1
		NEEDSMYSQL=1
	fi
fi

# -------------------------
# Nagtrap installation ?
# -------------------------
if [ $WITHNAGTRAP -eq 0 ]; then
	AskYesNo "Do you want to install Nagtrap ?"
	if [ "$REP" = "y" ]; then
		# snmptrapd must exist:
		type snmptrapd >/dev/null 2>&1
		if [ $? -ne 0 ]; then
			Info "Nagtrap needs snmptrapd which seems to be not installed"
		fi
		if [ ! -e /etc/snmp/snmptrapd.conf ]; then
			Info "/etc/snmp/snmptrapd.conf missing - you will have to update it by hand"
		fi
		# Check requirements: MySQL
		WITHNAGTRAP=1
		NEEDSMYSQL=1
	fi
fi

# -------------------------
# Blosxom installation ?
# -------------------------
if [ $WITHBLOSXOM -eq 0 ]; then
	AskYesNo "Do you want to install Blosxom ?"
	if [ "$REP" = "y" ]; then
		WITHBLOSXOM=1
	fi
fi

# -------------------------
# DNX-serveur installation ?
# -------------------------
if [ $WITHDNX -eq 0 ]; then
	AskYesNo "Do you want to install the server part of DNX ?"
	if [ "$REP" = "y" ]; then
		WITHDNX=1
	fi
fi

# -------------------------
# N2RRD installation ?
# -------------------------
if [ $WITHN2RRD -eq 0 ]; then
	AskYesNo "Do you want to install N2RRD to graph performance data ?"
	if [ "$REP" = "y" ]; then
		WITHN2RRD=1
	fi
fi

# --------------------------
# Check for MySQL components:
# --------------------------
# More tests added from v1.0.3
if [ $NEEDSMYSQL -eq 1 ]; then
	if pidof mysqld >/dev/null 2>&1; then
		Info "If you plan a local MySQL Server, please start it now"
	fi

	# added from v1.0.4: determine MySQL libraries location:
	MYSQLLIB=
	MYSQLINC=
	for DIR in /usr/lib64/mysql /usr/local/lib64/mysql /usr/lib/mysql /usr/local/lib/mysql; do
		if [ -e "$DIR/libmysqlclient.so" ]; then
			MYSQLLIB=${MYSQLLIB:=$DIR}
		fi
	done
	while [ ! -d "$MYSQLLIB" ]; do
		Info "The location of MySQL Client libraries (libmysqlclient.so) has not been found."
		read -p "Enter the Directory for MySQL Libraries : " MYSQLLIB
	done
	for DIR in /usr/include/mysql /usr/local/include/mysql; do
		if [ -e "$DIR/mysql.h" ]; then
			MYSQLINC=${MYSQLINC:=$DIR}
		fi
	done
	while [ ! -d "$MYSQLINC" ]; do
		Info "The location of MySQL Headers (mysql.h) has not been found."
		read -p "Enter the Directory for MySQL Headers: " MYSQLINC
	done

	# Ask for MySQL Connection parameters:
	DBSELECTED=0
	while [ $DBSELECTED -eq 0 ]; do
		DBSELECTED=1

		DBCNX=0
		while [ $DBCNX -eq 0 ]; do
			while [ "$DBSRV" = "" ]; do
				read -p "What is the name of the MySQL Server ? [localhost]: " DBSRV
				if [ "$DBSRV" = "" ]; then
					DBSRV="localhost"
				fi
			done

			while [ "$DBNAME" = "" ]; do
				read -p "What is the name of the MySQL Database ? [nagios]: " DBNAME
				if [ "$DBNAME" = "" ]; then
					DBNAME="nagios" 
				fi
			done

			while [ "$DBUSER" = "" ]; do
				read -p "What is the name for the MySQL User ? [nagios]: " DBUSER
				if [ "$DBUSER" = "" ]; then
					DBUSER="nagios" 
				fi
			done

			while [ "$DBPWD" = "" ]; do
				read -p "What is the password for MySQL User ? " DBPWD
				if [ "$DBPWD" = "" ]; then
					echo "Please enter a password - some extensions need one !"
				fi
			done

			read -p "What is the name of the MySQL Administrator Account (needed to create the database) ? " DBADMIN
			read -p "What is the password for the MySQL Administrative Account ? " DBADMINPWD

			# Make a connection try:
			if [ "$DBADMINPWD" = "" ]; then
				mysql -h $DBSRV -u $DBADMIN <<END
exit
END
			else
				mysql -h $DBSRV -u $DBADMIN -p$DBADMINPWD <<END
exit
END
			fi

			if [ $? -ne 0 ]; then
				Info "Could not connect to MySQL server..."
			else
				DBCNX=1
			fi
		done

		# Check the database does not exist:
		CREATEDB=1
		if [ "$DBADMINPWD" = "" ]; then
			mysqlshow -h $DBSRV -u $DBADMIN | grep $DBNAME >/dev/null
		else
			mysqlshow -h $DBSRV -u $DBADMIN -p$DBADMINPWD | grep $DBNAME >/dev/null
		fi

		if [ $? -eq 0 ]; then
			echo "It seems Database $DBNAME already exists."
			CREATEDB=0
			AskYesNo "Do you want to use it ? "
			if [ "$REP" != "y" ]; then
				Info "Please, select another Database Name"
				DBSELECTED=0
			fi
		fi

		# Create the database if needed:
		if [ $CREATEDB -eq 1 ]; then
			if [ "$DBADMINPWD" = "" ]; then
				mysql -h $DBSRV -u $DBADMIN <<END
create database $DBNAME;
grant INSERT,SELECT,UPDATE,DELETE ON $DBNAME.* TO $DBUSER@'localhost' IDENTIFIED BY '$DBPWD';
grant INSERT,SELECT,UPDATE,DELETE ON $DBNAME.* TO $DBUSER@'$HOSTNAME' IDENTIFIED BY '$DBPWD';
exit
END
			else
				mysql -h $DBSRV -u $DBADMIN -p$DBADMINPWD <<END
create database $DBNAME;
grant INSERT,SELECT,UPDATE,DELETE ON $DBNAME.* TO $DBUSER@'localhost' IDENTIFIED BY '$DBPWD';
grant INSERT,SELECT,UPDATE,DELETE ON $DBNAME.* TO $DBUSER@'$HOSTNAME' IDENTIFIED BY '$DBPWD';
exit
END
			fi
		fi

	done # end-while
fi

# ------------------------------------------------
# French documentation installation (from v0.10) ?
# ------------------------------------------------
# No: this doc is valid for Nagios v2 only
#AskYesNo "Do you want to install the French Documentation ?"
#WANTSFRENCHDOC=$REP
WANTSFRENCHDOC=n

# end of $INSTALL=y
fi

# --------------------------------------
# Write the parameters in params.sh file
# --------------------------------------
cat <<END >params.sh
OUTPUTFILE="$OUTPUTFILE"
TMPFILE="$TMPFILE"
TMPDIR="$TMPDIR"
DSTDIR="$DSTDIR"
NAGIOSVERS="$NAGIOSVERS"
PLUGINSVERS="$PLUGINSVERS"
EZNAGVERS="$EZNAGVERS"
NAGIOSUSR="$NAGIOSUSR"
NAGIOSGRP="$NAGIOSGRP"
CREATENAGIOSUSER="$CREATENAGIOSUSER"
CGIURL="$CGIURL"
HTMLURL="$HTMLURL"
APACHE_PNAME="$APACHE_PNAME"
APACHEUSR="$APACHEUSR"
APACHEGRP="$APACHEGRP"
APACHECONFDIR="$APACHECONFDIR"
APACHECONFFILE="$APACHECONFFILE"
NAGIOSCONF="$NAGIOSCONF"
USESAMPLEFILES="$USESAMPLEFILES"
HTPASSWD="$HTPASSWD"
WITHRRDTOOL="$WITHRRDTOOL"
RRDPATH="$RRDPATH"
RRDTOOLVERS="$RRDTOOLVERS"
WITHNMAP="$WITHNMAP"
NMAPPATH="$NMAPPATH"
NMAPVERS="$NMAPVERS"
NAGTRAPVERS="$NAGTRAPVERS"
SNMPTTVERS="$SNMPTTVERS"
ZLIBVERS="$ZLIBVERS"
LIBPNGVERS="$LIBPNGVERS"
FREETYPEVERS="$FREETYPEVERS"
LIBARTVERS="$LIBARTVERS"
LIBGDVERS="$LIBGDVERS"
BLOSXOMVERS="$BLOSXOMVERS"
DNXVERS="$DNXVERS"
N2RRDVERS="$N2RRDVERS"
WITHLIBGD="$WITHLIBGD"
WEBUSER="$WEBUSER"
WEBPASSWD="$WEBPASSWD"
NSCAVERS="$NSCAVERS"
NRPEVERS="$NRPEVERS"
NDOUTILSVERS="$NDOUTILSVERS"
NAGVISVERS="$NAGVISVERS"
GRAPHVIZVERS="$GRAPHVIZVERS"
WITHNSCA="$WITHNSCA"
MCRYPTVERS="$MCRYPTVERS"
NSCAPASSWD="$NSCAPASSWD"
NSCAENCODING=16
WITHNRPE="$WITHNRPE"
SUDOPATH="$SUDOPATH"
SUDOFILE="$SUDOFILE"
PHPPATH="$PHPPATH"
AUTHTYPE="$AUTHTYPE"
NOSTATUSMAP="$NOSTATUSMAP"
WANTSFRENCHDOC="$WANTSFRENCHDOC"
WITHNAGVIS="$WITHNAGVIZ"
WITHNDO="$WITHNDO"
WITHNAGTRAP="$WITHNAGTRAP"
WITHGRAPHVIZ="$WITHGRAPHVIZ"
WITHBLOSXOM="$WITHBLOSXOM"
WITHDNX="$WITHDNX"
WITHN2RRD="$WITHN2RRD"
MYSQLLIB="$MYSQLLIB"
MYSQLINC="$MYSQLINC"
UPDATE="$UPDATE"
INSTALL="$INSTALL"
NEEDSMYSQL="$NEEDSMYSQL"
DBNAME="$DBNAME"
DBUSER="$DBUSER"
DBPWD="$DBPWD"
DBSRV="$DBSRV"
DBADMIN="$DBADMIN"
DBADMINPWD="$DBADMINPWD"
CREATEDB="$CREATEDB"
MRTGINTEGRATION="$MRTGINTEGRATION"
MRTGCONF="$MRTGCONF"
END

# ----------------------------------------------------------------------------
Info "The Parameters have been stored in 'params.sh' file."
Info "If this script stops, you can run the 'install.sh' script now."

exit 0

# EOF
