#!/bin/bash
# ============================================================================
# 
# Installation script for RRDTool
#
# ----------------------------------------------------------------------------
#
# TODO:
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
if [ "$WITHRRDTOOL" = "0" ]; then
	Info "You did not wish to install RRDTool or it is already installed"
	exit 0
fi

echo "rrdtool Installation" | tee -a "$OUTPUTFILE"

tar xvz -C $TMPDIR -f distrib/rrdtool-$RRDTOOLVERS.tar.gz >>"$OUTPUTFILE" 2>&1
ln -s rrdtool-$RRDTOOLVERS "$TMPDIR/rrdtool"

# fixed with v0.4 (thanks to Mark Pierce)
#export BUILD_DIR=`pwd`/tmp/rrdtool
export BUILD_DIR=`pwd`/${TMPDIR}/rrdtool
export INSTALL_DIR="$DSTDIR/rrdtool"

cp -f distrib/zlib-$ZLIBVERS.tar.gz $BUILD_DIR
cp -f distrib/libpng-$LIBPNGVERS-config.tar.gz $BUILD_DIR
cp -f distrib/freetype-$FREETYPEVERS.tar.gz $BUILD_DIR
cp -f distrib/libart_lgpl-$LIBARTVERS.tar.gz $BUILD_DIR

# -------------------
# rrdtool Compilation:
# -------------------
pushd $BUILD_DIR >>"$OUTPUTFILE" 2>&1

# zlib:
echo "zlib Compilation..." | tee -a "$OUTPUTFILE"
tar xvfz zlib-$ZLIBVERS.tar.gz >>"$OUTPUTFILE" 2>&1
cd zlib-$ZLIBVERS
env CFLAGS="-O3 -fPIC" ./configure --prefix=$BUILD_DIR/lb >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1
make install >>"$OUTPUTFILE" 2>&1
cd ..

# libpng:
echo "libpng Compilation..." | tee -a "$OUTPUTFILE"
tar zxvf libpng-$LIBPNGVERS-config.tar.gz >>"$OUTPUTFILE" 2>&1
cd libpng-$LIBPNGVERS-config

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

env CPPFLAGS="-I$BUILD_DIR/lb/include" LDFLAGS="-L$BUILD_DIR/lb/lib" CFLAGS="-O3 -fPIC" \
	./configure --disable-shared --prefix=$BUILD_DIR/lb >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1
make install >>"$OUTPUTFILE" 2>&1
cd ..

# freetype:
echo "freetype Compilation..." | tee -a "$OUTPUTFILE"
tar zxvf freetype-$FREETYPEVERS.tar.gz >>"$OUTPUTFILE" 2>&1
cd freetype-$FREETYPEVERS

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

env CPPFLAGS="-I$BUILD_DIR/lb/include" LDFLAGS="-L$BUILD_DIR/lb/lib" CFLAGS="-O3 -fPIC" \
	./configure --disable-shared --prefix=$BUILD_DIR/lb >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1
make install >>"$OUTPUTFILE" 2>&1
cd ..

# libart_lgpl:
echo "libart Compilation..." | tee -a "$OUTPUTFILE"
tar zxvf libart_lgpl-$LIBARTVERS.tar.gz >>"$OUTPUTFILE" 2>&1
cd libart_lgpl-$LIBARTVERS

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

env CFLAGS="-O3 -fPIC" ./configure --disable-shared --prefix=$BUILD_DIR/lb >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1
make install >>"$OUTPUTFILE" 2>&1
cd ..

# rrdtool:
echo "rrdtool Compilation..." | tee -a "$OUTPUTFILE"
IR=-I$BUILD_DIR/lb/include
CPPFLAGS="$IR $IR/libart-2.0 $IR/freetype2 $IR/libpng"
LDFLAGS="-L$DSTDIR/rrdtool/lib"
CFLAGS=-O3
export CPPFLAGS LDFLAGS CFLAGS
if [ ! -d $DSTDIR/rrdtool/lib ]; then
	mkdir -p $DSTDIR/rrdtool/lib
fi
cp -rf lb/lib/* $DSTDIR/rrdtool/lib >>"$OUTPUTFILE" 2>&1

# added from v0.7.2: it seems ./configure failed on SuSE
# since it cannot create a tmp subdir at this step !?
mkdir tmp

./configure --prefix=$INSTALL_DIR --disable-python --disable-tcl >>"$OUTPUTFILE" 2>&1
make >>"$OUTPUTFILE" 2>&1
make install >>"$OUTPUTFILE" 2>&1

popd >>"$OUTPUTFILE" 2>&1

exit 0

# EOF
