#!/bin/bash
# ============================================================================
# 
# Installation and configuration of snmptt
#
# ----------------------------------------------------------------------------
#
# TODO: 
#
# ============================================================================
#
. ./utils.sh

# ----------------
# Global Variables:
# ----------------
. ./params.sh

CheckRoot

if [ "$UPDATE" = "y" ]; then
	exit 0
fi

if [ "$WITHNAGTRAP" = "0" ]; then
	Info "You did not wish to install Nagtrap: SNMPTT will not be installed!"
	exit 0
fi

# Requirements:
perl -MCPAN -e 'install SNMP' 

# ------------------------------------------------
# Extract the sources in the destination directory:
# ------------------------------------------------
echo "Extracting SNMPTT sources" | tee -a "$OUTPUTFILE"
tar xvz -C "$TMPDIR" -f distrib/snmptt_$SNMPTTVERS.tgz >> "$OUTPUTFILE" 2>&1
ln -s snmptt_$SNMPTTVERS "$TMPDIR/snmptt"

# ------------------------------------------------
# Copy files
# ------------------------------------------------
pushd tmp/snmptt >/dev/null
mkdir -p $DSTDIR/snmptt/{bin,sbin,etc}
mv snmpttconvert snmpttconvertmib snmptt-net-snmp-test $DSTDIR/snmptt/bin/
mv snmptt snmptthandler $DSTDIR/snmptt/sbin/
cp snmptt.ini $DSTDIR/snmptt/etc/
mkdir $DSTDIR/snmptt/etc/snmptt.d
cp examples/snmptt.conf.generic $DSTDIR/snmptt/etc/snmptt.d/
popd >/dev/null

# -----------------------------------------------
# Convert the MIBS
# -----------------------------------------------
for F in /usr/share/snmp/mibs/*; do
	B=`basename $F .txt`
	$DSTDIR/snmptt/bin/snmpttconvertmib --in=$F --out=$DSTDIR/snmptt/etc/snmptt.d/$B.conf >> "$OUTPUTFILE" 2>&1
done

# -----------------------------------------------
# Finalize the configuration:
# -----------------------------------------------
sed -i -e 's/^net_snmp_perl_enable.*=.*/net_snmp_perl_enable = 1/' \
       -e 's/^mysql_dbi_enable.*/mysql_dbi_enable = 1/' \
       -e "s/^mysql_dbi_database.*/mysql_dbi_database = $DBNAME/" \
       -e "s/^mysql_dbi_username.*/mysql_dbi_username = $DBUSER/" \
       -e "s/^mysql_dbi_password.*/mysql_dbi_password = $DBPWD/" \
       -e "s/^mysql_dbi_host.*/mysql_dbi_host = $DBSRV/" \
       -e 's/^unknown_trap_log_enable.*/unknown_trap_log_enable = 1/' \
	$DSTDIR/snmptt/etc/snmptt.ini

sed -i -e '/snmptt_conf_files.*END/,/END/d' \
	$DSTDIR/snmptt/etc/snmptt.ini

echo 'snmptt_conf_files = <<END' >> $DSTDIR/snmptt/etc/snmptt.ini
for F in $DSTDIR/snmptt/etc/snmptt.d/*; do
	echo $F >> $DSTDIR/snmptt/etc/snmptt.ini
done
echo 'END' >> $DSTDIR/snmptt/etc/snmptt.ini

# -----------------------------------------------
# Modify snmptrapd.conf
# -----------------------------------------------
SNMPTRAPDCONF=/etc/snmp/snmptrapd.conf
if [ -f "$SNMPTRAPDCONF" ]; then
	sed -i -e 's/^traphandle/#traphandle/i' \
	       -e 's/^authcommunity/#authcommunity/i' \
	       -e 's/^outputoption/#outputoption/i' \
	       $SNMPTRAPDCONF
	echo "traphandle default $DSTDIR/snmptt/sbin/snmptt --ini=$DSTDIR/snmptt/etc/snmptt.ini" >> $SNMPTRAPDCONF
	echo "authCommunity log,execute publicnagios" >> $SNMPTRAPDCONF
	echo "outputOption n" >> $SNMPTRAPDCONF
else
	echo "File $SNMPTRAPDCONF not found, you must add the following directives by hand:"
	echo "traphandle default $DSTDIR/snmptt/sbin/snmptt --ini=$DSTDIR/snmptt/etc/snmptt.ini"
	echo "authCommunity log,execute publicnagios"
	echo "outputOption n"
fi

# ------------------------------------
# Copy new nagios configuration file:
# ------------------------------------
cp distrib/conf/snmptt-generic-srv.cfg $DSTDIR/etc/objects/
chown $APACHEUSR:$NAGIOSGRP $DSTDIR/etc/objects/snmptt*
chmod 640 $DSTDIR/etc/objects/snmptt*

exit 0

# EOF
