#!/usr/bin/perl -w
#$Revision: 1.6 $
#$Author: trig_monkeypr0n $
# License: GPL
# Copyright (c) 2003 Jason Burnett (code@monkeypr0n.org)
# Nagios is a registered trademark of Ethan Galstad

use strict;
use Authen::SimplePam;
use POSIX qw( strftime );
use Carp;
use POE qw( Component::Server::TCP Component::Client::TCP 
			Wheel::Run Wheel::SocketFactory Wheel::ReadWrite );
use Sys::Syslog;
use Config::IniFiles;
use Getopt::Long;
use vars qw( $user $pass $pamh $res );
Getopt::Long::Configure('bundling');
GetOptions( qw(c=s) );
$| = 1;


if( $opt_c ){
	tie %cfg, 'Config::IniFiles', (
		-file => $opt_c,
		-reloadwarn => 1,
		-nocase => 1,
	);
}else{
	tie %cfg, 'Config::IniFiles', (
		-file => "popup-server.cfg",
		-reloadwarn => 1,
		-nocase => 1,
	);

}	

my $sock = $cfg{server}{sock};
my $pidfile = $cfg{server}{pidfile};
my $port = $cfg{server}{port};

my $VERSION = (qw($Revision: 1.6 $))[1];
my $service = "ssh";
my $auth = new Authen::SimplePam;

fork && exit 0;

#Signal handling
$SIG{'QUIT'} 	= 'CLEANUP';
$SIG{'TERM'} 	= 'CLEANUP';
$SIG{'INT'} 	= 'CLEANUP';
$SIG{'HUP'} 	= 'RESTART';

sub CLEANUP {
	warn "Caught Interrupt (^C), Aborting\n";
	unlink( $sock );
	exit(1);
}

sub RESTART {
	warn "\n\ngot HUP'd\n\n";
	exec( $0 ) or croak "HUP: unable to restart: $!\n";
}

#Logging method
sub sys_log {
	my $r_ip = shift;
	my $time = strftime "%Y-%m-%d %H:%M:%S ", localtime;
	syslog( 'info', "nagios: $time -- $r_ip connected\n" );
}

POE::Component::Server::TCP->new
  ( Alias => "alert_server",
    Port         => $port,
    # Handle client requests here.
    ClientInput => sub {
		my( $kernel, $heap, $input ) = @_[ KERNEL, HEAP, ARG0 ];
		if( $input eq 'quit' || $input eq 'exit' || $input eq 'done' ) {
			$heap->{client}->put( "Thanks for coming." );
			delete $heap->{client};
			delete $heap->{state};
			#$kernel->yield("shutdown");
			return;
		}
		#ignore states > 2
		if( ! $heap->{state} ) {
			$kernel->yield("shutdown");
		}elsif( $heap->{state} == 1 ) {
				($user, $pass) = split (/:/, $_[ARG0]);
				$heap->{user} = $user;
				$auth->auth_user( $user, $pass, $service );
				my $res = $auth->pam_result();
carp "res::$res\n";
				if( $res > 0 ){
					delete $heap->{client};
					delete $heap->{state};
					$kernel->yield("shutdown");
				}else{
					my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
					my $user = $heap->{user};
					my $session_id = $_[SESSION]->ID;
					$heap->{client}->put( "Welcome $heap->{user}" );
					$kernel->post( watch_admins=>new_watcher=>$heap->{client} );
					delete $heap->{state};
					warn "$heap->{remote_ip} has authed for user: $heap->{user}\n";
				}
			}
		},
	ClientConnected	=>	sub {
		my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
		my $session_id = $_[SESSION]->ID;
		$heap->{state} = 1;
		#$heap->{client}->put( "Connection Accepted." );
		warn "Session started with id $session_id from $heap->{remote_ip}\n";
	},
    ClientDisconnected => sub {
		my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
		my $user = $heap->{user};
		my $session_id = $_[SESSION]->ID;
		delete $heap->{state};
    },
	ClientShutdownOnError => 1,                       # Optional.

  );
POE::Session->create(
	inline_states	=>
		{
			_start	=> sub {
				my( $kernel, $heap ) = @_[ KERNEL, HEAP ];
				$kernel->alias_set( 'watch_admins' );
				$heap->{start_time} = time();
				},
			new_watcher	=> sub {
				my( $kernel, $heap, $client ) = @_[ KERNEL, HEAP, ARG0 ];
				push @{$heap->{watchers}}, $client;
			},
			new_alert	=> sub {
				my( $kernel, $heap, $alert ) = @_[ KERNEL, HEAP, ARG0 ];
				foreach( @{$heap->{watchers}} ) {
					$_->put( "alert: $alert" );
				}
					warn "new_alert: $alert\n";
			},
			_stop	=> sub {
				my $elapsed_runtime = time() - $_[HEAP]->{start_time};
				warn 'Session ', $_[SESSION]->ID, " elapsed runtime: $elapsed_runtime\n";
				carp "_stop: Stopped $!\n";
				},
		}
);

open( FILE, ">$pidfile" ) or croak "$pidfile: $!\n";
print FILE "$$\n";
close FILE;

Server::spawn( $sock );
$poe_kernel->run();
exit 0;

package Server;
use POE::Session;    # For KERNEL, HEAP, etc.
use Socket;          # For PF_UNIX.
use Carp;

sub spawn {
	my $rendezvous = shift;

	POE::Session->create
		( inline_states =>
			{
				hi_dammit => sub {
    							my $client_socket = $_[ARG0];
    							ServerSession::spawn($client_socket);
							},
				damn_dammit => sub {
								my($heap, $syscall, $errno, $error)=
									@_[ HEAP, ARG0 .. ARG2 ];
								$error = "Normal disconnection." unless $errno;
								carp "1 - Server socket encountered 
									$syscall error $errno: $error\n";
								delete $heap->{server};
							},
				_start => sub {
					my ( $kernel, $heap ) = @_[ KERNEL, HEAP ];
					$heap->{rendezvous} = $rendezvous;
					unlink $heap->{rendezvous} if -e $heap->{rendezvous};
					$heap->{server} = POE::Wheel::SocketFactory->new
						(	SocketDomain => PF_UNIX,
							SocketType	 => SOCK_STREAM,
							BindAddress  => $heap->{rendezvous},
							SuccessEvent => 'hi_dammit',
							FailureEvent => 'damn_dammit',
						) or croak "socketfactory->new() failed: $!\n";
				},
			},
		);
}

package ServerSession;
use POE::Session;    # For KERNEL, HEAP, etc.
use Carp;

sub spawn {
    my $socket = shift;
    POE::Session->create
      ( inline_states =>
          { _start				=> \&server_session_start,
            got_client_input	=> \&server_session_input,
            got_client_error	=> \&server_session_error,
          },
        args => [$socket],
      );
}

sub server_session_start {
    my ( $heap, $socket ) = @_[ HEAP, ARG0 ];
    $heap->{client} = POE::Wheel::ReadWrite->new
      ( Handle => $socket,
        InputEvent => 'got_client_input',
        ErrorEvent => 'got_client_error',
      );
    $heap->{client}->put( "Neospire Socket Server." );
}

sub server_session_input {
    my ( $kernel, $heap, $input ) = @_[ KERNEL, HEAP, ARG0 ];
    $heap->{client}->put(" sent $input to server" );
	$kernel->post( watch_admins=>new_alert=>$input );
	#carp "got input from unix socket: $input\n";
}

sub server_session_error {
    my ( $heap, $syscall, $errno, $error ) = @_[ HEAP, ARG0 .. ARG2 ];
    #carp"Server session encountered $syscall error $errno: $error\n" if $errno;
    delete $heap->{client};
}


